@@ -1,19 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/bgp/bgp_varrw.cc,v 1.35 2008/07/23 05:09:31 pavlin Exp $"
+#ident "$XORP: xorp/bgp/bgp_varrw.cc,v 1.40 2009/01/05 18:30:43 jtc Exp $"
 
 #include "bgp_module.h"
 #include "libxorp/xorp.h"
@@ -32,11 +37,11 @@ BGPVarRWCallbacks<A> BGPVarRW<A>::_callbacks;
 
 template <class A>
 BGPVarRW<A>::BGPVarRW(const string& name) :
-    _name(name), _orig_rtmsg(NULL), _filtered_rtmsg(NULL), 
-      _got_fmsg(false), _ptags(NULL), _wrote_ptags(false), 
-      _palist(NULL),
-      _no_modify(false),
-      _modified(false), _route_modify(false)
+    _name(name), _rtmsg(NULL), 
+    _got_fmsg(false), _ptags(NULL), _wrote_ptags(false), 
+    _palist(NULL),
+    _no_modify(false),
+    _modified(false), _route_modify(false)
 {
     XLOG_ASSERT( unsigned(VAR_BGPMAX) <= VAR_MAX);
 
@@ -54,17 +59,6 @@ template <class A>
 void
 BGPVarRW<A>::cleanup()
 {
-    // nobody ever obtained the filtered message but we actually created one. We
-    // must delete it as no one else can, at this point.
-    if (!_got_fmsg && _filtered_rtmsg) {
-        if (_filtered_rtmsg->changed())
-            _filtered_rtmsg->route()->unref();
-
-        delete _filtered_rtmsg;
-    }
-
-    if (_palist)
-        delete _palist;
 
     if (_ptags)
         delete _ptags;
@@ -72,16 +66,15 @@ BGPVarRW<A>::cleanup()
 
 template <class A>
 void
-BGPVarRW<A>::attach_route(const InternalMessage<A>& rtmsg, bool no_modify)
+BGPVarRW<A>::attach_route(InternalMessage<A>& rtmsg, bool no_modify)
 {
     cleanup();
 
-    _orig_rtmsg = &rtmsg;
-    _filtered_rtmsg = NULL;
+    _rtmsg = &rtmsg;
     _got_fmsg = false;
     _ptags = NULL;
     _wrote_ptags = false;
-    _palist = NULL;
+    _palist = rtmsg.attributes();
     _no_modify = no_modify;
     _modified = false;
     _route_modify = false;
@@ -96,39 +89,55 @@ BGPVarRW<A>::attach_route(const InternalMessage<A>& rtmsg, bool no_modify)
     }
 }
 
+template <class A>
+void
+BGPVarRW<A>::detach_route(InternalMessage<A>& rtmsg)
+{
+    UNUSED(rtmsg);
+    _palist = 0;
+}
+
+
 template <class A>
 Element*
 BGPVarRW<A>::read_policytags()
 {
-    return _orig_rtmsg->route()->policytags().element();
+    return _rtmsg->route()->policytags().element();
+}
+
+template <class A>
+Element*
+BGPVarRW<A>::read_tag()
+{
+    return _rtmsg->route()->policytags().element_tag();
 }
 
 template <class A>
 Element*
 BGPVarRW<A>::read_filter_im()
 {
-    return new ElemFilter(_orig_rtmsg->route()->policyfilter(0));
+    return new ElemFilter(_rtmsg->route()->policyfilter(0));
 }
 
 template <class A>
 Element*
 BGPVarRW<A>::read_filter_sm()
 {
-    return new ElemFilter(_orig_rtmsg->route()->policyfilter(1));
+    return new ElemFilter(_rtmsg->route()->policyfilter(1));
 }
 
 template <class A>
 Element*
 BGPVarRW<A>::read_filter_ex()
 {
-    return new ElemFilter(_orig_rtmsg->route()->policyfilter(2));
+    return new ElemFilter(_rtmsg->route()->policyfilter(2));
 }
 
 template <>
 Element*
 BGPVarRW<IPv4>::read_network4()
 {
-    return new ElemIPv4Net(_orig_rtmsg->route()->net());
+    return new ElemIPv4Net(_rtmsg->route()->net());
 }
 
 template <>
@@ -143,7 +152,7 @@ Element*
 BGPVarRW<IPv6>::read_network6()
 {
     return _ef.create(ElemIPv6Net::id, 
-		      _orig_rtmsg->route()->net().str().c_str());
+		      _rtmsg->route()->net().str().c_str());
 }
 
 template <>
@@ -157,8 +166,8 @@ template <>
 Element*
 BGPVarRW<IPv6>::read_nexthop6()
 {
-    return _ef.create(ElemIPv6::id, 
-		      _orig_rtmsg->route()->nexthop().str().c_str());
+    return _ef.create(ElemIPv6NextHop::id, 
+		      _palist->nexthop().str().c_str());
 }
 
 template <>
@@ -172,7 +181,7 @@ template <>
 Element*
 BGPVarRW<IPv4>::read_nexthop4()
 {
-    return new ElemIPv4(_orig_rtmsg->route()->nexthop());
+    return new ElemIPv4NextHop(_palist->nexthop());
 }
 
 template <>
@@ -186,14 +195,14 @@ template <class A>
 Element*
 BGPVarRW<A>::read_aspath()
 {
-    return new ElemASPath(_orig_rtmsg->route()->attributes()->aspath());
+    return new ElemASPath(_palist->aspath());
 }
 
 template <class A>
 Element*
 BGPVarRW<A>::read_origin()
 {
-    uint32_t origin = _orig_rtmsg->route()->attributes()->origin();
+    uint32_t origin = _palist->origin();
     return _ef.create(ElemU32::id, to_str(origin).c_str());
 }
 
@@ -201,11 +210,10 @@ template <class A>
 Element*
 BGPVarRW<A>::read_localpref()
 {
-    const LocalPrefAttribute* lpref = _orig_rtmsg->route()->attributes()->
-					local_pref_att();
-    if (lpref)
+    const LocalPrefAttribute* lpref = _palist->local_pref_att(); 
+    if (lpref) {
 	return _ef.create(ElemU32::id, to_str(lpref->localpref()).c_str());
-    else
+    } else
 	return NULL;
 }
 
@@ -213,8 +221,7 @@ template <class A>
 Element*
 BGPVarRW<A>::read_community()
 {
-    const CommunityAttribute* ca = _orig_rtmsg->route()->attributes()->
-				    community_att();
+    const CommunityAttribute* ca = _palist->community_att();
 
     // no community!
     if (!ca)
@@ -233,7 +240,7 @@ template <class A>
 Element*
 BGPVarRW<A>::read_med()
 {
-    const MEDAttribute* med = _orig_rtmsg->route()->attributes()->med_att();
+    const MEDAttribute* med = _palist->med_att();
     if (med)
 	return new ElemU32(med->med());
     else
@@ -244,7 +251,7 @@ template <class A>
 Element*
 BGPVarRW<A>::read_med_remove()
 {
-    const MEDAttribute* med = _orig_rtmsg->route()->attributes()->med_att();
+    const MEDAttribute* med = _palist->med_att();
     if (med)
 	return new ElemBool(false);	// XXX: default is don't remove the MED
     else
@@ -286,13 +293,6 @@ BGPVarRW<A>::single_read(const Id& id)
     return (this->*cb)();
 }
 
-template <class A>
-void
-BGPVarRW<A>::clone_palist()
-{
-    _palist = new PathAttributeList<A>(*_orig_rtmsg->route()->attributes());
-}
-
 template <class A>
 void
 BGPVarRW<A>::write_community(const Element& e)
@@ -303,9 +303,6 @@ BGPVarRW<A>::write_community(const Element& e)
 
     const ElemSetCom32& es = dynamic_cast<const ElemSetCom32&>(e);
 
-    if (!_palist)
-	clone_palist();
-
     if (_palist->community_att())
 	_palist->remove_attribute_by_type(COMMUNITY);
 	
@@ -324,7 +321,7 @@ Element*
 BGPVarRW<A>::read_neighbor()
 {
     Element* e = NULL;
-    const PeerHandler* ph = _orig_rtmsg->origin_peer();
+    const PeerHandler* ph = _rtmsg->origin_peer();
     if (ph != NULL && !ph->originate_route_handler()) {
 	e = _ef.create(ElemIPv4::id, ph->get_peer_addr().c_str());
     }
@@ -335,9 +332,9 @@ template <class A>
 InternalMessage<A>*
 BGPVarRW<A>::filtered_message()
 {
-    XLOG_ASSERT(_modified && _filtered_rtmsg);
+    XLOG_ASSERT(_modified && _rtmsg);
     _got_fmsg = true;
-    return _filtered_rtmsg;
+    return _rtmsg;
 }
 
 template <class A>
@@ -371,9 +368,10 @@ template <class A>
 void
 BGPVarRW<A>::write_policytags(const Element& e)
 {
-    XLOG_ASSERT(!_ptags);
+    if (!_ptags)
+	_ptags = new PolicyTags(_rtmsg->route()->policytags());
 
-    _ptags = new PolicyTags(e);
+    _ptags->set_ptags(e);
     _wrote_ptags = true;
     
     // XXX: maybe we should make policytags be like filter pointers... i.e. meta
@@ -381,18 +379,59 @@ BGPVarRW<A>::write_policytags(const Element& e)
     _route_modify = true;
 }
 
+template <class A>
+void
+BGPVarRW<A>::write_tag(const Element& e)
+{
+    if (!_ptags)
+	_ptags = new PolicyTags(_rtmsg->route()->policytags());
+
+    _ptags->set_tag(e);
+    _wrote_ptags = true;
+
+    _route_modify = true;
+}
+
 template <>
 void
 BGPVarRW<IPv4>::write_nexthop4(const Element& e)
+{
+    write_nexthop(e);
+}
+
+template <class A>
+void
+BGPVarRW<A>::write_nexthop(const Element& e)
 {
     _route_modify = true;
-    const ElemIPv4* eip = dynamic_cast<const ElemIPv4*>(&e);
+
+    const ElemNextHop<A>* eip = dynamic_cast<const ElemNextHop<A>*>(&e);
     XLOG_ASSERT(eip != NULL);
 
-    IPv4 nh(eip->val());
+    A nh;
+
+    switch (eip->var()) {
+    case ElemNextHop<A>::VAR_NONE:
+	nh = eip->addr();
+	break;
+
+    case ElemNextHop<A>::VAR_SELF:
+	XLOG_ASSERT(_self != nh);
+	nh = _self;
+	break;
+
+    case ElemNextHop<A>::VAR_PEER_ADDRESS:
+	XLOG_ASSERT(_peer != nh);
+	nh = _peer;
+	break;
+
+    case ElemNextHop<A>::VAR_DISCARD:
+    case ElemNextHop<A>::VAR_REJECT:
+    case ElemNextHop<A>::VAR_NEXT_TABLE:
+	XLOG_ASSERT(!"not implemented");
+	break;
+    }
 
-    if (!_palist)
-	clone_palist();
     _palist->replace_nexthop(nh);
 }
 
@@ -406,15 +445,7 @@ template <>
 void
 BGPVarRW<IPv6>::write_nexthop6(const Element& e)
 {
-    _route_modify = true;
-    const ElemIPv6* eip = dynamic_cast<const ElemIPv6*>(&e);
-    XLOG_ASSERT(eip != NULL);
-
-    IPv6 nh(eip->val());
-
-    if (!_palist)
-	clone_palist();
-    _palist->replace_nexthop(nh);
+    write_nexthop(e);
 }
 
 template <>
@@ -431,8 +462,6 @@ BGPVarRW<A>::write_aspath(const Element& e)
 
     const ElemASPath& aspath = dynamic_cast<const ElemASPath&>(e);
 
-    if (!_palist)
-	clone_palist();
     _palist->replace_AS_path(aspath.val());
 }
 
@@ -442,8 +471,6 @@ BGPVarRW<A>::write_med(const Element& e)
 {
     _route_modify = true;
 
-    if (!_palist)
-	clone_palist();
     if (_palist->med_att())
 	_palist->remove_attribute_by_type(MED);
 	
@@ -461,8 +488,6 @@ BGPVarRW<A>::write_med_remove(const Element& e)
     if (! med_remove.val())
 	return;			// Don't remove the MED
 
-    if (!_palist)
-	clone_palist();
     if (_palist->med_att())
 	_palist->remove_attribute_by_type(MED);
 
@@ -514,8 +539,6 @@ BGPVarRW<A>::write_localpref(const Element& e)
 {
     _route_modify = true;
 
-    if (!_palist)
-	clone_palist();
     if (_palist->local_pref_att())
 	_palist->remove_attribute_by_type(LOCAL_PREF);
 	
@@ -538,8 +561,6 @@ BGPVarRW<A>::write_origin(const Element& e)
 	
     origin = static_cast<OriginType>(u32.val());
     
-    if (!_palist)
-	clone_palist();
     _palist->replace_origin(origin);
 }
 
@@ -564,34 +585,30 @@ BGPVarRW<A>::end_write()
 
     // OK.  The real problem is that you can't clone a subnet route, and assign
     // path attributes to it!
-
-    if (_no_modify)
+    if (_no_modify) {
 	return;
+    }
+
+    // we used to duplicate the route because we needed to create a
+    // new path attribute list.  We don't do this anymore; we just
+    // modify the FastPathAttributeList in situe.
+
+    const SubnetRoute<A>* route = _rtmsg->route();
 
     // only meta routing stuff changed [i.e. policy filter pointers... so we can
     // get away without creating a new subnet route, etc]
     if (!_route_modify) {
 	for (int i = 0; i < 3; i++) {
 	    if (_wrote_pfilter[i])
-		_orig_rtmsg->route()->set_policyfilter(i, _pfilter[i]);
+		route->set_policyfilter(i, _pfilter[i]);
 	}
 	return;
     }
 
-    if (_palist)
-        _palist->rehash();
 
-    const SubnetRoute<A>* old_route = _orig_rtmsg->route();
-    SubnetRoute<A>* new_route = new SubnetRoute<A>(_orig_rtmsg->net(),
-						   _palist ? _palist :
-						   old_route->attributes(),
-						   old_route->original_route(),
-						   old_route->igp_metric());
     // propagate policy tags
     if (_wrote_ptags)
-	new_route->set_policytags(*_ptags);
-    else
-	new_route->set_policytags(old_route->policytags());
+	route->set_policytags(*_ptags);
 
     for (int i = 0; i < 3; i++) {
 	if (_wrote_pfilter[i]) {
@@ -600,36 +617,16 @@ BGPVarRW<A>::end_write()
 	    // If we just store it in the copy we send down stream, next time we
 	    // get a push, we will get the wrong filter pointer [it will always
 	    // be the original one which was in ribin,.. as we never update it].
-	    old_route->set_policyfilter(i, _pfilter[i]);
-	    new_route->set_policyfilter(i, _pfilter[i]);
+	   route->set_policyfilter(i, _pfilter[i]);
 	}
-	else
-	    new_route->set_policyfilter(i, old_route->policyfilter(i));
     }
 
-    // propagate route flags
-    new_route->set_filtered(old_route->is_filtered());
-    if (old_route->is_winner())
-	new_route->set_is_winner(old_route->igp_metric());
-    new_route->set_in_use(old_route->in_use());
-    new_route->set_nexthop_resolved(old_route->nexthop_resolved());
-    new_route->set_aggr_prefix_len(_aggr_prefix_len);
+    _rtmsg->set_changed();
+
     if (_aggr_brief_mode)
-	new_route->set_aggr_brief_mode();
+	route->set_aggr_brief_mode();
     else
-	new_route->clear_aggr_brief_mode();
-
-    _filtered_rtmsg = new InternalMessage<A>(new_route,
-					     _orig_rtmsg->origin_peer(),
-					     _orig_rtmsg->genid());
-
-    _filtered_rtmsg->set_changed();
-   
-    // propagate internal message flags
-    if (_orig_rtmsg->push())
-	_filtered_rtmsg->set_push();
-    if (_orig_rtmsg->from_previous_peering())
-	_filtered_rtmsg->set_from_previous_peering();
+	route->clear_aggr_brief_mode();
 
     _modified = true;
 }
@@ -648,26 +645,40 @@ BGPVarRW<A>::more_tracelog()
     string x = "BGP " + _name + " route: ";
     uint32_t level = trace();
 
-    const InternalMessage<A>& msg = ( modified() ? 
-				     (*_filtered_rtmsg) :
-				     *_orig_rtmsg);
     
     if (level > 0)
-	x += msg.net().str();
+	x += _rtmsg->net().str();
     if (level > 1) {
 	x += " Full route: ";
-	x += msg.str();
+	x += _rtmsg->str();
     }
 
     return x;
 }
 
+template <class A>
+void
+BGPVarRW<A>::set_peer(const A& peer)
+{
+    _peer = peer;
+}
+
+template <class A>
+void
+BGPVarRW<A>::set_self(const A& self)
+{
+    _self = self;
+}
+
 template <class A>
 BGPVarRWCallbacks<A>::BGPVarRWCallbacks()
 {
     init_rw(VarRW::VAR_POLICYTAGS, 
 	    &BGPVarRW<A>::read_policytags, &BGPVarRW<A>::write_policytags);
     
+    init_rw(VarRW::VAR_TAG,
+	    &BGPVarRW<A>::read_tag, &BGPVarRW<A>::write_tag);
+
     init_rw(VarRW::VAR_FILTER_IM, 
 	    &BGPVarRW<A>::read_filter_im, &BGPVarRW<A>::write_filter_im);
     
