@@ -1,5 +1,5 @@
 #
-# $XORP: xorp/bgp/harness/args.sh,v 1.3 2003/10/23 00:13:47 pavlin Exp $
+# $XORP: xorp/bgp/harness/args.sh,v 1.4 2006/04/04 12:07:44 bms Exp $
 #
 
 #
@@ -12,6 +12,20 @@ QUIET=""
 VERBOSE=""
 RESTART="yes"
 
+# Perform Win32 path conversion for runit if required.
+RUNIT="runit"
+RUNITDIR="../../utils"
+RUNITPRE=""
+if [ x"$OSTYPE" = xmsys ]; then
+    RUNITPRE="cmd //c"
+    RUNITDIR=$(cd ${RUNITDIR} && pwd -W)
+fi
+
+runit()
+{
+    ${RUNITPRE} ${RUNITDIR}/${RUNIT} "$@"
+}
+
 # -q (Quiet)
 # -v (Verbose)
 # -s (Single) Do not start ancillary programs. They must already be running.
