@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# $XORP: xorp/bgp/harness/lookup.py,v 1.3 2007/12/10 23:26:32 mjh Exp $
+# $XORP: xorp/bgp/harness/lookup.py,v 1.4 2008/11/08 06:14:44 mjh Exp $
 
 # On standard input take a list of update packets in XORP text format
 # On standard output generate a list of lookup xrls that can be passed to
@@ -9,7 +9,7 @@
 import sys
 import getopt
 
-AS_PREFIX_STRING = ' - AS Path Attribute ASPath: '
+AS_PREFIX_STRING = '\tAS Path Attribute ASPath: '
 
 def coord(command):
     """
