@@ -1,18 +1,23 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/bgp/harness/peer.cc,v 1.86 2008/07/23 05:09:43 pavlin Exp $"
+#ident "$XORP: xorp/bgp/harness/peer.cc,v 1.91 2009/01/05 18:30:45 jtc Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -71,7 +76,11 @@ Peer::Peer(EventLoop    *eventloop,
     Iptuple iptuple;
     _localdata = new LocalData(*eventloop);
     _localdata->set_use_4byte_asnums(false);
+    _localdata->set_as(AsNum(0));
     _peerdata = new BGPPeerData(*_localdata, iptuple, AsNum(0), IPv4(),0);
+    _peerdata->compute_peer_type();
+    _peerdata->set_multiprotocol<IPv6>(SAFI_UNICAST, BGPPeerData::NEGOTIATED);
+    _peerdata->set_multiprotocol<IPv6>(SAFI_UNICAST, BGPPeerData::SENT); 
 }
 
 Peer::Peer(const Peer& rhs)
@@ -493,8 +502,8 @@ Peer::send_packet(const string& line, const vector<string>& words)
     uint8_t buf[BGPPacket::MAXPACKETSIZE];
 
     XLOG_ASSERT(pkt->encode(buf, len, _peerdata));
+    printf("packet: %s\n", pkt->str().c_str());
     delete pkt;
-
     if (!_corrupt.empty()) {
 	list<Corrupt>::const_iterator i;
 	for (i = _corrupt.begin(); i != _corrupt.end(); i++) {
@@ -857,16 +866,13 @@ Peer::trie(const string& line, const vector<string>& words)
 	    /*
 	    ** Search for the AS Path in the update packet.
 	    */
-	    const list<PathAttribute*>& palist = bgpupdate->pa_list();
+	    FPAList4Ref palist = const_cast<UpdatePacket*>(bgpupdate)->pa_list();
 
 	    list<PathAttribute*>::const_iterator pai;
 	    const ASPath *aspath = 0;
-	    for(pai = palist.begin(); pai != palist.end(); pai++) {
-		if(AS_PATH == (*pai)->type())
-		    aspath = &((static_cast<ASPathAttribute *>(*pai))->
-			       as_path());
-	    }
-	    if(0 == aspath)
+	    if (palist->aspath_att())
+		aspath = &(palist->aspath());
+	    else
 		xorp_throw(InvalidString, 
 			   c_format("NO AS Path associated with route\n[%s]",
 				    line.c_str())); 
@@ -1018,8 +1024,9 @@ mrtd_routview_dump(const  UpdatePacket* p, const IPNet<A>& net,
     /*
     ** Figure out the total length of the attributes.
     */
+    size_t pa_len = BGPPacket::MAXPACKETSIZE;
     uint16_t length = 0;
-    
+#if 0    
     list <PathAttribute*>::const_iterator pai;
     for (pai = p->pa_list().begin(); pai != p->pa_list().end(); pai++) {
 	const PathAttribute* pa;
@@ -1029,6 +1036,10 @@ mrtd_routview_dump(const  UpdatePacket* p, const IPNet<A>& net,
 	XLOG_ASSERT(pa->encode(buf, pa_len, peerdata));
 	length += pa_len;
     }
+#endif
+    uint8_t buf[BGPPacket::MAXPACKETSIZE];
+    p->encode(buf, pa_len, peerdata);
+    length = pa_len;
 
     /*
     ** Due to alignment problems I can't use a structure overlay.
@@ -1094,6 +1105,7 @@ mrtd_routview_dump(const  UpdatePacket* p, const IPNet<A>& net,
     if(fwrite(&viewbuf[0], sizeof(viewbuf), 1, fp) != 1)
 	XLOG_FATAL("fwrite of %s failed: %s", fname.c_str(), strerror(errno));
 
+#if 0
     for (pai = p->pa_list().begin(); pai != p->pa_list().end(); pai++) {
 	const PathAttribute* pa;
 	pa = *pai;
@@ -1106,6 +1118,12 @@ mrtd_routview_dump(const  UpdatePacket* p, const IPNet<A>& net,
 	    XLOG_FATAL("fwrite of %s failed: %s", fname.c_str(),
 		       strerror(errno));
     }
+#endif
+    if(fwrite(buf, pa_len, 1, fp) != 1)
+	XLOG_FATAL("fwrite of %s failed: %s", fname.c_str(),
+		   strerror(errno));
+    
+
 
     fclose(fp);
 }
@@ -1119,7 +1137,7 @@ text_traffic_dump(const uint8_t *buf, const size_t len,
     if(0 == fp)
 	XLOG_FATAL("fopen of %s failed: %s", fname.c_str(), strerror(errno));
 
-    fprintf(fp, bgppp(buf, len, peerdata).c_str());
+    fprintf(fp, "%s", bgppp(buf, len, peerdata).c_str());
     fclose(fp);
 }
 
@@ -1431,7 +1449,7 @@ Peer::check_expect(BGPPacket *rec)
 	    break;
 	case MESSAGETYPEUPDATE:
 	    {
-	    UpdatePacket *pac = new UpdatePacket(rec_buf, rec_len, _peerdata);
+	    UpdatePacket *pac = new UpdatePacket(rec_buf, rec_len, _peerdata, 0, false);
 	    _expect._bad = pac;
 	    }
 	    break;
@@ -1525,10 +1543,16 @@ Peer::datain(const bool& status, const TimeVal& tv,
 	case MESSAGETYPEOPEN: {
 	    debug_msg("OPEN Packet RECEIVED\n");
 	    OpenPacket pac(buf, length);
-	    debug_msg(pac.str().c_str());
+	    debug_msg("%s", pac.str().c_str());
 	    
-	    if(_session && !_established && _passive)
-		send_open();
+	    if(_session && !_established) {
+		if(_passive) {
+		    send_open();
+		} else {
+		    send_keepalive();
+		    _established = true;
+		}
+	    }
 	    check_expect(&pac);
 	}
 	    break;
@@ -1542,11 +1566,7 @@ Peer::datain(const bool& status, const TimeVal& tv,
 	    ** Send any received keepalive right back.
 	    ** At the moment we are not bothering to maintain a timer
 	    ** to send keepalives. An incoming keepalive prompts a
-	    ** keepalive response. At the beginning of a conversation
-	    ** this keepalive response will cause the peer to go to
-	    ** established. As opens must have been exchanged for the
-	    ** peer to send a keepalive. 
-	    ** 
+	    ** keepalive response.
 	    */
 	    if(_keepalive || (_session && !_established)) {
 		XrlTestPeerV0p1Client test_peer(_xrlrouter);
@@ -1563,8 +1583,8 @@ Peer::datain(const bool& status, const TimeVal& tv,
 	    break;
 	case MESSAGETYPEUPDATE: {
 	    debug_msg("UPDATE Packet RECEIVED\n");
-	    UpdatePacket pac(buf, length, _peerdata);
-	    debug_msg(pac.str().c_str());
+	    UpdatePacket pac(buf, length, _peerdata, 0, false);
+	    debug_msg("%s", pac.str().c_str());
 	    /*
 	    ** Save the update message in the receive trie.
 	    */
@@ -1575,7 +1595,7 @@ Peer::datain(const bool& status, const TimeVal& tv,
 	case MESSAGETYPENOTIFICATION: {
 	    debug_msg("NOTIFICATION Packet RECEIVED\n");
 	    NotificationPacket pac(buf, length);
-	    debug_msg(pac.str().c_str());
+	    debug_msg("%s", pac.str().c_str());
 	    check_expect(&pac);
 	}
 	    break;
@@ -1655,6 +1675,23 @@ Peer::send_open()
     send_message(buf, len, callback(this, &Peer::xrl_callback, "open"));
 }
 
+void
+Peer::send_keepalive()
+{
+    /*
+    ** Create an keepalive packet and send it in.
+    */
+    KeepAlivePacket keepalive;
+    size_t len = BGPPacket::MAXPACKETSIZE;
+    uint8_t *buf = new uint8_t[BGPPacket::MAXPACKETSIZE];
+    
+    XLOG_ASSERT(keepalive.encode(buf, len, _peerdata));
+    debug_msg("KEEPALIVE Packet SENT\n");
+
+    _busy++;
+    send_message(buf, len, callback(this, &Peer::xrl_callback, "keepalive"));
+}
+
 /*
 ** The standard BGP code that deals with path attributes contains
 ** fairly stringent checking. For testing we want to be able to create
@@ -1692,6 +1729,7 @@ public:
 	} else {
 	    _wire_size = 3 + _size;
 	}
+	_type = _data[1];
     };
     
     PathAttribute *clone() const { 
@@ -1733,7 +1771,7 @@ Peer::path_attribute(const char *)
     uint16_t max_len = sizeof(path);
     size_t actual_length;
 
-    return PathAttribute::create(&path[0], max_len, actual_length, _peerdata);
+    return PathAttribute::create(&path[0], max_len, actual_length, _peerdata, 4);
 }
 
 /**
