@@ -1,7 +1,7 @@
-#/usr/bin/env bash
+#!/usr/bin/env bash
 
 #
-# $XORP: xorp/bgp/harness/soak.sh,v 1.3 2003/01/29 07:03:58 atanu Exp $
+# $XORP: xorp/bgp/harness/soak.sh,v 1.4 2005/04/17 19:34:33 pavlin Exp $
 #
 
 #
@@ -9,6 +9,9 @@
 #
 set -e
 
+# srcdir is set by make for check target
+if [ "X${srcdir}" = "X" ] ; then srcdir=`dirname $0` ; fi
+
 TESTS="test_peering1.sh test_peering2.sh test_routing1.sh test_rib1.sh 
     test_rib_fea1.sh
     test_terminate.sh"
@@ -17,8 +20,8 @@ while :
 do
     for i in $TESTS
     do
-	./$i
-	./$i -l
+	${srcdir}/$i
+	${srcdir}/$i -l
     done
 done
 
