@@ -1,18 +1,23 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/bgp/harness/test_peer.cc,v 1.52 2008/07/23 05:09:43 pavlin Exp $"
+#ident "$XORP: xorp/bgp/harness/test_peer.cc,v 1.56 2009/01/05 18:30:45 jtc Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -275,7 +280,10 @@ TestPeer::TestPeer(EventLoop& eventloop, XrlRouter& xrlrouter,
      _bgp_bytes(0)
 {
     _localdata = new LocalData(eventloop);
+    _localdata->set_as(AsNum(0));
     _peerdata = new BGPPeerData(*_localdata, Iptuple(), AsNum(0), IPv4(), 0);
+    // we force IBGP, as this does fewer tests
+    _peerdata->compute_peer_type();
 }
 
 TestPeer::~TestPeer()
@@ -382,7 +390,8 @@ TestPeer::connect(const string& host, const uint32_t& port,
    */
    if (0 != _coordinator.length() &&
        !_eventloop.add_ioevent_cb(s, IOT_READ,
-				callback(this, &TestPeer::receive))) {
+				  callback(this, &TestPeer::receive),
+				  XorpTask::PRIORITY_LOWEST)) {
 	comm_sock_close(s);
 	XLOG_WARNING("Failed to add socket %s to eventloop", s.str().c_str());
 	return false;
@@ -395,7 +404,7 @@ TestPeer::connect(const string& host, const uint32_t& port,
         XLOG_FATAL("Failed to go non-blocking: %s", comm_get_last_error_str());
     }
     delete _async_writer;
-    _async_writer = new AsyncFileWriter(_eventloop, s);
+    _async_writer = new AsyncFileWriter(_eventloop, s, XorpTask::PRIORITY_LOWEST);
 
     _s = s;
 
@@ -447,8 +456,9 @@ TestPeer::listen(const string& host, const uint32_t& port,
     }
 
     if(!_eventloop.add_ioevent_cb(s, IOT_ACCEPT,
-				callback(this,
-					 &TestPeer::connect_attempt))) {
+				  callback(this,
+					   &TestPeer::connect_attempt),
+				  XorpTask::PRIORITY_LOWEST)) {
 	comm_sock_close(s);
 	error_string = c_format("Failed to add socket %s to eventloop",
 				s.str().c_str());
@@ -668,7 +678,8 @@ TestPeer::connect_attempt(XorpFd fd, IoEventType type)
    */
    if(0 != _coordinator.length() &&
        !_eventloop.add_ioevent_cb(connfd, IOT_READ,
-				callback(this, &TestPeer::receive))) {
+				  callback(this, &TestPeer::receive),
+				  XorpTask::PRIORITY_LOWEST)) {
 	comm_sock_close(connfd);
 	XLOG_WARNING("Failed to add socket %s to eventloop",
 			connfd.str().c_str());
@@ -682,7 +693,7 @@ TestPeer::connect_attempt(XorpFd fd, IoEventType type)
         XLOG_FATAL("Failed to go non-blocking: %s", comm_get_last_error_str());
     }
     delete _async_writer;
-   _async_writer = new AsyncFileWriter(_eventloop, connfd);
+   _async_writer = new AsyncFileWriter(_eventloop, connfd, XorpTask::PRIORITY_LOWEST);
 
    _s = connfd;
 }
