@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 
 #
-# $XORP: xorp/bgp/harness/test_peering2.sh,v 1.62 2008/04/25 19:10:01 atanu Exp $
+# $XORP: xorp/bgp/harness/test_peering2.sh,v 1.65 2008/11/08 06:14:44 mjh Exp $
 #
 
 #
@@ -810,8 +810,7 @@ test15()
     coord peer1 dump sent text ipv4 traffic $SENT
 
     # send in the saved file
-    UPDATES=
-    NOBLOCK=true coord peer1 send dump mrtd update $TFILE $UPDATES
+    NOBLOCK=true coord peer1 send dump mrtd update $TFILE
 
     # Wait for the whole feed to be sent it may not all have arrived at BGP
     bgp_peer_unchanged peer1
@@ -834,6 +833,7 @@ test15()
     # Check all the sent packets and verify that they arrived at peer3
     XRLS=${TMPDIR}/xrls.$EXT
     echo "Building lookup list in $XRLS"
+    cat $SENT > sentdata
     cat $SENT | $PYTHON $srcdir/lookup.py --peer peer3 \
 					 --trie recv \
 					 --add 65008 > $XRLS
@@ -843,6 +843,7 @@ test15()
     # Check all the received packets and verify that they were sent by peer1
     XRLS=${TMPDIR}/xrls.$EXT
     echo "Building lookup list in $XRLS"
+    cat $RECV > recvdata
     cat $RECV | $PYTHON $srcdir/lookup.py --peer peer1 \
 					 --trie sent \
 					 --remove 65008 > $XRLS
@@ -880,8 +881,7 @@ test16()
     coord peer2 dump sent text ipv4 traffic $SENT
 
     # send in the saved file
-    UPDATES=
-    NOBLOCK=true coord peer2 send dump mrtd update $TFILE $UPDATES
+    NOBLOCK=true coord peer2 send dump mrtd update $TFILE
 
     # Wait for the whole feed to be sent it may not all have arrived at BGP
     bgp_peer_unchanged peer2
