@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 
 #
-# $XORP: xorp/bgp/harness/test_route_flap_damping1.sh,v 1.3 2006/08/16 22:10:14 atanu Exp $
+# $XORP: xorp/bgp/harness/test_route_flap_damping1.sh,v 1.4 2007/12/10 23:26:33 mjh Exp $
 #
 
 #
@@ -60,6 +60,7 @@ PEER_PORT1_IPV6=20004
 PEER_PORT2_IPV6=20005
 PEER_PORT3_IPV6=20006
 AS=65008
+USE4BYTEAS=false
 PEER1_AS=65001
 PEER2_AS=65002
 PEER3_AS=$AS
@@ -84,7 +85,7 @@ configure_bgp()
 {
     LOCALHOST=$HOST
     AS=65008
-    local_config $AS $ID
+    local_config $AS $ID $USE4BYTEAS
 
     # Don't try and talk to the rib.
     register_rib ""
