@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 
 #
-# $XORP: xorp/bgp/harness/test_route_reflection1.sh,v 1.6 2006/08/16 22:10:14 atanu Exp $
+# $XORP: xorp/bgp/harness/test_route_reflection1.sh,v 1.8 2008/11/08 06:14:44 mjh Exp $
 #
 
 #
@@ -60,6 +60,7 @@ PEER_PORT1_IPV6=20004
 PEER_PORT2_IPV6=20005
 PEER_PORT3_IPV6=20006
 AS=65008
+USE4BYTEAS=false
 PEER1_AS=$AS
 PEER2_AS=$AS
 PEER3_AS=$AS
@@ -85,7 +86,7 @@ configure_bgp()
 {
     LOCALHOST=$HOST
     AS=65008
-    local_config $AS $ID
+    local_config $AS $ID $USE4BYTEAS
     route_reflector $CLUSTER_ID false
 
     # Don't try and talk to the rib.
@@ -264,16 +265,16 @@ test4()
 
     config_peers_ipv4
 
-    PACKET="packet update
+    BASE_PACKET="packet update
 	origin 2
 	aspath 1
 	nexthop $NEXT_HOP
 	nlri 10.10.10.0/24
-	localpref 10
-	clusterlist 1.2.3.4
-	originatorid $ID"
+	localpref 10"
 
-    RR_PACKET="$PACKET originatorid $ID clusterlist $CLUSTER_ID"
+    PACKET="$BASE_PACKET originatorid $ID clusterlist 1.2.3.4"
+
+    RR_PACKET="$BASE_PACKET originatorid $ID clusterlist $CLUSTER_ID"
 
     coord peer1 expect $RR_PACKET	
     coord peer2 expect $RR_PACKET	
@@ -308,7 +309,7 @@ test5()
 	localpref 10
 	clusterlist $CLUSTER_ID"
 
-    RR_PACKET="$PACKET originatorid $ID clusterlist $CLUSTER_ID"
+    RR_PACKET="$PACKET originatorid $ID"
 
     coord peer1 expect $RR_PACKET	
     coord peer2 expect $RR_PACKET	
@@ -333,6 +334,7 @@ TESTS='test1 test2 test3 test4 test5'
 # Include command line
 . ${srcdir}/args.sh
 
+#START_PROGRAMS="no"
 if [ $START_PROGRAMS = "yes" ]
 then
     CXRL="$CALLXRL -r 10"
