@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 
 #
-# $XORP: xorp/bgp/harness/test_route_reflection2.sh,v 1.3 2006/08/16 22:10:14 atanu Exp $
+# $XORP: xorp/bgp/harness/test_route_reflection2.sh,v 1.4 2007/12/10 23:26:33 mjh Exp $
 #
 
 #
@@ -61,6 +61,7 @@ PEER_PORT1_IPV6=20004
 PEER_PORT2_IPV6=20005
 PEER_PORT3_IPV6=20006
 AS=65008
+USE4BYTEAS=false
 PEER1_AS=$AS
 PEER2_AS=$AS
 PEER3_AS=65000
@@ -86,7 +87,7 @@ configure_bgp()
 {
     LOCALHOST=$HOST
     AS=65008
-    local_config $AS $ID
+    local_config $AS $ID $USE4BYTEAS
     route_reflector $CLUSTER_ID false
 
     # Don't try and talk to the rib.
