@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 
 #
-# $XORP: xorp/bgp/harness/test_routing2.sh,v 1.21 2006/08/16 22:10:14 atanu Exp $
+# $XORP: xorp/bgp/harness/test_routing2.sh,v 1.22 2007/12/10 23:26:34 mjh Exp $
 #
 
 #
@@ -59,6 +59,7 @@ PEER_PORT1_IPV6=20004
 PEER_PORT2_IPV6=20005
 PEER_PORT3_IPV6=20006
 AS=65008
+USE4BYTEAS=false 
 PEER1_AS=$AS
 PEER2_AS=$AS
 PEER3_AS=$AS
@@ -98,7 +99,7 @@ configure_bgp()
     LOCALHOST=$HOST
     AS=65008
     ID=192.150.187.78
-    local_config $AS $ID
+    local_config $AS $ID $USE4BYTEAS
 
     # IBGP - IPV4
     PEER=$HOST
@@ -453,6 +454,8 @@ test4()
 	nlri 10.10.10.10/24 \
 	nlri 20.20.20.20/24
 
+    # Wait for the route changes to propogate.
+    sleep 2
     result=$(lookup_route_by_dest4 10.10.10.10 true false)
     echo $result
 
