@@ -1,17 +1,22 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
 #ident "$XORP"
 
@@ -41,7 +46,7 @@ DampingTable<A>::~DampingTable()
 
 template<class A>
 int
-DampingTable<A>::add_route(const InternalMessage<A> &rtmsg,
+DampingTable<A>::add_route(InternalMessage<A> &rtmsg,
 			   BGPRouteTable<A> *caller)
 {
     debug_msg("\n         %s\n caller: %s\n rtmsg: %p route: %p\n%s\n",
@@ -53,6 +58,7 @@ DampingTable<A>::add_route(const InternalMessage<A> &rtmsg,
 
     XLOG_ASSERT(caller == this->_parent);
     XLOG_ASSERT(this->_next_table != NULL);
+    XLOG_ASSERT(!rtmsg.attributes()->is_locked());
 
     if (!damping())
 	return this->_next_table->
@@ -82,8 +88,8 @@ DampingTable<A>::add_route(const InternalMessage<A> &rtmsg,
 
 template<class A>
 int
-DampingTable<A>::replace_route(const InternalMessage<A> &old_rtmsg, 
-			       const InternalMessage<A> &new_rtmsg, 
+DampingTable<A>::replace_route(InternalMessage<A> &old_rtmsg, 
+			       InternalMessage<A> &new_rtmsg, 
 			       BGPRouteTable<A> *caller)
 {
     debug_msg("\n         %s\n"
@@ -164,8 +170,8 @@ DampingTable<A>::replace_route(const InternalMessage<A> &old_rtmsg,
 
 template<class A>
 int
-DampingTable<A>::delete_route(const InternalMessage<A> &rtmsg, 
-			    BGPRouteTable<A> *caller)
+DampingTable<A>::delete_route(InternalMessage<A> &rtmsg, 
+			      BGPRouteTable<A> *caller)
 {
     debug_msg("\n         %s\n caller: %s\n rtmsg: %p route: %p\n%s\n",
 	      this->tablename().c_str(),
@@ -219,7 +225,7 @@ DampingTable<A>::push(BGPRouteTable<A> *caller)
 
 template<class A>
 int 
-DampingTable<A>::route_dump(const InternalMessage<A> &rtmsg,
+DampingTable<A>::route_dump(InternalMessage<A> &rtmsg,
 			    BGPRouteTable<A> *caller,
 			    const PeerHandler *dump_peer)
 {
@@ -239,15 +245,16 @@ DampingTable<A>::route_dump(const InternalMessage<A> &rtmsg,
 template<class A>
 const SubnetRoute<A>*
 DampingTable<A>::lookup_route(const IPNet<A> &net,
-			      uint32_t& genid) const
+			      uint32_t& genid,
+			      FPAListRef& pa_list) const
 {
     if (!damping())
-	return this->_parent->lookup_route(net, genid);
+	return this->_parent->lookup_route(net, genid, pa_list);
 
     if (is_this_route_damped(net))
 	return 0;
 
-    return this->_parent->lookup_route(net, genid);
+    return this->_parent->lookup_route(net, genid, pa_list);
 }
 
 template<class A>
