@@ -1,18 +1,23 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/bgp/route_table_dump.cc,v 1.42 2008/07/23 05:09:36 pavlin Exp $"
+#ident "$XORP: xorp/bgp/route_table_dump.cc,v 1.45 2009/01/05 18:30:43 jtc Exp $"
 
 //#define DEBUG_LOGGING
 //#define DEBUG_PRINT_FUNCTION_NAME
@@ -60,7 +65,7 @@ DumpTable<A>::DumpTable(string table_name,
 
 template<class A>
 int
-DumpTable<A>::add_route(const InternalMessage<A> &rtmsg,
+DumpTable<A>::add_route(InternalMessage<A> &rtmsg,
 			BGPRouteTable<A> *caller) 
 {
     debug_msg("\n         %s\n caller: %s\n"
@@ -88,7 +93,7 @@ DumpTable<A>::add_route(const InternalMessage<A> &rtmsg,
 			   rtmsg.net().str().c_str()));
 #endif
 	return this->_next_table->add_route(rtmsg,
-				      static_cast<BGPRouteTable<A>*>(this));
+					   static_cast<BGPRouteTable<A>*>(this));
     } else {
 	cp(3);
 #ifdef AUDIT_ENABLE
@@ -108,8 +113,8 @@ DumpTable<A>::add_route(const InternalMessage<A> &rtmsg,
 
 template<class A>
 int
-DumpTable<A>::replace_route(const InternalMessage<A> &old_rtmsg,
-			    const InternalMessage<A> &new_rtmsg,
+DumpTable<A>::replace_route(InternalMessage<A> &old_rtmsg,
+			    InternalMessage<A> &new_rtmsg,
 			    BGPRouteTable<A> *caller) 
 {
     debug_msg("%s::replace_route %p -> %p\n",
@@ -163,7 +168,7 @@ DumpTable<A>::replace_route(const InternalMessage<A> &old_rtmsg,
 
 template<class A>
 int
-DumpTable<A>::route_dump(const InternalMessage<A> &rtmsg,
+DumpTable<A>::route_dump(InternalMessage<A> &rtmsg,
 			 BGPRouteTable<A> *caller,
 			 const PeerHandler* dump_peer) 
 {
@@ -196,7 +201,7 @@ DumpTable<A>::route_dump(const InternalMessage<A> &rtmsg,
 
 template<class A>
 int
-DumpTable<A>::delete_route(const InternalMessage<A> &rtmsg,
+DumpTable<A>::delete_route(InternalMessage<A> &rtmsg,
 			   BGPRouteTable<A> *caller) 
 {
     debug_msg("%s::delete_route %p\n",
@@ -241,10 +246,11 @@ DumpTable<A>::push(BGPRouteTable<A> *caller)
 
 template<class A>
 const SubnetRoute<A>*
-DumpTable<A>::lookup_route(const IPNet<A> &net, uint32_t& genid) const 
+DumpTable<A>::lookup_route(const IPNet<A> &net, uint32_t& genid,
+			   FPAListRef& pa_list) const 
 {
     cp(13);
-    return this->_parent->lookup_route(net, genid);
+    return this->_parent->lookup_route(net, genid, pa_list);
 }
 
 template<class A>
