@@ -1,18 +1,23 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/bgp/test_deletion.cc,v 1.35 2008/07/23 05:09:38 pavlin Exp $"
+#ident "$XORP: xorp/bgp/test_deletion.cc,v 1.38 2009/01/05 18:30:43 jtc Exp $"
 
 #include "bgp_module.h"
 
@@ -105,19 +110,19 @@ test_deletion(TestInfo& /*info*/)
     aspath3.prepend_as(AsNum(8));
     ASPathAttribute aspathatt3(aspath3);
 
-    PathAttributeList<IPv4>* palist1 =
-	new PathAttributeList<IPv4>(nhatt1, aspathatt1, igp_origin_att);
+    FPAList4Ref fpalist1 =
+	new FastPathAttributeList<IPv4>(nhatt1, aspathatt1, igp_origin_att);
+    PAListRef<IPv4> palist1 = new PathAttributeList<IPv4>(fpalist1);
 
-    PathAttributeList<IPv4>* palist2 =
-	new PathAttributeList<IPv4>(nhatt2, aspathatt2, igp_origin_att);
+    FPAList4Ref fpalist2 =
+	new FastPathAttributeList<IPv4>(nhatt2, aspathatt2, igp_origin_att);
+    PAListRef<IPv4> palist2 = new PathAttributeList<IPv4>(fpalist2);
 
-    PathAttributeList<IPv4>* palist3 =
-	new PathAttributeList<IPv4>(nhatt3, aspathatt3, igp_origin_att);
+    FPAList4Ref fpalist3 =
+	new FastPathAttributeList<IPv4>(nhatt3, aspathatt3, igp_origin_att);
+    PAListRef<IPv4> palist3 = new PathAttributeList<IPv4>(fpalist3);
 
-    //create a subnet route
-    SubnetRoute<IPv4> *sr1;
-
-    InternalMessage<IPv4>* msg;
+    PolicyTags pt;
 
     //================================================================
     //Test1: trivial add and peering goes down
@@ -129,11 +134,7 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net1, fpalist1, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("PEERING GOES DOWN");
@@ -160,27 +161,15 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net1, fpalist1, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net2, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net2, fpalist1, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net3, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net3, fpalist1, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("PEERING GOES DOWN");
@@ -207,27 +196,15 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net1, fpalist1, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net2, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net2, fpalist2, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net3, palist3, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net3, fpalist3, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("PEERING GOES DOWN");
@@ -254,35 +231,19 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net1, fpalist1, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net2, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net2, fpalist2, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net3, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net3, fpalist1, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net4, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net4, fpalist2, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("PEERING GOES DOWN");
@@ -339,35 +300,19 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net1, fpalist1, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net2, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net2, fpalist2, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net3, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net3, fpalist1, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net4, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net4, fpalist2, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("PEERING GOES DOWN");
@@ -379,19 +324,11 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net1, fpalist1, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net4, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net4, fpalist2, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
@@ -429,35 +366,19 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net1, fpalist1, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net2, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net2, fpalist2, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net3, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net3, fpalist1, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net4, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net4, fpalist2, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("PEERING GOES DOWN");
@@ -469,19 +390,11 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net1, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net1, fpalist2, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net4, palist3, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net4, fpalist3, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
@@ -519,35 +432,19 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net1, fpalist1, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net2, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net2, fpalist2, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net3, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net3, fpalist1, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net4, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net4, fpalist2, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("PEERING GOES DOWN");
@@ -559,19 +456,11 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net1, palist3, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net1, fpalist3, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net3, palist3, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net3, fpalist3, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
@@ -609,19 +498,11 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net1, fpalist1, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net2, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net2, fpalist2, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("PEERING GOES DOWN");
@@ -633,19 +514,11 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net1, fpalist1, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net2, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net2, fpalist2, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
@@ -681,11 +554,7 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net1, fpalist1, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("PEERING GOES DOWN");
@@ -697,11 +566,7 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("ADD A ROUTE");
-    sr1 = new SubnetRoute<IPv4>(net2, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
-    ribin->add_route(*msg, NULL);
-    sr1->unref();
-    delete msg;
+    ribin->add_route(net2, fpalist2, pt);
 
     debug_table->write_separator();
     debug_table->write_comment("PEERING GOES DOWN");
@@ -726,9 +591,12 @@ test_deletion(TestInfo& /*info*/)
     debug_table->write_comment("SHUTDOWN AND CLEAN UP");
     delete ribin;
     delete debug_table;
-    delete palist1;
-    delete palist2;
-    delete palist3;
+    palist1.release();
+    palist2.release();
+    palist3.release();
+    fpalist1 = 0;
+    fpalist2 = 0;
+    fpalist3 = 0;
 
     FILE *file = fopen(filename.c_str(), "r");
     if (file == NULL) {
