@@ -1,49 +1,25 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
 /*
- * Copyright (c) 2001-2008 XORP, Inc.
+ * Copyright (c) 2001-2009 XORP, Inc.
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software")
- * to deal in the Software without restriction, subject to the conditions
- * listed in the XORP LICENSE file. These conditions include: you must
- * preserve this copyright notice, and you cannot mention the copyright
- * holders in advertising related to the Software without their permission.
- * The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
- * notice is a summary of the XORP LICENSE file; the license in that file is
- * legally binding.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License, Version 2, June
+ * 1991 as published by the Free Software Foundation. Redistribution
+ * and/or modification of this program under the terms of any other
+ * version of the GNU General Public License is not permitted.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+ * see the GNU General Public License, Version 2, a copy of which can be
+ * found in the XORP LICENSE.gpl file.
+ * 
+ * XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+ * http://xorp.net
  */
 
-/*
- * Portions of this software have one or more of the following copyrights, and
- * are subject to the license below. The relevant source files are clearly
- * marked; they refer to this file using the phrase ``the XORP LICENSE file''.
- *
- * Copyright (c) 2001-2008 XORP, Inc.
- * 
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- * 
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- * 
- * The names and trademarks of copyright holders may not be used in
- * advertising or publicity pertaining to the software without specific
- * prior permission. Title to copyright in this software and any associated
- * documentation will at all times remain with the copyright holders.
- * 
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+#ident "$XORP: xorp/cli/libtecla/nettest_libtecla.c,v 1.12 2009/01/05 18:30:46 jtc Exp $"
 
 
 /*
