@@ -1,18 +1,23 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/fea/data_plane/ifconfig/ifconfig_parse_routing_socket.cc,v 1.24 2008/07/23 05:10:28 pavlin Exp $"
+#ident "$XORP: xorp/fea/data_plane/ifconfig/ifconfig_parse_routing_socket.cc,v 1.29 2009/01/05 18:30:53 jtc Exp $"
 
 #include "fea/fea_module.h"
 
@@ -47,7 +52,6 @@
 #endif
 
 #include "fea/ifconfig.hh"
-#include "fea/ifconfig_get.hh"
 #include "fea/data_plane/control_socket/system_utilities.hh"
 #include "fea/data_plane/control_socket/routing_socket_utilities.hh"
 
@@ -213,6 +217,16 @@ IfConfigGetSysctl::parse_buffer_routing_socket(IfConfig& ifconfig,
     for (offset = 0; offset < buffer.size(); offset += ifm->ifm_msglen) {
 	ifm = align_data.payload_by_offset(offset);
 	if (ifm->ifm_version != RTM_VERSION) {
+#if defined(RTM_OVERSION) && defined(HOST_OS_OPENBSD)
+	    //
+	    // XXX: Silently ignore old messages.
+	    // The OpenBSD kernel sends each message twice, once as
+	    // RTM_VERSION and once as RTM_OVERSION, hence we need to ignore
+	    // the RTM_OVERSION duplicates.
+	    //
+	    if (ifm->ifm_version == RTM_OVERSION)
+		continue;
+#endif // RTM_OVERSION && HOST_OS_OPENBSD
 	    XLOG_ERROR("RTM version mismatch: expected %d got %d",
 		       RTM_VERSION,
 		       ifm->ifm_version);
@@ -468,9 +482,9 @@ rtm_ifinfo_to_fea_cfg(const struct if_msghdr* ifm, IfTree& iftree,
 		struct ether_addr ea;
 		memcpy(&ea, sdl->sdl_data + sdl->sdl_nlen,
 		       sdl->sdl_alen);
-		EtherMac ether_mac(ea);
-		if (is_newlink || (ether_mac != EtherMac(ifp->mac())))
-		    ifp->set_mac(ether_mac);
+		Mac mac(ea);
+		if (is_newlink || (mac != ifp->mac()))
+		    ifp->set_mac(mac);
 		break;
 	    } else if (sdl->sdl_alen != 0) {
 		XLOG_ERROR("Address size %d uncatered for interface %s",
@@ -496,9 +510,9 @@ rtm_ifinfo_to_fea_cfg(const struct if_msghdr* ifm, IfTree& iftree,
 	    } else {
 		struct ether_addr ea;
 		memcpy(&ea, ifridx.ifr_hwaddr.sa_data, sizeof(ea));
-		EtherMac ether_mac(ea);
-		if (is_newlink || (ether_mac != EtherMac(ifp->mac())))
-		    ifp->set_mac(ether_mac);
+		Mac mac(ea);
+		if (is_newlink || (mac != ifp->mac()))
+		    ifp->set_mac(mac);
 		close(s);
 		break;
 	    }
