@@ -1,18 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/fea/data_plane/ifconfig/ifconfig_set_netlink_socket.cc,v 1.21 2008/07/23 05:10:30 pavlin Exp $"
+#ident "$XORP: xorp/fea/data_plane/ifconfig/ifconfig_set_netlink_socket.cc,v 1.26 2009/01/05 18:30:53 jtc Exp $"
 
 #include "fea/fea_module.h"
 
@@ -39,7 +45,6 @@
 #endif
 
 #include "fea/ifconfig.hh"
-#include "fea/ifconfig_set.hh"
 #include "fea/data_plane/control_socket/netlink_socket_utilities.hh"
 
 #include "ifconfig_set_netlink_socket.hh"
@@ -224,6 +229,12 @@ IfConfigSetNetlinkSocket::config_interface_begin(
 
  done:
     if (was_disabled) {
+	// We need to "hide" the Linux quirk of toggling the interface down / up
+	// and make sure that our configuration is up to date.  Otherwise we'll
+	// later be notified (by the observer) that the interface went down
+	// unexpectedly and things may break, when instead it is only a
+	// transient state.
+        wait_interface_status(pulled_ifp, false);
 	if (set_interface_status(config_iface.ifname(),
 				 config_iface.pif_index(),
 				 config_iface.interface_flags(),
@@ -232,6 +243,7 @@ IfConfigSetNetlinkSocket::config_interface_begin(
 	    != XORP_OK) {
 	    return (XORP_ERROR);
 	}
+        wait_interface_status(pulled_ifp, true);
     }
 
     return (ret_value);
@@ -623,6 +635,26 @@ IfConfigSetNetlinkSocket::set_interface_status(const string& ifname,
 #endif // HAVE_NETLINK_SOCKETS_SET_FLAGS_IS_BROKEN
 }
 
+void
+IfConfigSetNetlinkSocket::wait_interface_status(const IfTreeInterface* ifp,
+						bool is_enabled)
+{
+    NetlinkSocket* ns = dynamic_cast<NetlinkSocket*>(fea_data_plane_manager()
+						     .ifconfig_observer());
+    string error_msg;
+
+    // This is supposed to run only on Linux.  I could enforce this more, but
+    // I'll leave the code flexible in case other platforms need such a
+    // mechanism.
+    if (!ns)
+	return;
+
+    while (ifp->enabled() != is_enabled) {
+	if (ns->force_recvmsg(0, true, error_msg) != XORP_OK)
+	    XLOG_ERROR("Netlink force_recvmsg(): %s", error_msg.c_str());
+    }
+}
+
 int
 IfConfigSetNetlinkSocket::set_interface_mac_address(const string& ifname,
 						    uint32_t if_index,
@@ -631,18 +663,7 @@ IfConfigSetNetlinkSocket::set_interface_mac_address(const string& ifname,
 {
     struct ether_addr ether_addr;
 
-    // XXX: Hard-coded assumption that this is an Ethernet interface
-    try {
-	EtherMac ether_mac(mac);
-	if (ether_mac.copy_out(ether_addr) != EtherMac::ADDR_BYTELEN) {
-	    error_msg = c_format("Expected Ethernet MAC address, got \"%s\"",
-				 mac.str().c_str());
-	    return (XORP_ERROR);
-	}
-    } catch (const BadMac& bad_mac) {
-	error_msg = c_format("Invalid MAC address \"%s\"", mac.str().c_str());
-	return (XORP_ERROR);
-    }
+    mac.copy_out(ether_addr);
 
 #ifdef RTM_SETLINK
     static const size_t	buffer_size = sizeof(struct nlmsghdr)
