@@ -1,19 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8 sw=4:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/fea/data_plane/io/io_ip_socket.cc,v 1.22 2008/07/23 05:10:32 pavlin Exp $"
+#ident "$XORP: xorp/fea/data_plane/io/io_ip_socket.cc,v 1.29 2009/01/05 18:30:54 jtc Exp $"
 
 //
 // I/O IP raw communication support.
@@ -268,7 +273,7 @@ IoIpSocket::IoIpSocket(FeaDataPlaneManager& fea_data_plane_manager,
     _sndiov[0].iov_len		= 0;
 
     // recvmsg() and sendmsg() related initialization
-#ifndef HOST_OS_WINDOWS    
+#ifndef HOST_OS_WINDOWS
     switch (family) {
     case AF_INET:
 	_rcvmh.msg_name		= (caddr_t)&_from4;
@@ -1790,17 +1795,26 @@ IoIpSocket::send_packet(const string& if_name,
 			string& error_msg)
 {
     size_t	ip_hdr_len = 0;
+    int		ctllen = 0;
     int		int_val;
     int		ret_value = XORP_OK;
     const IfTreeInterface* ifp = NULL;
     const IfTreeVif* vifp = NULL;
+    struct cmsghdr* cmsgp;
     void*	cmsg_data;	// XXX: CMSG_DATA() is aligned, hence void ptr
 
     UNUSED(int_val);
+    UNUSED(cmsgp);
     UNUSED(cmsg_data);
 
     XLOG_ASSERT(ext_headers_type.size() == ext_headers_payload.size());
 
+    // Initialize state that might be modified later
+#ifndef HOST_OS_WINDOWS
+    _sndmh.msg_control = (caddr_t)_sndcmsgbuf;
+    _sndmh.msg_controllen = 0;
+#endif
+
     ifp = iftree().find_interface(if_name);
     if (ifp == NULL) {
 	error_msg = c_format("No interface %s", if_name.c_str());
@@ -2031,6 +2045,57 @@ IoIpSocket::send_packet(const string& if_name,
 	    break;
 	}
 
+	//
+	// Use raw IP header
+	//
+
+	//
+	// First, estimate total length of ancillary data
+	//
+
+	// Space for IP_PKTINFO
+#ifdef IP_PKTINFO
+	ctllen += CMSG_SPACE(sizeof(struct in_pktinfo));
+#endif
+
+	//
+	// Now setup the ancillary data
+	//
+	XLOG_ASSERT(ctllen <= CMSG_BUF_SIZE);		// XXX
+#ifndef HOST_OS_WINDOWS
+	_sndmh.msg_controllen = ctllen;
+	cmsgp = CMSG_FIRSTHDR(&_sndmh);
+#endif
+
+#ifdef IP_PKTINFO
+#ifndef HOST_OS_WINDOWS
+	// XXX: Linux
+	{
+	    //
+	    // XXX: Linux ignores the source address in IP header for the
+	    // routing table lookup when the packet is sent with sendmsg(2).
+	    // The in_pktinfo logic below makes sure the kernel considers
+	    // the source address for the routing table lookup
+	    // (e.g., selecting the proper forwarding table when the system
+	    // is configured to use multiple forwarding tables), and for
+	    // setting the IP source route options.
+	    // See ip(7) for description of the usage of IP_PKTINFO
+	    // and struct in_pktinfo.
+	    //
+	    struct in_pktinfo *sndpktinfo;
+
+	    // Add the IP_PKTINFO ancillary data
+	    cmsgp->cmsg_len   = CMSG_LEN(sizeof(struct in_pktinfo));
+	    cmsgp->cmsg_level = SOL_IP;
+	    cmsgp->cmsg_type  = IP_PKTINFO;
+	    cmsg_data = CMSG_DATA(cmsgp);
+	    sndpktinfo = reinterpret_cast<struct in_pktinfo *>(cmsg_data);
+	    memset(sndpktinfo, 0, sizeof(*sndpktinfo));
+	    src_address.copy_out(sndpktinfo->ipi_spec_dst);
+	}
+#endif // ! HOST_OS_WINDOWS
+#endif // IP_PKTINFO
+
 	//
 	// Set the IPv4 header
 	//
@@ -2105,11 +2170,11 @@ IoIpSocket::send_packet(const string& if_name,
 #ifdef HAVE_IPV6
     case AF_INET6:
     {
-	int ctllen = 0;
 	int hbhlen = 0;
-	struct cmsghdr *cmsgp;
 	struct in6_pktinfo *sndpktinfo;
-	
+
+	UNUSED(hbhlen);			// XXX: might be unused for older OSs
+
 	//
 	// XXX: unlikely IPv4, in IPv6 the 'header' is specified as
 	// ancillary data.
@@ -2118,11 +2183,12 @@ IoIpSocket::send_packet(const string& if_name,
 	//
 	// First, estimate total length of ancillary data
 	//
+
 	// Space for IPV6_PKTINFO
-	ctllen = CMSG_SPACE(sizeof(struct in6_pktinfo));
-	
+	ctllen += CMSG_SPACE(sizeof(struct in6_pktinfo));
+
+	// Space for Router Alert option
 	if (ip_router_alert) {
-	    // Space for Router Alert option
 #ifdef HAVE_RFC3542
 	    if ((hbhlen = inet6_opt_init(NULL, 0)) == -1) {
 		error_msg = c_format("inet6_opt_init(NULL) failed");
@@ -2148,15 +2214,15 @@ IoIpSocket::send_packet(const string& if_name,
 	    //
 	    hbhlen = inet6_option_space(sizeof(ra_opt6));
 	    ctllen += hbhlen;
-#else
-	    UNUSED(hbhlen);
 #endif
 #endif // ! HAVE_RFC3542
 	}
+
 	// Space for IPV6_TCLASS
 #ifdef IPV6_TCLASS 
 	ctllen += CMSG_SPACE(sizeof(int));
 #endif
+
 	// Space for IPV6_HOPLIMIT
 	ctllen += CMSG_SPACE(sizeof(int));
 
@@ -2174,12 +2240,11 @@ IoIpSocket::send_packet(const string& if_name,
 	    }
 	    ctllen += CMSG_SPACE(ext_headers_payload[i].size());
 	}
-
-	XLOG_ASSERT(ctllen <= CMSG_BUF_SIZE);   // XXX
 	
 	//
 	// Now setup the ancillary data
 	//
+	XLOG_ASSERT(ctllen <= CMSG_BUF_SIZE);		// XXX
 	_sndmh.msg_controllen = ctllen;
 	cmsgp = CMSG_FIRSTHDR(&_sndmh);
 	
@@ -2406,18 +2471,16 @@ IoIpSocket::proto_socket_transmit(const IfTreeInterface* ifp,
 #ifndef HOST_OS_WINDOWS    
 
     // Set some sendmsg()-related fields
+    if (_sndmh.msg_controllen == 0)
+	_sndmh.msg_control = NULL;
     switch (family()) {
     case AF_INET:
 	dst_address.copy_out(_to4);
-	_sndmh.msg_namelen	= sizeof(_to4);
-	_sndmh.msg_control	= NULL;
-	_sndmh.msg_controllen	= 0;
 	break;
 #ifdef HAVE_IPV6
     case AF_INET6:
 	dst_address.copy_out(_to6);
 	system_adjust_sockaddr_in6_send(_to6, vifp->pif_index());
-	_sndmh.msg_namelen  = sizeof(_to6);
 	break;
 #endif // HAVE_IPV6
     default:
