@@ -1,18 +1,23 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2008 XORP, Inc.
+// Copyright (c) 2008-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/fea/firewall_manager.cc,v 1.6 2008/07/23 05:10:08 pavlin Exp $"
+#ident "$XORP: xorp/fea/firewall_manager.cc,v 1.9 2009/01/05 18:30:49 jtc Exp $"
 
 #include "fea_module.h"
 
@@ -75,6 +80,12 @@ FirewallManager::status(string& reason) const
 int
 FirewallManager::start_transaction(uint32_t& tid, string& error_msg)
 {
+    if (start(error_msg) != XORP_OK) {
+	error_msg = c_format("Cannot start FirewallManager: %s",
+			     error_msg.c_str());
+	return (XORP_ERROR);
+    }
+
     if (_ftm->start(tid) != true) {
 	error_msg = c_format("Resource limit on number of pending transactions hit");
 	return (XORP_ERROR);
