@@ -1,18 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2007-2008 XORP, Inc.
+// Copyright (c) 2007-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/fea/io_link_manager.cc,v 1.9 2008/07/23 05:10:10 pavlin Exp $"
+#ident "$XORP: xorp/fea/io_link_manager.cc,v 1.15 2009/01/05 18:30:49 jtc Exp $"
 
 #include "fea_module.h"
 
@@ -24,6 +30,41 @@
 #include "iftree.hh"
 #include "io_link_manager.hh"
 
+// A filter that matches no packets - useful for TX only.
+class TxOnlyFilter : public IoLinkComm::InputFilter {
+public:
+    // A MAC address that will "never" match
+    static const Mac filter_mac;
+
+    TxOnlyFilter(IoLinkManager&	io_link_manager,
+		 const string&	receiver_name,
+		 const string&	if_name,
+		 const string&	vif_name,
+		 uint16_t	ether_type)
+	: IoLinkComm::InputFilter(io_link_manager, receiver_name, if_name,
+				  vif_name, ether_type,
+				  "ether src " + filter_mac.str())
+    {}
+
+    void recv(const struct MacHeaderInfo& header,
+	      const vector<uint8_t>& payload)
+    {
+	UNUSED(payload);
+
+	if (header.src_address == filter_mac)
+	    return;
+
+	XLOG_FATAL("Receiving data on a TX only filter");
+    }
+
+    void bye()
+    {
+	delete this;
+    }
+};
+
+const Mac TxOnlyFilter::filter_mac("66:66:66:66:66:66");
+
 //
 // Filter class for checking incoming raw link-level packets and checking
 // whether to forward them.
@@ -80,6 +121,12 @@ public:
     void bye() {}
 
     int join_multicast_group(const Mac& group_address, string& error_msg) {
+	if (! group_address.is_multicast()) {
+	    error_msg = c_format("Cannot join group %s: not a multicast "
+				 "address",
+				 group_address.str().c_str());
+	    return (XORP_ERROR);
+	}
 	if (_io_link_comm.join_multicast_group(group_address, receiver_name(),
 					       error_msg)
 	    != XORP_OK) {
@@ -282,11 +329,7 @@ IoLinkComm::join_multicast_group(const Mac&	group_address,
     //
     // Check the arguments
     //
-    if (! group_address.is_multicast()) {
-	error_msg = c_format("Cannot join group %s: not a multicast address",
-			     group_address.str().c_str());
-	return (XORP_ERROR);
-    }
+    // LinkVifInputFilter checks that the group address is multicast.
     if (receiver_name.empty()) {
 	error_msg = c_format("Cannot join group %s on interface %s vif %s: "
 			     "empty receiver name",
@@ -533,6 +576,11 @@ IoLinkManager::IoLinkManager(FeaNode&		fea_node,
 IoLinkManager::~IoLinkManager()
 {
     erase_filters(_comm_table, _filters, _filters.begin(), _filters.end());
+
+    // Erase any TX only entries
+    for (CommTable::iterator i = _comm_table.begin(); i != _comm_table.end();
+	 ++i)
+	delete i->second;
 }
 
 void
@@ -596,14 +644,9 @@ IoLinkManager::erase_filters(CommTable& comm_table, FilterBag& filters,
     }
 }
 
-int
-IoLinkManager::send(const string&	if_name,
-		    const string&	vif_name,
-		    const Mac&		src_address,
-		    const Mac&		dst_address,
-		    uint16_t		ether_type,
-		    const vector<uint8_t>& payload,
-		    string&		error_msg)
+IoLinkComm&
+IoLinkManager::find_iolink_comm(const string& if_name, const string& vif_name,
+				uint16_t ether_type)
 {
     CommTableKey key(if_name, vif_name, ether_type, "");
 
@@ -625,21 +668,103 @@ IoLinkManager::send(const string&	if_name,
 	}
     }
 
-    if (cti == _comm_table.end()) {
-	error_msg = c_format("EtherType protocol %u is not registered "
-			     "on interface %s vif %s",
-			     XORP_UINT_CAST(ether_type),
-			     if_name.c_str(), vif_name.c_str());
-	return (XORP_ERROR);
-    }
-    IoLinkComm* io_link_comm = cti->second;
+    IoLinkComm* io_link_comm = NULL;
+
+    if (cti == _comm_table.end())
+	io_link_comm = &add_iolink_comm_txonly(if_name, vif_name, ether_type);
+    else
+	io_link_comm = cti->second;
+
     XLOG_ASSERT(io_link_comm != NULL);
 
-    return (io_link_comm->send_packet(src_address,
-				      dst_address,
-				      ether_type,
-				      payload,
-				      error_msg));
+    return *io_link_comm;
+}
+
+int
+IoLinkManager::send(const string&	if_name,
+		    const string&	vif_name,
+		    const Mac&		src_address,
+		    const Mac&		dst_address,
+		    uint16_t		ether_type,
+		    const vector<uint8_t>& payload,
+		    string&		error_msg)
+{
+    IoLinkComm& io_link_comm = find_iolink_comm(if_name, vif_name, ether_type);
+
+    return (io_link_comm.send_packet(src_address,
+				     dst_address,
+				     ether_type,
+				     payload,
+				     error_msg));
+}
+
+IoLinkComm&
+IoLinkManager::add_iolink_comm_txonly(const string&  if_name,
+				      const string&  vif_name,
+				      uint16_t       ether_type)
+{
+    //
+    // TODO: XXX: Ideally we'd configure the io plugins to do TX only, but
+    // what we do now is create a "txonlyfilter" - a filter that matches no
+    // packets and hopefully receives nothing.
+    //
+    string receiver_name  = "txonly";
+    string filter_program = "";
+
+    TxOnlyFilter* filter = new TxOnlyFilter(*this, receiver_name, if_name,
+					    vif_name, ether_type);
+    filter_program = filter->filter_program();
+
+    IoLinkComm* io_link_comm = NULL;
+    CommTableKey key(if_name, vif_name, ether_type, filter_program);
+    CommTable::iterator i = _comm_table.find(key);
+
+    if (_comm_table.find(key) != _comm_table.end())
+	io_link_comm = i->second;
+    else {
+	io_link_comm = new IoLinkComm(*this, iftree(), if_name, vif_name,
+				      ether_type, filter_program);
+	_comm_table[key] = io_link_comm;
+    }
+    XLOG_ASSERT(io_link_comm != NULL);
+
+    int rc = io_link_comm->add_filter(filter);
+    XLOG_ASSERT(rc == XORP_OK);
+
+    return *io_link_comm;
+}
+
+int
+IoLinkManager::add_multicast_mac(const string& if_name, const Mac& mac,
+				 string& error_msg)
+{
+    return (add_remove_multicast_mac(true, if_name, mac, error_msg));
+}
+
+int
+IoLinkManager::remove_multicast_mac(const string& if_name, const Mac& mac,
+				    string& error_msg)
+{
+    return (add_remove_multicast_mac(false, if_name, mac, error_msg));
+}
+
+int
+IoLinkManager::add_remove_multicast_mac(bool add, const string& if_name,
+				        const Mac& mac, string& error_msg)
+{
+    string vif_name	 = if_name;
+    uint16_t ether_type  = ETHERTYPE_IP;
+    string receiver_name = "add_remove_mac";
+    int rc;
+
+    IoLinkComm& io_link_comm = find_iolink_comm(if_name, vif_name, ether_type);
+
+    if (add)
+	rc = io_link_comm.join_multicast_group(mac, receiver_name, error_msg);
+    else
+	rc = io_link_comm.leave_multicast_group(mac, receiver_name, error_msg);
+
+    return (rc);
 }
 
 int
