@@ -1,18 +1,23 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/fea/mfea_mrouter.cc,v 1.63 2008/07/23 05:10:11 pavlin Exp $"
+#ident "$XORP: xorp/fea/mfea_mrouter.cc,v 1.67 2009/01/05 18:30:49 jtc Exp $"
 
 //
 // Multicast routing kernel-access specific implementation.
@@ -98,10 +103,6 @@
 //
 // Local constants definitions
 //
-#define IO_BUF_SIZE		(64*1024)  // I/O buffer(s) size
-#define CMSG_BUF_SIZE		(10*1024)  // 'rcvcmsgbuf' and 'sndcmsgbuf'
-#define SO_RCV_BUF_SIZE_MIN	(48*1024)  // Min. socket buffer size
-#define SO_RCV_BUF_SIZE_MAX	(256*1024) // Desired socket buffer size
 
 //
 // Local structures/classes, typedefs and macros
@@ -134,55 +135,6 @@ MfeaMrouter::MfeaMrouter(MfeaNode& mfea_node)
 {
     string error_msg;
 
-    // Allocate the buffers
-    _rcvbuf0 = new uint8_t[IO_BUF_SIZE];
-    _sndbuf0 = new uint8_t[IO_BUF_SIZE];
-    _rcvbuf1 = new uint8_t[IO_BUF_SIZE];
-    _sndbuf1 = new uint8_t[IO_BUF_SIZE];
-    _rcvcmsgbuf = new uint8_t[CMSG_BUF_SIZE];
-    _sndcmsgbuf = new uint8_t[CMSG_BUF_SIZE];
-
-    // Scatter/gatter array initialization
-    _rcviov[0].iov_base		= (caddr_t)_rcvbuf0;
-    _rcviov[1].iov_base		= (caddr_t)_rcvbuf1;
-    _rcviov[0].iov_len		= IO_BUF_SIZE;
-    _rcviov[1].iov_len		= IO_BUF_SIZE;
-    _sndiov[0].iov_base		= (caddr_t)_sndbuf0;
-    _sndiov[1].iov_base		= (caddr_t)_sndbuf1;
-    _sndiov[0].iov_len		= 0;
-    _sndiov[1].iov_len		= 0;
-
-    // recvmsg() and sendmsg() related initialization
-#ifndef HOST_OS_WINDOWS
-    switch (family()) {
-    case AF_INET:
-	_rcvmh.msg_name		= (caddr_t)&_from4;
-	_sndmh.msg_name		= (caddr_t)&_to4;
-	_rcvmh.msg_namelen	= sizeof(_from4);
-	_sndmh.msg_namelen	= sizeof(_to4);
-	break;
-#ifdef HAVE_IPV6
-    case AF_INET6:
-	_rcvmh.msg_name		= (caddr_t)&_from6;
-	_sndmh.msg_name		= (caddr_t)&_to6;
-	_rcvmh.msg_namelen	= sizeof(_from6);
-	_sndmh.msg_namelen	= sizeof(_to6);
-	break;
-#endif // HAVE_IPV6
-    default:
-	XLOG_UNREACHABLE();
-	break;
-    }
-    _rcvmh.msg_iov		= _rcviov;
-    _sndmh.msg_iov		= _sndiov;
-    _rcvmh.msg_iovlen		= 1;
-    _sndmh.msg_iovlen		= 1;
-    _rcvmh.msg_control		= (caddr_t)_rcvcmsgbuf;
-    _sndmh.msg_control		= (caddr_t)_sndcmsgbuf;
-    _rcvmh.msg_controllen	= CMSG_BUF_SIZE;
-    _sndmh.msg_controllen	= 0;
-#endif // ! HOST_OS_WINDOWS
-
     //
     // Get the old state from the underlying system
     //
@@ -209,14 +161,6 @@ MfeaMrouter::MfeaMrouter(MfeaNode& mfea_node)
 MfeaMrouter::~MfeaMrouter()
 {
     stop();
-    
-    // Free the buffers
-    delete[] _rcvbuf0;
-    delete[] _sndbuf0;
-    delete[] _rcvbuf1;
-    delete[] _sndbuf1;
-    delete[] _rcvcmsgbuf;
-    delete[] _sndcmsgbuf;
 }
 
 /**
@@ -1275,27 +1219,9 @@ MfeaMrouter::delete_multicast_vif(uint32_t vif_index)
 #else
 	int ret_value = -1;
 
-	//
-	// XXX: In case of Linux, MRT_DEL_VIF expects an argument
-	// of type "struct vifctl", while other systems expect
-	// an argument of type "vifi_t".
-	//
-	// TODO: note that currently (2004/06/09) Linux doesn't support
-	// IPv6 multicast routing, hence the above is a guess based on
-	// the difference in case of IPv4.
-	//
-#ifdef HOST_OS_LINUX
-	struct mif6ctl mc;
-	memset(&mc, 0, sizeof(mc));
-	mc.mif6c_mifi = mfea_vif->vif_index();
-	mc.mif6c_pifi = mfea_vif->pif_index();
-	ret_value = setsockopt(_mrouter_socket, IPPROTO_IP, MRT_DEL_VIF,
-			       (void *)&mc, sizeof(mc));
-#else
 	mifi_t vifi = mfea_vif->vif_index();
 	ret_value = setsockopt(_mrouter_socket, IPPROTO_IPV6, MRT6_DEL_MIF,
 			       (void *)&vifi, sizeof(vifi));
-#endif
 	if (ret_value < 0) {
 	    XLOG_ERROR("setsockopt(MRT6_DEL_MIF, vif %s) failed: %s",
 		       mfea_vif->name().c_str(), strerror(errno));
