@@ -1,19 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/fea/test_fea_rawlink.cc,v 1.4 2008/07/23 05:10:12 pavlin Exp $"
+#ident "$XORP: xorp/fea/test_fea_rawlink.cc,v 1.8 2009/01/05 18:30:49 jtc Exp $"
 
 #include "fea_module.h"
 
@@ -81,7 +86,7 @@ public:
 		IPv4 finder_host, uint16_t finder_port,
 		const string& ifname,
 		const string& vifname,
-		const EtherMac& src,
+		const Mac& src,
 		bool is_sender)
 	: _e(e),
 	  _fea_target_name(fea_target_name),
@@ -179,7 +184,7 @@ public:
      * @param ipg_ms interpacket gap in milliseconds.
      */
     void
-    start_sending(const EtherMac& dest, uint32_t bytes, uint32_t ipg_ms)
+    start_sending(const Mac& dest, uint32_t bytes, uint32_t ipg_ms)
     {
 	_t_snd = _e.new_periodic_ms(ipg_ms,
 				    callback(this,
@@ -198,7 +203,7 @@ public:
 
 protected:
     bool
-    send_data(EtherMac dest, uint32_t bytes)
+    send_data(Mac dest, uint32_t bytes)
     {
 	vector<uint8_t> d(bytes, FILLER_VALUE);
 
@@ -315,9 +320,8 @@ protected:
 	XLOG_ASSERT(if_name == _ifname);
 	XLOG_ASSERT(vif_name == _vifname);
 
-	// If Mac isn't an EtherMac, a BadMac exception will be thrown.
-	XLOG_ASSERT(EtherMac(dst_address) == _src);
-	XLOG_ASSERT(EtherMac(src_address) == _src);
+	XLOG_ASSERT(dst_address == _src);
+	XLOG_ASSERT(src_address == _src);
 	XLOG_ASSERT(ether_type == ETHER_TYPE);
 
 	// Now check the payload.
@@ -338,7 +342,7 @@ private:
     string	_ifname;
     string	_vifname;
 
-    EtherMac	_src;
+    Mac		_src;
     bool	_is_sender;
 
     uint32_t	_p_snd;		// packets sent
@@ -372,7 +376,7 @@ static void
 create_test_link(EventLoop*	pe,
 		 string		ifname,
 		 string		vifname,
-		 EtherMac	src,
+		 Mac		src,
 		 bool		is_sender,
 		 TestRawLink**	ppu)
 {
@@ -399,7 +403,7 @@ register_test_link(TestRawLink** ppu, bool *err)
 }
 
 static void
-start_sending(TestRawLink** ppu, EtherMac addr)
+start_sending(TestRawLink** ppu, Mac addr)
 {
     TestRawLink* pu = *ppu;
     pu->start_sending(addr, 512, 200);
@@ -487,7 +491,13 @@ test_main(IPv4 finder_host, uint16_t finder_port,
     string fea_target_name = "fea";
     vector<XorpTimer> ev;	// Vector for timed events
     bool eflag(false);		// Error flag set by timed events
-    EtherMac mac;
+    Mac mac;
+
+#ifndef HAVE_PCAP
+    fprintf(stdout, "Test Skipped: No L2 I/O mechanism found: "
+	    "HAVE_PCAP is not defined.\n");
+    return 0;
+#endif
 
     //
     // Wait for FEA bringup and pull the IfTree.
