@@ -1,19 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/fib2mrib/fib2mrib_varrw.cc,v 1.10 2008/07/23 05:10:35 pavlin Exp $"
+#ident "$XORP: xorp/fib2mrib/fib2mrib_varrw.cc,v 1.14 2009/01/05 18:30:54 jtc Exp $"
 
 #include "fib2mrib_module.h"
 
@@ -31,14 +36,14 @@ Fib2mribVarRW::Fib2mribVarRW(Fib2mribRoute& route)
 void
 Fib2mribVarRW::start_read()
 {
-    initialize(VAR_POLICYTAGS, _route.policytags().element());
+    initialize(_route.policytags());
 
     if (_is_ipv4) {
 	initialize(VAR_NETWORK4,
 		   _ef.create(ElemIPv4Net::id,
 			      _route.network().str().c_str()));
 	initialize(VAR_NEXTHOP4,
-		   _ef.create(ElemIPv4::id,
+		   _ef.create(ElemIPv4NextHop::id,
 			      _route.nexthop().str().c_str()));
 	
 	initialize(VAR_NETWORK6, NULL);
@@ -50,7 +55,7 @@ Fib2mribVarRW::start_read()
 		   _ef.create(ElemIPv6Net::id,
 			      _route.network().str().c_str()));
 	initialize(VAR_NEXTHOP6,
-		   _ef.create(ElemIPv6::id,
+		   _ef.create(ElemIPv6NextHop::id,
 			      _route.nexthop().str().c_str()));
 
 	initialize(VAR_NETWORK4, NULL);
@@ -65,11 +70,8 @@ Fib2mribVarRW::start_read()
 }
 
 void
-Fib2mribVarRW::single_write(const Id& id, const Element& e)
+Fib2mribVarRW::single_write(const Id& /* id */, const Element& /* e */)
 {
-    if (id == VAR_POLICYTAGS) {
-	_route.set_policytags(e);
-    }
 }
 
 Element*
