@@ -1,18 +1,25 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License, Version
+// 2.1, June 1999 as published by the Free Software Foundation.
+// Redistribution and/or modification of this program under the terms of
+// any other version of the GNU Lesser General Public License is not
+// permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU Lesser General Public License, Version 2.1, a copy of
+// which can be found in the XORP LICENSE.lgpl file.
+// 
+// XORP, Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/libfeaclient/ifmgr_xrl_mirror.cc,v 1.32 2008/07/23 05:10:37 pavlin Exp $"
+#ident "$XORP: xorp/libfeaclient/ifmgr_xrl_mirror.cc,v 1.36 2009/01/05 18:30:55 jtc Exp $"
 
 #include "libxorp/status_codes.h"
 #include "libxorp/eventloop.hh"
@@ -1014,7 +1021,9 @@ IfMgrXrlMirror::IfMgrXrlMirror(EventLoop&	e,
 
     : ServiceBase("FEA Interface Mirror"),
       _e(e), _finder_addr(finder_addr), _finder_port(finder_port),
-      _dispatcher(_iftree), _rtarget(rtarget), _rtr(NULL), _xrl_tgt(NULL)
+      _dispatcher(_iftree), _rtarget(rtarget), _rtr(NULL), _xrl_tgt(NULL),
+      _updates_delay(TimeVal::ZERO())
+
 {
 }
 
@@ -1025,7 +1034,8 @@ IfMgrXrlMirror::IfMgrXrlMirror(EventLoop&	e,
 
     : ServiceBase("FEA Interface Mirror"),
       _e(e), _finder_hostname(finder_hostname), _finder_port(finder_port),
-      _dispatcher(_iftree), _rtarget(rtarget), _rtr(NULL), _xrl_tgt(NULL)
+      _dispatcher(_iftree), _rtarget(rtarget), _rtr(NULL), _xrl_tgt(NULL),
+      _updates_delay(TimeVal::ZERO())
 {
 }
 
@@ -1120,8 +1130,30 @@ IfMgrXrlMirror::tree_complete()
     }
 }
 
+void
+IfMgrXrlMirror::delay_updates(const TimeVal& delay)
+{
+    _updates_delay = delay;
+}
+
 void
 IfMgrXrlMirror::updates_made()
+{
+    if (_updates_delay.is_zero()) {
+	do_updates();
+	return;
+    }
+
+    if (_updates_timer.scheduled())
+	return;
+
+    _updates_timer = _e.new_oneoff_after(
+	_updates_delay,
+	callback(this, &IfMgrXrlMirror::do_updates));
+}
+
+void
+IfMgrXrlMirror::do_updates()
 {
     if (status() & ServiceStatus(SERVICE_SHUTTING_DOWN
 				 | SERVICE_SHUTDOWN
