@@ -1,18 +1,25 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2006-2008 XORP, Inc.
+// Copyright (c) 2006-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License, Version
+// 2.1, June 1999 as published by the Free Software Foundation.
+// Redistribution and/or modification of this program under the terms of
+// any other version of the GNU Lesser General Public License is not
+// permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU Lesser General Public License, Version 2.1, a copy of
+// which can be found in the XORP LICENSE.lgpl file.
+// 
+// XORP, Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/libproto/packet.cc,v 1.6 2008/07/23 05:10:39 pavlin Exp $"
+#ident "$XORP: xorp/libproto/packet.cc,v 1.16 2009/01/05 18:30:55 jtc Exp $"
 
 
 //
@@ -160,10 +167,8 @@ IpHeader4::fragment(size_t mtu, list<vector<uint8_t> >& fragments,
 	first_ip.set_ip_off(first_ip.ip_off() | IpHeader4::FRAGMENT_FLAGS_IP_MF);
 	first_ip.set_ip_len(first_frag_len);
 	first_ip.set_ip_sum(0);
-	if (do_checksum) {
-	    first_ip.set_ip_sum(ntohs(inet_checksum(first_ip.data(),
-						    first_ip_hl)));
-	}
+	if (do_checksum)
+	    first_ip.compute_checksum();
 
 	// Add the first fragment
 	fragments.push_back(first_frag);
@@ -208,10 +213,8 @@ IpHeader4::fragment(size_t mtu, list<vector<uint8_t> >& fragments,
 	}
 
 	frag_ip4.set_ip_sum(0);
-	if (do_checksum) {
-	    frag_ip4.set_ip_sum(ntohs(inet_checksum(frag_ip4.data(),
-						    frag_ip_hl)));
-	}
+	if (do_checksum)
+	    frag_ip4.compute_checksum();
 
 	// Add the fragment
 	fragments.push_back(frag_buf);
@@ -221,3 +224,148 @@ IpHeader4::fragment(size_t mtu, list<vector<uint8_t> >& fragments,
 
     return (XORP_OK);
 }
+
+void
+IpHeader4Writer::compute_checksum()
+{
+    set_ip_sum(0);
+    set_ip_sum(ntohs(inet_checksum(data(), ip_header_len())));
+}
+
+ArpHeader&
+ArpHeader::assign(uint8_t* data)
+{
+    ArpHeader* h = reinterpret_cast<ArpHeader*>(data);
+
+    memset(h, 0, sizeof(*h));
+
+    return *h;
+}
+
+const ArpHeader&
+ArpHeader::assign(const PAYLOAD& payload)
+{
+    const ArpHeader* h = reinterpret_cast<const ArpHeader*>(&payload[0]);
+
+    unsigned size = sizeof(*h);
+
+    if (payload.size() < size)
+	xorp_throw(BadPacketException, "ARP packet too small");
+
+    size += h->ah_hw_len * 2 + h->ah_proto_len * 2;
+
+    if (payload.size() < size)
+	xorp_throw(BadPacketException, "ARP packet too small");
+
+    return *h;
+}
+
+bool
+ArpHeader::is_request() const
+{
+    return ntohs(ah_op) == ARP_REQUEST;
+}
+
+IPv4
+ArpHeader::get_request() const
+{
+    if (!is_request())
+	xorp_throw(BadPacketException, "Not an ARP request");
+
+    if (ntohs(ah_proto_fmt) != ETHERTYPE_IP)
+	xorp_throw(BadPacketException, "Not an IPv4 ARP");
+
+    IPv4 ip;
+
+    ip.copy_in(&ah_data[ah_hw_len * 2 + ah_proto_len]);
+
+    return ip;
+}
+
+void
+ArpHeader::make_reply(PAYLOAD& out, const Mac& mac) const
+{
+    // sanity checks
+    if (!is_request())
+	xorp_throw(BadPacketException, "Not an ARP request");
+
+    if (ntohs(ah_hw_fmt) != HW_ETHER)
+	xorp_throw(BadPacketException, "Not an ethernet ARP");
+
+    // allocate size
+    int size = sizeof(*this) + ah_hw_len * 2 + ah_proto_len * 2;
+    out.reserve(size);
+    out.resize(size);
+
+    // copy request into reply
+    memcpy(&out[0], this, size);
+
+    // make it a reply
+    ArpHeader& reply = const_cast<ArpHeader&>(ArpHeader::assign(out));
+    reply.ah_op = htons(ARP_REPLY);
+
+    // set the destination
+    size = ah_hw_len + ah_proto_len;
+    memcpy(&reply.ah_data[size], reply.ah_data, size);
+
+    // set the source
+    mac.copy_out(reply.ah_data);
+    size += ah_hw_len;
+    memcpy(&reply.ah_data[ah_hw_len], &ah_data[size], ah_proto_len);
+}
+
+void
+ArpHeader::set_sender(const Mac& mac, const IPv4& ip)
+{
+    ah_hw_fmt = htons(HW_ETHER);
+    ah_hw_len = mac.copy_out(ah_data);
+
+    ah_proto_fmt = htons(ETHERTYPE_IP);
+    ah_proto_len = ip.copy_out(&ah_data[ah_hw_len]);
+}
+
+void
+ArpHeader::set_request(const IPv4& ip)
+{
+    XLOG_ASSERT(ah_op == 0);
+    XLOG_ASSERT(ah_proto_fmt == htons(ETHERTYPE_IP));
+
+    ah_op = htons(ARP_REQUEST);
+
+    ip.copy_out(&ah_data[ah_hw_len * 2 + ah_proto_len]);
+}
+
+void
+ArpHeader::set_reply(const Mac& mac, const IPv4& ip)
+{
+    XLOG_ASSERT(ah_op == 0);
+    XLOG_ASSERT(ah_hw_fmt    == htons(HW_ETHER));
+    XLOG_ASSERT(ah_proto_fmt == htons(ETHERTYPE_IP));
+
+    set_request(ip);
+    ah_op = htons(ARP_REPLY);
+
+    mac.copy_out(&ah_data[ah_hw_len + ah_proto_len]);
+}
+
+uint32_t
+ArpHeader::size() const
+{
+    return sizeof(*this) + ah_hw_len * 2 + ah_proto_len * 2;
+}
+
+void
+ArpHeader::make_gratuitous(PAYLOAD& data, const Mac& mac, const IPv4& ip)
+{
+    uint32_t sz = sizeof(ArpHeader) + 6 * 2 + 4 * 2;
+
+    data.resize(sz, 0);
+
+    ArpHeader& arp = ArpHeader::assign(&data[0]);
+
+    arp.set_sender(mac, ip);
+    arp.set_request(ip);
+
+    XLOG_ASSERT(arp.size() <= data.capacity());
+    data.resize(arp.size());
+}
