@@ -1,18 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2006-2008 XORP, Inc.
+// Copyright (c) 2006-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License, Version
+// 2.1, June 1999 as published by the Free Software Foundation.
+// Redistribution and/or modification of this program under the terms of
+// any other version of the GNU Lesser General Public License is not
+// permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU Lesser General Public License, Version 2.1, a copy of
+// which can be found in the XORP LICENSE.lgpl file.
+// 
+// XORP, Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/libproto/test_packet.cc,v 1.5 2008/07/23 05:10:40 pavlin Exp $"
+#ident "$XORP: xorp/libproto/test_packet.cc,v 1.8 2009/01/05 18:30:55 jtc Exp $"
 
 #include "libproto_module.h"
 #include "libxorp/xorp.h"
@@ -160,7 +166,7 @@ test_ipv4_header(TestInfo& test_info)
     uint16_t ip_fragment_flags = 0xe000;
     uint8_t ip_ttl = 64;
     uint8_t ip_p = 100;
-    uint16_t ip_sum = 0xf123;
+    uint16_t ip_sum = 0x7acd;
     IPv4 ip_src("1.2.3.4");
     IPv4 ip_dst("5.6.7.8");
 
@@ -245,6 +251,10 @@ test_ipv4_header(TestInfo& test_info)
     // Test the IPv4 packet header checksum
     //
     verbose_assert(iph.ip_sum() == ip_sum, "IPv4 checksum");
+    // Set the checksum to garbage and compute it again
+    iphw.set_ip_sum(0xffff);
+    iphw.compute_checksum();
+    verbose_assert(iph.ip_sum() == ip_sum, "IPv4 checksum");
 
     //
     // Test the IPv4 source and destination address
