@@ -1,18 +1,25 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License, Version
+// 2.1, June 1999 as published by the Free Software Foundation.
+// Redistribution and/or modification of this program under the terms of
+// any other version of the GNU Lesser General Public License is not
+// permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU Lesser General Public License, Version 2.1, a copy of
+// which can be found in the XORP LICENSE.lgpl file.
+// 
+// XORP, Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/libxipc/xrl_args.cc,v 1.18 2008/07/23 05:10:44 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/xrl_args.cc,v 1.33 2009/01/05 18:30:56 jtc Exp $"
 
 #include "xrl_module.h"
 
@@ -20,6 +27,8 @@
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 
+#include "libproto/packet.hh"
+
 #include <functional>
 #include <stdexcept>
 
@@ -32,12 +41,18 @@
 XrlArgs&
 XrlArgs::add(const XrlAtom& xa) throw (XrlAtomFound)
 {
-    const_iterator p;
-    for (p = _args.begin(); p != _args.end(); ++p) {
-	if (p->name() == xa.name()) {
-	    throw XrlAtomFound();
+    if (!xa.name().empty()) {
+	const_iterator p;
+
+	for (p = _args.begin(); p != _args.end(); ++p) {
+	    if (p->name() == xa.name()) {
+		throw XrlAtomFound();
+	    }
 	}
+
+	_have_name = true;
     }
+
     _args.push_back(xa);
     return *this;
 }
@@ -56,6 +71,22 @@ XrlArgs::get(const XrlAtom& dataless) const throw (XrlAtomNotFound)
     return *p;
 }
 
+const XrlAtom&
+XrlArgs::get(unsigned idx, const char* name) const throw (XrlAtomNotFound)
+{
+    if (!_have_name)
+	return _args[idx];
+
+    for (const_iterator i = _args.begin(); i != _args.end(); ++i) {
+	const XrlAtom& a = *i;
+
+	if (a.name().compare(name) == 0)
+	    return a;
+    }
+
+    throw XrlAtomNotFound();
+}
+
 void
 XrlArgs::remove(const XrlAtom& dataless) throw (XrlAtomNotFound)
 {
@@ -554,14 +585,7 @@ XrlArgs::operator==(const XrlArgs& t) const
 const XrlAtom&
 XrlArgs::operator[](uint32_t index) const
 {
-    const_iterator ai = _args.begin();
-    while (index != 0 && ai != _args.end()) {
-        index--;
-    }
-    if (ai == _args.end()) {
-        throw out_of_range("XrlArgs");
-    }
-    return *ai;
+    return _args[index];
 }
 
 const XrlAtom&
@@ -601,6 +625,7 @@ XrlArgs::str() const
 }
 
 XrlArgs::XrlArgs(const char* serialized) throw (InvalidString)
+		: _have_name(false)
 {
     string s(serialized);
 
@@ -631,9 +656,13 @@ static const uint32_t PACKING_CHECK_CODE = 0xcc;
 static const uint32_t PACKING_MAX_COUNT	 = 0x00ffffff;
 
 size_t
-XrlArgs::packed_bytes() const
+XrlArgs::packed_bytes(XrlAtom* head) const
 {
     size_t total_bytes = 0;
+
+    if (head)
+	total_bytes += head->packed_bytes();
+
     for (const_iterator ci = _args.begin(); ci != _args.end(); ++ci) {
 	total_bytes += ci->packed_bytes();
     }
@@ -641,7 +670,7 @@ XrlArgs::packed_bytes() const
 }
 
 size_t
-XrlArgs::pack(uint8_t* buffer, size_t buffer_bytes) const
+XrlArgs::pack(uint8_t* buffer, size_t buffer_bytes, XrlAtom* head) const
 {
     size_t total_bytes = 0;
 
@@ -652,6 +681,9 @@ XrlArgs::pack(uint8_t* buffer, size_t buffer_bytes) const
 
     // Pack header
     uint32_t cnt = _args.size();
+    if (head)
+	cnt++;
+
     if (cnt > PACKING_MAX_COUNT) {
 	// log a message, bounds hit
 	return 0;
@@ -662,10 +694,21 @@ XrlArgs::pack(uint8_t* buffer, size_t buffer_bytes) const
     memcpy(buffer, &header, sizeof(header));
     total_bytes += sizeof(header);
 
+    if (head) {
+	size_t atom_bytes = head->pack(buffer + total_bytes,
+				       buffer_bytes - total_bytes);
+
+	if (atom_bytes == 0)
+	    return 0;
+
+	total_bytes += atom_bytes;
+    }
+
     // Pack atoms
     for (const_iterator ci = _args.begin(); ci != _args.end(); ++ci) {
 	size_t atom_bytes = ci->pack(buffer + total_bytes,
 				     buffer_bytes - total_bytes);
+
 	if (atom_bytes == 0) {
 	    return 0;
 	}
@@ -675,34 +718,35 @@ XrlArgs::pack(uint8_t* buffer, size_t buffer_bytes) const
 }
 
 size_t
-XrlArgs::unpack(const uint8_t* buffer, size_t buffer_bytes)
+XrlArgs::unpack(const uint8_t* buffer, size_t buffer_bytes, XrlAtom* head)
 {
-    // Unpack header
-    if (buffer_bytes < 4)
+    uint32_t cnt;
+    int added = 0;
+    size_t used_bytes = unpack_header(cnt, buffer, buffer_bytes);
+    XrlAtom* atom;
+    _have_name = false;
+
+    if (!used_bytes)
 	return 0;
 
-    uint32_t header;
-    memcpy(&header, buffer, sizeof(header));
-    header = ntohl(header);
-
-    // Check header sanity
-    if ((header >> 24) != PACKING_CHECK_CODE) {
-	return 0;
-    }
-
-    uint32_t cnt = header & PACKING_MAX_COUNT;
-    size_t used_bytes = sizeof(header);
-    list<XrlAtom> atoms;
-
     while (cnt != 0) {
-	atoms.push_back(XrlAtom());
-	XrlAtom& atom = atoms.back();
-	size_t atom_bytes = atom.unpack(buffer + used_bytes,
-					buffer_bytes - used_bytes);
-	if (atom_bytes == 0) {
-	    return 0;
+	if (head) {
+	    atom = head;
+	    head = NULL;
+	} else {
+	    _args.push_back(XrlAtom());
+	    atom = &_args.back();
+	    added++;
 	}
 
+	size_t atom_bytes = atom->unpack(buffer + used_bytes,
+					 buffer_bytes - used_bytes);
+	if (atom_bytes == 0)
+	    goto __error;
+
+	if (!_have_name && !atom->name().empty())
+	    _have_name = true;
+
 	used_bytes += atom_bytes;
 	--cnt;
 	if (used_bytes >= buffer_bytes) {
@@ -711,12 +755,57 @@ XrlArgs::unpack(const uint8_t* buffer, size_t buffer_bytes)
 	    break;
 	}
     }
-    if (cnt != 0) {
+    if (cnt != 0)
 	// Unexpected truncation
+	goto __error;
+
+    return used_bytes;
+
+__error:
+    while (added--)
+	_args.pop_back();
+
+    return 0;
+}
+
+size_t
+XrlArgs::unpack_header(uint32_t& cnt, const uint8_t* in, size_t len)
+{
+    // Unpack header
+    if (len < 4)
 	return 0;
+
+    uint32_t header = extract_32(in);
+
+    // Check header sanity
+    if ((header >> 24) != PACKING_CHECK_CODE)
+	return 0;
+
+    cnt = header & PACKING_MAX_COUNT;
+
+    return sizeof(header);
+}
+
+size_t
+XrlArgs::fill(const uint8_t* in, size_t len)
+{
+    size_t tot = len;
+    _have_name = false;
+
+    for (ATOMS::iterator i = _args.begin(); i != _args.end(); ++i) {
+	XrlAtom& atom = *i;
+
+	size_t sz = atom.unpack(in, len);
+	if (sz == 0)
+	    return 0;
+
+	if (!_have_name && !atom.name().empty())
+	    _have_name = true;
+
+	XLOG_ASSERT(sz <= len);
+	in  += sz;
+	len -= sz;
     }
 
-    // Append atoms unpacked to existing atoms list
-    _args.splice(_args.end(), atoms);
-    return used_bytes;
+    return tot - len;
 }
