@@ -1,19 +1,25 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License, Version
+// 2.1, June 1999 as published by the Free Software Foundation.
+// Redistribution and/or modification of this program under the terms of
+// any other version of the GNU Lesser General Public License is not
+// permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU Lesser General Public License, Version 2.1, a copy of
+// which can be found in the XORP LICENSE.lgpl file.
+// 
+// XORP, Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/libxipc/xrl_atom.cc,v 1.32 2008/07/23 05:10:44 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/xrl_atom.cc,v 1.47 2009/01/05 18:30:56 jtc Exp $"
 
 #include "xrl_module.h"
 
@@ -22,6 +28,8 @@
 #include "libxorp/c_format.hh"
 #include "libxorp/xlog.h"
 
+#include "libproto/packet.hh"
+
 #include <map>
 #include <string>
 
@@ -363,6 +371,7 @@ XrlAtom::copy(const XrlAtom& xa)
 
     _type = xa._type;
     _have_data = xa._have_data;
+    _own = true;
 
     if (_have_data) {
         switch (_type) {
@@ -416,7 +425,7 @@ XrlAtom::copy(const XrlAtom& xa)
 void
 XrlAtom::discard_dynamic()
 {
-    if (_have_data) {
+    if (_own && _have_data) {
         switch (_type) {
         case xrlatom_no_type:
 	case xrlatom_boolean:
@@ -482,7 +491,9 @@ XrlAtom::str() const
 }
 
 XrlAtom::XrlAtom(const char* serialized) throw (InvalidString, BadName)
-    : _type(xrlatom_no_type), _have_data(false)
+    : _type(xrlatom_no_type),
+      _have_data(false),
+      _own(true)
 {
 
     const char *start, *sep;
@@ -522,7 +533,9 @@ XrlAtom::XrlAtom(const char* serialized) throw (InvalidString, BadName)
 
 XrlAtom::XrlAtom(const string& name, XrlAtomType t,
 		 const string& serialized_data) throw (InvalidString)
-    : _type(t)
+    : _type(t),
+      _have_data(false),
+      _own(true)
 {
     set_name(name);
     ssize_t bad_pos = data_from_c_str(serialized_data.c_str());
@@ -532,7 +545,9 @@ XrlAtom::XrlAtom(const string& name, XrlAtomType t,
 
 XrlAtom::XrlAtom(const char* name, XrlAtomType t,
 		 const string& serialized_data) throw (InvalidString)
-    : _type(t)
+    : _type(t),
+      _have_data(false),
+      _own(true)
 {
     set_name(name);
     ssize_t bad_pos = data_from_c_str(serialized_data.c_str());
@@ -816,7 +831,24 @@ XrlAtom::unpack_name(const uint8_t* buffer, size_t buffer_bytes)
 	return 0;
     }
     const char* s = reinterpret_cast<const char*>(buffer + sizeof(sz));
-    set_name(string(s, sz).c_str());
+
+    // if we're recycling the atom, just make sure the name is the same
+    int name_size = _atom_name.size();
+
+    if (name_size) {
+	if (name_size != sz)
+	    xorp_throw(BadName, s);
+
+	if (::memcmp(_atom_name.c_str(), s, name_size) != 0)
+	    xorp_throw(BadName, s);
+
+    } else {
+	_atom_name.assign(s, sz);
+
+	if (!valid_name(_atom_name))
+	    xorp_throw(BadName, s);
+    }
+
     return sizeof(sz) + sz;
 }
 
@@ -860,8 +892,14 @@ size_t
 XrlAtom::unpack_ipv4(const uint8_t* b)
 {
     uint32_t a;
-    memcpy(&a, b, sizeof(a));
-    _ipv4 = new IPv4(a);
+
+    // XXX ntohl ?
+    if (_type == xrlatom_no_type) {
+	memcpy(&a, b, sizeof(a));
+	_ipv4 = new IPv4(a);
+    } else
+	_ipv4->copy_in(b);
+
     return sizeof(a);
 }
 
@@ -880,7 +918,12 @@ XrlAtom::unpack_ipv4net(const uint8_t* b)
     uint32_t a;
     memcpy(&a, b, sizeof(a));
     IPv4 v(a);
-    _ipv4net = new IPv4Net(v, b[sizeof(a)]);
+
+    if (_type == xrlatom_no_type)
+	_ipv4net = new IPv4Net(v, b[sizeof(a)]);
+    else
+	*_ipv4net = IPv4Net(v, b[sizeof(a)]);
+
     return sizeof(a) + sizeof(uint8_t);
 }
 
@@ -896,8 +939,13 @@ size_t
 XrlAtom::unpack_ipv6(const uint8_t* buffer)
 {
     uint32_t a[4];
-    memcpy(a, buffer, sizeof(a));
-    _ipv6 = new IPv6(a);
+    
+    if (_type == xrlatom_no_type) {
+	memcpy(a, buffer, sizeof(a));
+	_ipv6 = new IPv6(a);
+    } else
+	_ipv6->copy_in(buffer);
+
     return sizeof(a);
 }
 
@@ -916,7 +964,12 @@ XrlAtom::unpack_ipv6net(const uint8_t* buffer)
     uint32_t a[4];
     memcpy(a, buffer, sizeof(a));
     IPv6 v(a);
-    _ipv6net = new IPv6Net(v, buffer[sizeof(a)]);
+
+    if (_type == xrlatom_no_type)
+	_ipv6net = new IPv6Net(v, buffer[sizeof(a)]);
+    else
+	*_ipv6net = IPv6Net(v, buffer[sizeof(a)]);
+
     return sizeof(a) + sizeof(uint8_t);
 }
 
@@ -948,7 +1001,12 @@ XrlAtom::unpack_mac(const uint8_t* buffer, size_t buffer_bytes)
     }
     const char* text = reinterpret_cast<const char*>(buffer + sizeof(len));
     try {
-	_mac = new Mac(string(text, len));
+	string s(text, len);
+
+	if (_type == xrlatom_no_type)
+	    _mac = new Mac(s.c_str());
+	else
+	    _mac->copy_in(s.c_str());
     }
     catch (const InvalidString&) {
 	_mac = 0;
@@ -986,10 +1044,46 @@ XrlAtom::unpack_text(const uint8_t* buffer, size_t buffer_bytes)
 	return 0;
     }
     const char *text = reinterpret_cast<const char*>(buffer + sizeof(len));
-    _text = new string(text, len);
+
+    if (_type == xrlatom_no_type)
+	_text = new string(text, len);
+    else
+	_text->assign(text, len);
+
     return sizeof(len) + len;
 }
 
+size_t
+XrlAtom::peek_text(const char*& t, uint32_t& tl, const uint8_t* buf, size_t len)
+{
+    // XrlAtom header
+    if (len == 0)
+	return 0;
+
+    if (*buf != (xrlatom_text | DATA_PRESENT)) {
+	return 0;
+    }
+
+    buf++;
+    len--;
+
+    // Text header
+    if (len < sizeof(tl))
+	return 0;
+
+    tl = extract_32(buf);
+
+    buf += sizeof(tl);
+    len -= sizeof(tl);
+
+    if (len < tl)
+	return 0;
+
+    t = reinterpret_cast<const char*>(buf);
+
+    return 1 + sizeof(tl) + tl;
+}
+
 size_t
 XrlAtom::pack_list(uint8_t* buffer, size_t buffer_bytes) const
 {
@@ -1023,11 +1117,12 @@ XrlAtom::unpack_list(const uint8_t* buffer, size_t buffer_bytes)
     nelem = ntohl(nelem);
     used += sizeof(nelem);
 
-    _list = new XrlAtomList;
+    // check if we're overwriting
+    if (_type == xrlatom_no_type)
+	_list = new XrlAtomList;
 
     for (size_t i = 0; i < nelem; i++) {
-	XrlAtom tmp;
-	size_t unpacked = tmp.unpack(buffer + used, buffer_bytes - used);
+	size_t unpacked = _list->modify(i, buffer + used, buffer_bytes - used);
 	if (unpacked == 0) {
 	    // Failed to unpack item
 	    delete _list;
@@ -1036,8 +1131,9 @@ XrlAtom::unpack_list(const uint8_t* buffer, size_t buffer_bytes)
 	}
 	used += unpacked;
 	assert(used <= buffer_bytes);
-	_list->append(tmp);
     }
+    _list->set_size(nelem);
+
     return used;
 }
 
@@ -1066,6 +1162,10 @@ XrlAtom::unpack_binary(const uint8_t* buffer, size_t buffer_bytes)
 	_binary = 0;
 	return 0;
     }
+    
+    if (_type != xrlatom_no_type)
+	delete _binary;
+
     _binary = new vector<uint8_t>(buffer + sizeof(len),
 				  buffer + sizeof(len) + len);
     return sizeof(len) + len;
@@ -1169,7 +1269,9 @@ XrlAtom::unpack(const uint8_t* buffer, size_t buffer_bytes)
 
     if (header & NAME_PRESENT) {
 	try {
-	    size_t used = unpack_name(buffer + unpacked, buffer_bytes - unpacked);
+	    size_t used = unpack_name(buffer + unpacked,
+				      buffer_bytes - unpacked);
+
 	    if (used == 0) {
 		debug_msg("Invalid name\n");
 		return 0;
@@ -1179,14 +1281,17 @@ XrlAtom::unpack(const uint8_t* buffer, size_t buffer_bytes)
 	    debug_msg("Unpacking failed:\n%s\n", bn.str().c_str());
 	    return 0;
 	}
-    }
+    } else
+	_atom_name.clear();
 
     if (header & DATA_PRESENT) {
 	int t = header & ~(NAME_PRESENT | DATA_PRESENT);
 	if (t < xrlatom_start || t > xrlatom_end) {
 	    debug_msg("Type %d invalid\n", t);
 	}
-	_type = XrlAtomType(t);
+
+	XrlAtomType old_type = _type;
+	XrlAtomType type = _type = XrlAtomType(t);
 	_have_data = true;
 
 	debug_msg("Unpacked %u remain %u\n",
@@ -1199,13 +1304,16 @@ XrlAtom::unpack(const uint8_t* buffer, size_t buffer_bytes)
 		      XORP_UINT_CAST(buffer_bytes),
 		      XORP_UINT_CAST(packed_bytes()), _type);
 	    _have_data = false;
+	    _type = old_type;
 	    return 0;
 	}
+	_type = old_type;
 
 	// unpack
 	size_t used = 0;
-	switch (_type) {
+	switch (type) {
 	case xrlatom_no_type:
+	    _type = old_type;
 	    return 0;
 	case xrlatom_boolean:
 	    used = unpack_boolean(buffer + unpacked);
@@ -1245,8 +1353,10 @@ XrlAtom::unpack(const uint8_t* buffer, size_t buffer_bytes)
 
 	    // ... Your type here ...
 	}
+	_type = type;
 
 	if (used == 0) {
+	    _type = xrlatom_no_type;
 	    debug_msg("Blow! Data unpacking failed\n");
 	    _have_data = 0;
 	    return 0;
@@ -1286,3 +1396,14 @@ XrlAtom::valid_type(const string& s)
     XrlAtomType t = resolve_xrlatom_name(s.c_str());
     return (t != xrlatom_no_type);
 }
+
+void
+XrlAtom::abandon_data()
+{
+    bool have_data = _have_data;
+
+    discard_dynamic();
+
+    _own       = false;
+    _have_data = have_data;
+}
