@@ -1,19 +1,25 @@
-
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License, Version
+// 2.1, June 1999 as published by the Free Software Foundation.
+// Redistribution and/or modification of this program under the terms of
+// any other version of the GNU Lesser General Public License is not
+// permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU Lesser General Public License, Version 2.1, a copy of
+// which can be found in the XORP LICENSE.lgpl file.
+// 
+// XORP, Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/libxipc/xrl_pf_factory.cc,v 1.19 2008/07/23 05:10:46 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/xrl_pf_factory.cc,v 1.23 2009/01/05 18:30:56 jtc Exp $"
 
 #include "xrl_module.h"
 
@@ -28,6 +34,7 @@
 #include "xrl_pf_sudp.hh"
 #include "xrl_pf_stcp.hh"
 #include "xrl_pf_kill.hh"
+#include "xrl_pf_unix.hh"
 
 
 // STCP senders are a special case.  Constructing an STCP sender has
@@ -50,7 +57,8 @@ XrlPFSenderFactory::create_sender(EventLoop&	eventloop,
 	    return new XrlPFInProcSender(eventloop, address);
 	} else if (strcmp(XrlPFKillSender::protocol_name(), protocol) == 0) {
 	    return new XrlPFKillSender(eventloop, address);
-	}
+	} else if (strcmp(XrlPFUNIXSender::protocol_name(), protocol) == 0)
+	    return new XrlPFUNIXSender(eventloop, address);
     } catch (XorpException& e) {
 	XLOG_ERROR("XrlPFSenderFactory::create failed: %s\n", e.str().c_str());
     }
