@@ -1,20 +1,27 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License, Version
+// 2.1, June 1999 as published by the Free Software Foundation.
+// Redistribution and/or modification of this program under the terms of
+// any other version of the GNU Lesser General Public License is not
+// permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU Lesser General Public License, Version 2.1, a copy of
+// which can be found in the XORP LICENSE.lgpl file.
+// 
+// XORP, Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
 //#define DEBUG_LOGGING
 
-#ident "$XORP: xorp/libxipc/xrl_pf_stcp.cc,v 1.62 2008/07/23 05:10:46 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/xrl_pf_stcp.cc,v 1.73 2009/01/05 18:30:56 jtc Exp $"
 
 #include "libxorp/xorp.h"
 
@@ -52,7 +59,8 @@
 const char* XrlPFSTCPSender::_protocol   = "stcp";
 const char* XrlPFSTCPListener::_protocol = "stcp";
 
-static const uint32_t 	DEFAULT_SENDER_KEEPALIVE_MS = 10000;
+static const TimeVal 	DEFAULT_SENDER_KEEPALIVE_TIME = TimeVal(10, 0);
+static const TimeVal	QUIET_LIFE_TIME = TimeVal(180, 0);   // 3 min
 
 // The maximum number of bytes worth of XRL buffered before send()
 // returns false.  Resource preservation.
@@ -86,11 +94,11 @@ public:
 	_responses_size(0)
     {
 	EventLoop& e = _parent.eventloop();
-	_life_timer = e.new_oneoff_after_ms(QUIET_LIFE_MS,
-					    callback(this,
-						     &STCPRequestHandler::die,
-						     "life timer expired",
-						     true));
+	_life_timer = e.new_oneoff_after(QUIET_LIFE_TIME,
+					 callback(this,
+						  &STCPRequestHandler::die,
+						  "life timer expired",
+						  true));
 	_reader.start();
 	debug_msg("STCPRequestHandler (%p) fd = %s\n",
 		  this, sock.str().c_str());
@@ -107,7 +115,8 @@ public:
 	_sock.clear();
     }
 
-    void dispatch_request(uint32_t seqno, const uint8_t* buffer, size_t bytes);
+    void dispatch_request(uint32_t seqno, bool batch, const uint8_t* buffer,
+			  size_t bytes);
     void ack_helo(uint32_t seqno);
 
     void read_event(BufferedAsyncReader* 	reader,
@@ -127,6 +136,9 @@ public:
     bool response_pending() const;
 
 private:
+    XrlError do_dispatch(const uint8_t* packed_xrl, size_t packed_xrl_bytes,
+                         XrlArgs& response);
+
     XrlPFSTCPListener& _parent;
     XorpFd _sock;
 
@@ -145,8 +157,6 @@ private:
 
     void parse_header(const uint8_t* buffer, size_t buffer_bytes);
     void parse_payload();
-
-    static const int QUIET_LIFE_MS = 180 * 1000;	// 3 minutes
 };
 
 void
@@ -196,7 +206,8 @@ STCPRequestHandler::read_event(BufferedAsyncReader*		/* source */,
 	    uint8_t* xrl_data = buffer;
 	    xrl_data += STCPPacketHeader::header_size() + sph.error_note_bytes();
 	    size_t   xrl_data_bytes = sph.payload_bytes();
-	    dispatch_request(sph.seqno(), xrl_data, xrl_data_bytes);
+	    dispatch_request(sph.seqno(), sph.batch(),
+			     xrl_data, xrl_data_bytes);
 	    _reader.dispose(sph.frame_bytes());
 	    buffer += sph.frame_bytes();
 	    buffer_bytes -= sph.frame_bytes();
@@ -212,36 +223,64 @@ STCPRequestHandler::read_event(BufferedAsyncReader*		/* source */,
     _reader.set_trigger_bytes(STCPPacketHeader::header_size());
 }
 
-void
-STCPRequestHandler::dispatch_request(uint32_t 		seqno,
-				     const uint8_t* 	packed_xrl,
-				     size_t 		packed_xrl_bytes)
+XrlError
+STCPRequestHandler::do_dispatch(const uint8_t* packed_xrl,
+			        size_t packed_xrl_bytes,
+			        XrlArgs& response)
 {
+    static XrlError e(XrlError::INTERNAL_ERROR().error_code(), "corrupt xrl");
+
     const XrlDispatcher* d = _parent.dispatcher();
     assert(d != 0);
 
-    Xrl xrl;
-    bool unpack_failed = false;
+    string command;
+    size_t cmdsz = Xrl::unpack_command(command, packed_xrl, packed_xrl_bytes);
+    if (!cmdsz)
+	return e;
+
+    XrlDispatcher::XI* xi = d->lookup_xrl(command);
+    if (!xi)
+	return e;
+
+    Xrl& xrl = xi->_xrl;
+
     try {
-	if (xrl.unpack(packed_xrl, packed_xrl_bytes) != packed_xrl_bytes)
-	    unpack_failed = true;
+	if (xi->_new) {
+	    if (xrl.unpack(packed_xrl, packed_xrl_bytes) != packed_xrl_bytes)
+		return e;
+
+	    xi->_new = false;
+	} else {
+	    packed_xrl       += cmdsz;
+	    packed_xrl_bytes -= cmdsz;
+
+	    if (xrl.fill(packed_xrl, packed_xrl_bytes) != packed_xrl_bytes)
+		return e;
+	}
     } catch (...) {
-	unpack_failed = true;
+	return e;
     }
 
-    XrlError e;
+    return d->dispatch_xrl_fast(*xi, response);
+}
+
+void
+STCPRequestHandler::dispatch_request(uint32_t 		seqno,
+				     bool		batch,
+				     const uint8_t* 	packed_xrl,
+				     size_t 		packed_xrl_bytes)
+{
     XrlArgs response;
+    XrlError e;
 
-    if (unpack_failed == false) {
-	e = d->dispatch_xrl(xrl.command(), xrl.args(), response);
-    } else {
-	e = XrlError(XrlError::INTERNAL_ERROR().error_code(), "corrupt xrl");
-    }
+    e = do_dispatch(packed_xrl, packed_xrl_bytes, response);
 
     size_t xrl_response_bytes = response.packed_bytes();
     size_t note_bytes = e.note().size();
 
-    _responses.push_back(ReplyPacket(STCPPacketHeader::header_size() + note_bytes + xrl_response_bytes));
+    _responses.push_back(ReplyPacket(STCPPacketHeader::header_size()
+			 + note_bytes + xrl_response_bytes));
+
     _responses_size++;
     ReplyPacket& r = _responses.back();
 
@@ -262,7 +301,7 @@ STCPRequestHandler::dispatch_request(uint32_t 		seqno,
 		       callback(this, &STCPRequestHandler::update_writer));
 
     debug_msg("about to start_writer (%d)\n", _responses.size() != 0);
-    if (_writer.running() == false) {
+    if (!batch && _writer.running() == false) {
 	_writer.start();
     }
 }
@@ -328,7 +367,7 @@ STCPRequestHandler::update_writer(AsyncFileWriter::Event ev,
 void
 STCPRequestHandler::postpone_death()
 {
-    _life_timer.schedule_after_ms(QUIET_LIFE_MS);
+    _life_timer.schedule_after(QUIET_LIFE_TIME);
 }
 
 void
@@ -382,6 +421,11 @@ XrlPFSTCPListener::XrlPFSTCPListener(EventLoop&	    e,
 			     callback(this, &XrlPFSTCPListener::connect_hook));
 }
 
+XrlPFSTCPListener::XrlPFSTCPListener(EventLoop* e, XrlDispatcher* x)
+    : XrlPFListener(*e, x)
+{
+}
+
 XrlPFSTCPListener::~XrlPFSTCPListener()
 {
     while (_request_handlers.empty() == false) {
@@ -454,53 +498,73 @@ public:
 public:
     RequestState(XrlPFSTCPSender* p,
 		 uint32_t	  sn,
+		 bool		  batch,
 		 const Xrl&	  x,
 		 const Callback&  cb)
-	: _p(p), _sn(sn), _cb(cb), _keepalive(false)
+	: _p(p), _sn(sn), _b(_buffer), _cb(cb), _keepalive(false)
     {
 	size_t header_bytes = STCPPacketHeader::header_size();
 	size_t xrl_bytes = x.packed_bytes();
-	_b.resize(header_bytes + xrl_bytes);
+	size_t total = header_bytes + xrl_bytes;
+
+	if (total > sizeof(_buffer))
+	    _b = new uint8_t[total];
+
+	_size = total;
 
 	// Prepare header
-	STCPPacketHeader sph(&_b[0]);
+	STCPPacketHeader sph(_b);
 	sph.initialize(_sn, STCP_PT_REQUEST, XrlError::OKAY(), xrl_bytes);
+	sph.set_batch(batch);
 
 	// Pack XRL data
-	x.pack(&_b[0] + header_bytes, xrl_bytes);
+	x.pack(_b + header_bytes, xrl_bytes);
 
 	debug_msg("RequestState (%p - seqno %u)\n", this, XORP_UINT_CAST(sn));
 	debug_msg("RequestState Xrl = %s\n", x.str().c_str());
     }
 
     RequestState(XrlPFSTCPSender* p, uint32_t sn)
-	: _p(p), _sn(sn), _keepalive(true)
+	: _p(p), _sn(sn), _b(_buffer), _keepalive(true)
     {
 	size_t header_bytes = STCPPacketHeader::header_size();
-	_b.resize(header_bytes);
+
+	XLOG_ASSERT(sizeof(_buffer) >= header_bytes);
+
+	_size = header_bytes;
 
 	// Prepare header
-	STCPPacketHeader sph(&_b[0]);
+	STCPPacketHeader sph(_b);
 	sph.initialize(_sn, STCP_PT_HELO, XrlError::OKAY(), 0);
     }
 
     bool		has_seqno(uint32_t n) const { return _sn == n; }
     XrlPFSTCPSender*	parent() const		{ return _p; }
     uint32_t		seqno() const		{ return _sn; }
-    vector<uint8_t>&	buffer() 		{ return _b; }
+    uint8_t*		buffer() 		{ return _b; }
     Callback&		cb() 			{ return _cb; }
+    uint32_t		size() const		{ return _size; }
 
-    bool is_keepalive() {
-	if (_b.size() < STCPPacketHeader::header_size())
+    bool is_keepalive()
+    {
+	if (size() < STCPPacketHeader::header_size())
 	    return false;
-	const STCPPacketHeader sph(&_b[0]);
+	const STCPPacketHeader sph(_b);
 	STCPPacketType pt = sph.type();
 	return pt == STCP_PT_HELO || pt == STCP_PT_HELO_ACK;
     }
 
+    void set_batch(bool batch)
+    {
+	STCPPacketHeader sph(_b);
+	sph.set_batch(batch);
+    }
+
     ~RequestState()
     {
 	//	debug_msg("~RequestState (%p - seqno %u)\n", this, seqno());
+	if (_b != _buffer)
+	    delete [] _b;
     }
 
 private:
@@ -510,7 +574,9 @@ private:
 private:
     XrlPFSTCPSender*	_p;				// parent
     uint32_t		_sn;				// sequence number
-    vector<uint8_t>	_b;
+    uint8_t*		_b;
+    uint8_t		_buffer[256];	// XXX important performance parameter
+    uint32_t		_size;
     Callback		_cb;
     bool		_keepalive;
 };
@@ -576,14 +642,29 @@ XrlPFSTCPSender::XrlPFSTCPSender(EventLoop& e, const char* addr_slash_port)
     throw (XrlPFConstructorError)
     : XrlPFSender(e, addr_slash_port),
       _uid(_next_uid++),
-      _keepalive_ms(DEFAULT_SENDER_KEEPALIVE_MS)
+      _keepalive_time(DEFAULT_SENDER_KEEPALIVE_TIME),
+      _batching(false)
 {
     _sock = create_connected_tcp4_socket(addr_slash_port);
+    construct();
+}
+
+XrlPFSTCPSender::XrlPFSTCPSender(EventLoop* e, const char* addr_slash_port)
+    : XrlPFSender(*e, addr_slash_port),
+      _uid(_next_uid++), _writer(NULL),
+      _keepalive_time(DEFAULT_SENDER_KEEPALIVE_TIME),
+      _reader(NULL), _batching(false)
+{
+}
+
+void
+XrlPFSTCPSender::construct()
+{
     debug_msg("stcp sender (%p) fd = %s\n", this, _sock.str().c_str());
     if (!_sock.is_valid()) {
-	debug_msg("failed to connect to %s\n", addr_slash_port);
+	debug_msg("failed to connect to %s\n", address().c_str());
 	xorp_throw(XrlPFConstructorError,
-		   c_format("Could not connect to %s\n", addr_slash_port));
+		   c_format("Could not connect to %s\n", address().c_str()));
     }
 
     if (comm_sock_set_blocking(_sock, 0) != XORP_OK) {
@@ -596,14 +677,14 @@ XrlPFSTCPSender::XrlPFSTCPSender(EventLoop& e, const char* addr_slash_port)
 			    comm_get_error_str(err)));
     }
 
-    _reader = new BufferedAsyncReader(e, _sock, 4 * 65536,
+    _reader = new BufferedAsyncReader(_eventloop, _sock, 4 * 65536,
 				      callback(this,
 					       &XrlPFSTCPSender::read_event));
 
     _reader->set_trigger_bytes(STCPPacketHeader::header_size());
     _reader->start();
 
-    _writer = new AsyncFileWriter(e, _sock, MAX_WRITES);
+    _writer = new AsyncFileWriter(_eventloop, _sock, MAX_WRITES);
 
     _current_seqno   = 0;
     _active_bytes    = 0;
@@ -709,7 +790,10 @@ XrlPFSTCPSender::send(const Xrl&	x,
 
     debug_msg("Seqno %u send %s\n", XORP_UINT_CAST(_current_seqno),
 	      x.str().c_str());
-    send_request(new RequestState(this, _current_seqno++, x, cb));
+
+    RequestState* rs = new RequestState(this, _current_seqno++,
+					_batching, x, cb);
+    send_request(rs);
 
     xassert(_requests_waiting.size() + _requests_sent.size() == _active_requests);
 
@@ -720,14 +804,13 @@ void
 XrlPFSTCPSender::send_request(RequestState* rs)
 {
     _requests_waiting.push_back(rs);
-    const vector<uint8_t>& buffer = rs->buffer();
-    _active_bytes += buffer.size();
+    _active_bytes += rs->size();
     _active_requests ++;
-    _writer->add_buffer(&buffer[0], buffer.size(),
+    _writer->add_buffer(rs->buffer(), rs->size(),
 			callback(this, &XrlPFSTCPSender::update_writer));
-    if (_writer->running() == false) {
+
+    if ((!_batching || rs->is_keepalive()) && _writer->running() == false)
 	_writer->start();
-    }
 }
 
 void
@@ -735,7 +818,7 @@ XrlPFSTCPSender::dispose_request()
 {
     assert(_requests_sent.empty() == false);
     xassert(_requests_sent.size() + _requests_waiting.size() == _active_requests);
-    _active_bytes -= _requests_sent.front()->buffer().size();
+    _active_bytes -= _requests_sent.front()->size();
     _active_requests -= 1;
     _requests_sent.pop_front();
     xassert(_requests_waiting.size() == _writer->buffers_remaining());
@@ -891,17 +974,17 @@ XrlPFSTCPSender::protocol() const
 // Sender keepalive related
 
 void
-XrlPFSTCPSender::set_keepalive_ms(uint32_t t)
+XrlPFSTCPSender::set_keepalive_time(const TimeVal& time)
 {
-    _keepalive_ms = t;
+    _keepalive_time = time;
     start_keepalives();
 }
 
 void
 XrlPFSTCPSender::start_keepalives()
 {
-    _keepalive_timer = _eventloop.new_periodic_ms(
-	_keepalive_ms,
+    _keepalive_timer = _eventloop.new_periodic(
+	_keepalive_time,
 	callback(this, &XrlPFSTCPSender::send_keepalive),
 	XorpTask::PRIORITY_XRL_KEEPALIVE);
 }
@@ -916,8 +999,8 @@ void
 XrlPFSTCPSender::defer_keepalives()
 {
     if (_keepalive_timer.scheduled()) {
-	_keepalive_timer.schedule_after_ms(_keepalive_ms,
-					   XorpTask::PRIORITY_XRL_KEEPALIVE);
+	_keepalive_timer.schedule_after(_keepalive_time,
+					XorpTask::PRIORITY_XRL_KEEPALIVE);
     }
 }
 
@@ -927,8 +1010,7 @@ XrlPFSTCPSender::send_keepalive()
     TimeVal now;
 
     _eventloop.current_time(now);
-    if ((now - _keepalive_last_fired).to_ms() <
-	 static_cast<int32_t>(_keepalive_ms)) {
+    if (now - _keepalive_last_fired < _keepalive_time) {
 	// This keepalive timer has fired too soon. Rate-limit it.
 	debug_msg("Dropping keepalive due to rate-limiting\n");
 	return true;
@@ -948,3 +1030,25 @@ XrlPFSTCPSender::send_keepalive()
 
     return true;
 }
+
+void
+XrlPFSTCPSender::batch_start()
+{
+    _batching = true;
+}
+
+void
+XrlPFSTCPSender::batch_stop()
+{
+    _batching = false;
+
+    // If we aint got no requests, we may not be able to signal to the receiver
+    // to stop batching and we could deadlock.  In this case we should probably
+    // send a keep-alive to resume things.  -sorbo
+    XLOG_ASSERT(_requests_waiting.size());
+
+    _requests_waiting.back()->set_batch(false);
+
+    if (_writer->running() == false)
+	_writer->start();
+}
