@@ -1,18 +1,25 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License, Version
+// 2.1, June 1999 as published by the Free Software Foundation.
+// Redistribution and/or modification of this program under the terms of
+// any other version of the GNU Lesser General Public License is not
+// permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU Lesser General Public License, Version 2.1, a copy of
+// which can be found in the XORP LICENSE.lgpl file.
+// 
+// XORP, Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/libxipc/xrl_pf_stcp_ph.cc,v 1.18 2008/07/23 05:10:46 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/xrl_pf_stcp_ph.cc,v 1.22 2009/01/05 18:30:56 jtc Exp $"
 
 #include "xrl_module.h"
 #include "libxorp/xorp.h"
@@ -49,13 +56,14 @@ STCPPacketHeader::STCPPacketHeader(uint8_t* data)
       _major(_data + _major_offset),
       _minor(_data + _minor_offset),
       _seqno(_data + _seqno_offset),
+      _flags(_data + _flags_offset),
       _type(_data + _type_offset),
       _error_code(_data + _error_code_offset),
       _error_note_bytes(_data + _error_note_bytes_offset),
       _xrl_data_bytes(_data + _xrl_data_bytes_offset)
 {
     static_assert(STCPPacketHeader::SIZE == _fourcc_sizeof + _major_sizeof
-		  + _minor_sizeof + _seqno_sizeof + _type_sizeof
+		  + _minor_sizeof + _seqno_sizeof + _flags_sizeof + _type_sizeof
 		  + _error_code_sizeof + _error_note_bytes_sizeof
 		  + _xrl_data_bytes_sizeof);
     static_assert(STCPPacketHeader::SIZE ==
@@ -71,7 +79,8 @@ STCPPacketHeader::initialize(uint32_t		seqno,
     embed_32(_fourcc, PROTO_FOURCC);
     embed_8(_major, PROTO_MAJOR);
     embed_8(_minor, PROTO_MINOR);
-    embed_16(_type, type);
+    embed_8(_flags, 0);
+    embed_8(_type, type);
     embed_32(_seqno, seqno);
     embed_32(_error_code, xrl_err.error_code());
     embed_32(_error_note_bytes, xrl_err.note().size());
@@ -131,7 +140,7 @@ STCPPacketHeader::minor() const
 STCPPacketType
 STCPPacketHeader::type() const
 {
-    return STCPPacketType(extract_16(_type));
+    return STCPPacketType(extract_8(_type));
 }
 
 uint32_t
@@ -169,3 +178,20 @@ STCPPacketHeader::frame_bytes() const
 {
     return header_size() + error_note_bytes() + payload_bytes();
 }
+
+bool
+STCPPacketHeader::batch() const
+{
+    return extract_8(_flags) & FLAG_BATCH_MASK;
+}
+
+void
+STCPPacketHeader::set_batch(bool batch)
+{
+    uint8_t flags = extract_8(_flags);
+
+    flags &= ~(1 << FLAG_BATCH_SHIFT);
+    flags |= batch << FLAG_BATCH_SHIFT;
+
+    embed_8(_flags, flags);
+}
