@@ -1,18 +1,25 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License, Version
+// 2.1, June 1999 as published by the Free Software Foundation.
+// Redistribution and/or modification of this program under the terms of
+// any other version of the GNU Lesser General Public License is not
+// permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU Lesser General Public License, Version 2.1, a copy of
+// which can be found in the XORP LICENSE.lgpl file.
+// 
+// XORP, Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/libxipc/xrl_router.cc,v 1.58 2008/07/23 05:10:46 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/xrl_router.cc,v 1.73 2009/01/05 18:30:56 jtc Exp $"
 
 #include "xrl_module.h"
 #include "libxorp/debug.h"
@@ -281,6 +288,8 @@ XrlRouter::~XrlRouter()
     if (_icnt == 0)
 	XrlPFSenderFactory::shutdown();
 
+    for (XIM::iterator i = _xi_cache.begin(); i != _xi_cache.end(); ++i)
+	delete i->second;
 }
 
 bool
@@ -378,48 +387,17 @@ XrlRouter::send_resolved(const Xrl&		xrl,
 			 bool  direct_call)
 {
     try {
-	Xrl x(dbe->values().front().c_str());
-
-	XrlPFSender* s = 0;
-	list<XrlPFSender*>::iterator i;
-	for (i = _senders.begin(); i != _senders.end(); ++i) {
-	    s = *i;
-
-	    if (s->protocol() != x.protocol() || s->address()  != x.target()) {
-		continue;
-	    }
-
-	    if (s->alive()) {
-		goto __got_sender;
-	    }
-
-	    XLOG_INFO("Sender died (protocol = \"%s\", address = \"%s\")",
-		      s->protocol(), s->address().c_str());
-	    XrlPFSenderFactory::destroy_sender(s);
-	    _senders.erase(i);
-	    break;
-	}
-
-	s = XrlPFSenderFactory::create_sender(_e, x.protocol().c_str(),
-					      x.target().c_str());
+	XrlPFSender* s = get_sender(xrl, const_cast<FinderDBEntry*>(dbe));
 	if (s == 0) {
-	    XLOG_ERROR("Could not create XrlPFSender for protocol = \"%s\" "
-		       "address = \"%s\" ",
-		       x.protocol().c_str(), x.target().c_str());
-
 	    // Notify Finder client that result was bad.
 	    _fc->uncache_result(dbe);
 
 	    // Coerce finder client to check with Finder.
 	    return send(xrl, cb);
 	}
-	XLOG_ASSERT(s->protocol() == x.protocol());
-	XLOG_ASSERT(s->address()  == x.target());
-	_senders.push_back(s);
 
-    __got_sender:
-	Xrl tmp(xrl);
-	x.args().swap(tmp.args());
+	const Xrl& x = dbe->xrls().front();
+    	x.set_args(xrl);
 	if (s) {
 	    trace_xrl("Sending ", x);
 	    return s->send(x, direct_call,
@@ -432,6 +410,87 @@ XrlRouter::send_resolved(const Xrl&		xrl,
     return false;
 }
 
+XrlPFSender*
+XrlRouter::get_sender(const Xrl& xrl, FinderDBEntry* dbe)
+{
+    const Xrl& x = dbe->xrls().front();
+    XrlPFSender* s = NULL;
+
+    // Use the cache pointer to the sender.
+    if (xrl.resolved()) {
+	s = xrl.resolved_sender();
+
+	if (s->alive())
+	    return s;
+
+	XLOG_ASSERT(s->protocol() == x.protocol());
+	XLOG_ASSERT(s->address()  == x.target());
+
+	xrl.set_resolved(false);
+	xrl.set_resolved_sender(NULL);
+    }
+
+    // Find a new sender.
+    for (list<XrlPFSender*>::iterator i = _senders.begin();
+	 i != _senders.end(); ++i) {
+	s = *i;
+
+	if (s->protocol() != x.protocol() || s->address()  != x.target())
+	    continue;
+
+	if (s->alive()) {
+	    xrl.set_resolved(true);
+	    xrl.set_resolved_sender(s);
+	    return s;
+	}
+
+	XLOG_INFO("Sender died (protocol = \"%s\", address = \"%s\")",
+		  s->protocol(), s->address().c_str());
+	XrlPFSenderFactory::destroy_sender(s);
+	_senders.erase(i);
+	_senders2.erase(xrl.target());
+	break;
+    }
+
+    s = NULL;
+
+    // create sender
+    while (dbe->xrls().size()) {
+	const Xrl& x = dbe->xrls().front();
+
+	s = XrlPFSenderFactory::create_sender(_e, x.protocol().c_str(),
+					      x.target().c_str());
+	if (s)
+	    break;
+
+	XLOG_ERROR("Could not create XrlPFSender for protocol = \"%s\" "
+		   "address = \"%s\" ",
+		   x.protocol().c_str(), x.target().c_str());
+
+	dbe->pop_front();
+    }
+    if (!s)
+	return NULL;
+
+    const Xrl& front = dbe->xrls().front();
+
+    XLOG_ASSERT(s->protocol() == front.protocol());
+    XLOG_ASSERT(s->address()  == front.target());
+    _senders.push_back(s);
+    _senders2[xrl.target()] = s;
+
+    // Don't do this here as it is set in the Xrl that is stored with
+    // the finder client, so is not used. But if the connection needs
+    // to be re-established then it will be used and the state will be
+    // bad.
+#if	0
+    xrl.set_resolved(true);
+    xrl.set_resolved_sender(s);
+#endif
+
+    return s;
+}
+
 void
 XrlRouter::resolve_callback(const XrlError&	 	e,
 			    const FinderDBEntry*	dbe,
@@ -443,7 +502,10 @@ XrlRouter::resolve_callback(const XrlError&	 	e,
     _dsl.erase(i);
 
     if (e == XrlError::OKAY()) {
-	if (send_resolved(ds->xrl(), dbe, ds->cb(), false) == false) {
+	const Xrl& xrl = ds->xrl();
+	xrl.set_resolved(false);
+	xrl.set_resolved_sender(NULL);
+	if (send_resolved(xrl, dbe, ds->cb(), false) == false) {
 	    // We tried to force sender to send xrl and it declined the
 	    // opportunity.  This should only happen when it's out of buffer
 	    // space
@@ -529,7 +591,7 @@ XrlRouter::send(const Xrl& xrl, const XrlCallback& user_cb)
     //
     // Finder directed Xrl - takes custom path through FinderClient.
     //
-    if (xrl.protocol() == "finder" && xrl.target().substr(0,6) == "finder") {
+    if (xrl.to_finder()) {
 	if (_fc->forward_finder_xrl(xrl, xcb)) {
 	    return true;
 	}
@@ -560,7 +622,8 @@ XrlRouter::send(const Xrl& xrl, const XrlCallback& user_cb)
     // care about re-ordering between protocol families (at this time), but
     // we do within protocol families.
     //
-    string xrl_no_args = xrl.string_no_args();
+    const string& xrl_no_args = xrl.string_no_args();
+    // We can't cache this object.
     const FinderDBEntry* fdbe = _fc->query_cache(xrl_no_args);
     if (_dsl.empty() && fdbe) {
 	return send_resolved(xrl, fdbe, xcb, true);
@@ -590,6 +653,29 @@ XrlRouter::dispatch_xrl(const string&	method_name,
     return XrlError::NO_SUCH_METHOD();
 }
 
+XrlDispatcher::XI*
+XrlRouter::lookup_xrl(const string& name) const
+{
+    // fast path
+    XIM::const_iterator i = _xi_cache.find(name);
+
+    if (i != _xi_cache.end())
+	return i->second;
+
+    // slow path
+    string resolved;
+    if (_fc->query_self(name, resolved) != true)
+	return NULL;
+
+    XI* xi = XrlDispatcher::lookup_xrl(resolved);
+    if (!xi)
+	return NULL;
+
+    _xi_cache[name] = xi;
+
+    return xi;
+}
+
 IPv4
 XrlRouter::finder_address() const
 {
@@ -622,6 +708,35 @@ XrlRouter::finder_ready_event(const string& tgt_name)
     debug_msg("Finder target ready event: \"%s\"\n", tgt_name.c_str());
 }
 
+void
+XrlRouter::batch_start(const string& target)
+{
+    XrlPFSender& sender = get_sender(target);
+
+    sender.batch_start();
+}
+
+void
+XrlRouter::batch_stop(const string& target)
+{
+    XrlPFSender& sender = get_sender(target);
+
+    sender.batch_stop();
+}
+
+XrlPFSender&
+XrlRouter::get_sender(const string& target)
+{
+    XrlPFSender* s = 0;
+
+    SENDERS::iterator i = _senders2.find(target);
+    XLOG_ASSERT(i != _senders2.end());
+
+    s = i->second;
+
+    return *s;
+}
+
 
 // ----------------------------------------------------------------------------
 // wait_until_xrl_router_is_ready
