@@ -1,19 +1,25 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License, Version
+// 2.1, June 1999 as published by the Free Software Foundation.
+// Redistribution and/or modification of this program under the terms of
+// any other version of the GNU Lesser General Public License is not
+// permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU Lesser General Public License, Version 2.1, a copy of
+// which can be found in the XORP LICENSE.lgpl file.
+// 
+// XORP, Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/libxorp/asyncio.cc,v 1.42 2008/07/23 05:10:49 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/asyncio.cc,v 1.48 2009/01/05 18:30:57 jtc Exp $"
 
 #include "libxorp_module.h"
 
@@ -509,9 +515,8 @@ AsyncFileWriter::write(XorpFd fd, IoEventType type)
     size_t total_bytes = 0;
     ssize_t done = 0;
     int flags = 0;
-    bool mod_signals = true;
 #ifndef HOST_OS_WINDOWS
-    sig_t saved_sigpipe = SIG_ERR;
+    bool mod_signals = true;
 #endif
 
 #ifdef MSG_NOSIGNAL
@@ -577,12 +582,6 @@ AsyncFileWriter::write(XorpFd fd, IoEventType type)
 	//
 	XLOG_ASSERT(! dst_addr.is_zero());
 
-	if (mod_signals) {
-#ifndef HOST_OS_WINDOWS
-	    saved_sigpipe = signal(SIGPIPE, SIG_IGN);
-#endif
-	}
-
 	switch (dst_addr.af()) {
 	case AF_INET:
 	{
@@ -628,12 +627,6 @@ AsyncFileWriter::write(XorpFd fd, IoEventType type)
 #endif
 	}
 
-	if (mod_signals) {
-#ifndef HOST_OS_WINDOWS
-	    signal(SIGPIPE, saved_sigpipe);
-#endif
-	}
-
     } else {
 	//
 	// Write the data to the socket/file descriptor
@@ -679,11 +672,9 @@ AsyncFileWriter::write(XorpFd fd, IoEventType type)
 	    if (done < 0)
 		_last_error = errno;
 	} else {
-	    saved_sigpipe = signal(SIGPIPE, SIG_IGN);
 	    done = ::writev(_fd, _iov, (int)iov_cnt);
 	    if (done < 0)
 		_last_error = errno;
-	    signal(SIGPIPE, saved_sigpipe);
 	}
 	errno = 0;
 #endif // ! HOST_OS_WINDOWS
@@ -714,7 +705,14 @@ void
 AsyncFileWriter::complete_transfer(ssize_t sdone)
 {
     if (sdone < 0) {
-	XLOG_ERROR("Write error %d\n", _last_error);
+	do {
+	    // XXX: don't print an error if the error code is EPIPE
+#ifdef EPIPE
+	    if (_last_error == EPIPE)
+		break;
+#endif
+	    XLOG_ERROR("Write error %d\n", _last_error);
+	} while (false);
 	stop();
 	BufferInfo* head = _buffers.front();
 	head->dispatch_callback(OS_ERROR);
