@@ -1,18 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License, Version
+// 2.1, June 1999 as published by the Free Software Foundation.
+// Redistribution and/or modification of this program under the terms of
+// any other version of the GNU Lesser General Public License is not
+// permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU Lesser General Public License, Version 2.1, a copy of
+// which can be found in the XORP LICENSE.lgpl file.
+// 
+// XORP, Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/libxorp/buffered_asyncio.cc,v 1.15 2008/07/23 05:10:49 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/buffered_asyncio.cc,v 1.18 2009/01/05 18:30:57 jtc Exp $"
 
 #include "libxorp_module.h"
 #include "xorp.h"
@@ -24,11 +30,13 @@
 extern bool is_pseudo_error(const char* name, XorpFd fd, int error_num);
 
 BufferedAsyncReader::BufferedAsyncReader(EventLoop& 		e,
-					 XorpFd 			fd,
+					 XorpFd 		fd,
 					 size_t 		reserve_bytes,
-					 const Callback& 	cb)
+					 const Callback& 	cb,
+					 int			priority)
     : _eventloop(e), _fd(fd), _cb(cb), _buffer(reserve_bytes),
-      _last_error(0)
+      _last_error(0),
+      _priority(priority)
 {
     _config.head 	  = &_buffer[0];
     _config.head_bytes 	  = 0;
@@ -116,14 +124,16 @@ BufferedAsyncReader::start()
 {
     if (_eventloop.add_ioevent_cb(_fd, IOT_READ,
 				  callback(this,
-					   &BufferedAsyncReader::io_event)) ==
+					   &BufferedAsyncReader::io_event),
+				  _priority) ==
 	false) {
 	XLOG_ERROR("BufferedAsyncReader: failed to add I/O event callback.");
     }
 #ifdef HOST_OS_WINDOWS
     if (_eventloop.add_ioevent_cb(_fd, IOT_DISCONNECT,
 				  callback(this,
-					   &BufferedAsyncReader::io_event)) ==
+					   &BufferedAsyncReader::io_event),
+				  _priority) ==
 	false) {
 	XLOG_ERROR("BufferedAsyncReader: failed to add I/O event callback.");
     }
