@@ -1,18 +1,25 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License, Version
+// 2.1, June 1999 as published by the Free Software Foundation.
+// Redistribution and/or modification of this program under the terms of
+// any other version of the GNU Lesser General Public License is not
+// permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU Lesser General Public License, Version 2.1, a copy of
+// which can be found in the XORP LICENSE.lgpl file.
+// 
+// XORP, Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/libxorp/profile.cc,v 1.11 2008/07/23 05:10:53 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/profile.cc,v 1.17 2009/01/05 18:30:58 jtc Exp $"
 
 #include "libxorp_module.h"
 #include "xorp.h"
@@ -226,3 +233,111 @@ Profile::list() const
     return for_each(_profiles.begin(), _profiles.end(), List()).result();
 }
 
+// simple profiler
+SP::SAMPLE
+SP::sampler_time()
+{
+    TimeVal tv;
+
+    TimerList::system_gettimeofday(&tv);
+
+    SAMPLE ret = tv.sec();
+
+    ret *= (SAMPLE) 1000000;
+    ret += (SAMPLE) tv.usec();
+
+    return ret;
+}
+
+#ifdef __HAVE_TSC__
+// XXX watch out on SMP systems - make sure u're always reading the same tsc
+// (i.e., same core running the process - set affinity.  On Linux use taskset).
+// Or disable smp.  -sorbo
+SP::SAMPLE
+SP::sampler_tsc(void)
+{   
+    uint64_t tsc;
+
+    __asm__ volatile (".byte 0x0f, 0x31" : "=A" (tsc));
+
+    return tsc;
+}
+#endif // __HAVE_TSC__
+
+namespace SP {
+    SAMPLE      _samples[SP_MAX_SAMPLES];
+    const char* _desc[SP_MAX_SAMPLES];
+    unsigned    _samplec;
+#ifdef __HAVE_TSC__
+    SAMPLER	_sampler = SP::sampler_tsc;
+#else
+    SAMPLER	_sampler = SP::sampler_time;
+#endif // __HAVE_TSC__
+}
+
+void
+SP::set_sampler(SAMPLER sampler)
+{
+    _sampler = sampler;
+}
+
+void
+SP::add_sample(const char* desc)
+{
+    if (!_sampler)
+	return;
+
+    XLOG_ASSERT(_samplec < SP_MAX_SAMPLES);
+
+    _samples[_samplec] = _sampler();
+    _desc[_samplec]    = desc;
+
+    _samplec++;
+}
+
+void
+SP::print_samples()
+{
+    if (!_samplec)
+	return;
+
+    double total = _samples[_samplec - 1] - _samples[0];
+
+    printf("\n");
+    printf("Absolute time\tElapsed time\tPercentage\tDescription\n");
+
+    for (unsigned i = 0; i < _samplec; i++) {
+        printf("%llu\t", (long long unsigned) _samples[i]);
+
+        if (i != 0) {
+            SAMPLE a, b, diff;
+
+            a = _samples[i - 1];
+            b = _samples[i];
+
+            XLOG_ASSERT(a <= b);
+
+            diff = b - a;
+
+            printf("%12llu\t%10.2f\t",
+		   (long long unsigned) diff, (double) diff / total * 100.0);
+        } else
+            printf("\t\t\t\t");
+
+        printf("%s\n", _desc[i]);
+    }
+
+    printf("Total %llu\n", (long long unsigned) total);
+    printf("\n");
+
+    _samplec = 0;
+}
+
+SP::SAMPLE
+SP::sample()
+{
+    if (_sampler)
+	return _sampler();
+
+    return 0;
+}
