@@ -1,19 +1,25 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License, Version
+// 2.1, June 1999 as published by the Free Software Foundation.
+// Redistribution and/or modification of this program under the terms of
+// any other version of the GNU Lesser General Public License is not
+// permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU Lesser General Public License, Version 2.1, a copy of
+// which can be found in the XORP LICENSE.lgpl file.
+// 
+// XORP, Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-// $XORP: xorp/libxorp/run_command.cc,v 1.34 2008/07/23 05:10:54 pavlin Exp $
+#ident "$XORP: xorp/libxorp/run_command.cc,v 1.38 2009/01/05 18:30:58 jtc Exp $"
 
 #include "libxorp_module.h"
 
@@ -132,8 +138,9 @@ RunCommand::RunCommand(EventLoop&			eventloop,
 		       RunCommand::OutputCallback	stdout_cb,
 		       RunCommand::OutputCallback	stderr_cb,
 		       RunCommand::DoneCallback		done_cb,
-		       bool				redirect_stderr_to_stdout)
-    : RunCommandBase(eventloop, command, command),
+		       bool				redirect_stderr_to_stdout,
+		       int task_priority)
+    : RunCommandBase(eventloop, command, command, task_priority),
       _stdout_cb(stdout_cb),
       _stderr_cb(stderr_cb),
       _done_cb(done_cb),
@@ -147,8 +154,9 @@ RunShellCommand::RunShellCommand(EventLoop&		eventloop,
 				 const string&		argument_string,
 				 RunShellCommand::OutputCallback stdout_cb,
 				 RunShellCommand::OutputCallback stderr_cb,
-				 RunShellCommand::DoneCallback	 done_cb)
-    : RunCommandBase(eventloop, get_path_bshell(), command),
+				 RunShellCommand::DoneCallback	 done_cb,
+				 int task_priority)
+    : RunCommandBase(eventloop, get_path_bshell(), command, task_priority),
       _stdout_cb(stdout_cb),
       _stderr_cb(stderr_cb),
       _done_cb(done_cb)
@@ -165,7 +173,8 @@ RunShellCommand::RunShellCommand(EventLoop&		eventloop,
 
 RunCommandBase::RunCommandBase(EventLoop&		eventloop,
 			       const string&		command,
-			       const string&		real_command_name)
+			       const string&		real_command_name,
+			       int task_priority)
     : _eventloop(eventloop),
       _command(command),
       _real_command_name(real_command_name),
@@ -186,7 +195,8 @@ RunCommandBase::RunCommandBase(EventLoop&		eventloop,
       _command_term_signal(0),
       _command_stop_signal(0),
       _stdout_eof_received(false),
-      _stderr_eof_received(false)
+      _stderr_eof_received(false),
+      _task_priority(task_priority)
 {
     memset(_stdout_buffer, 0, BUF_SIZE);
     memset(_stderr_buffer, 0, BUF_SIZE);
@@ -339,7 +349,8 @@ RunCommandBase::execute()
 
     // Create the stdout and stderr readers
     _stdout_file_reader = new AsyncFileReader(_eventloop,
-					      XorpFd(fileno(_stdout_stream)));
+					      XorpFd(fileno(_stdout_stream)),
+					      task_priority());
     _stdout_file_reader->add_buffer(
 	_stdout_buffer,
 	BUF_SIZE,
@@ -353,7 +364,8 @@ RunCommandBase::execute()
     }
 
     _stderr_file_reader = new AsyncFileReader(_eventloop,
-					      XorpFd(fileno(_stderr_stream)));
+					      XorpFd(fileno(_stderr_stream)),
+					      task_priority());
     _stderr_file_reader->add_buffer(
 	_stderr_buffer,
 	BUF_SIZE,
