@@ -1,18 +1,25 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License, Version
+// 2.1, June 1999 as published by the Free Software Foundation.
+// Redistribution and/or modification of this program under the terms of
+// any other version of the GNU Lesser General Public License is not
+// permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU Lesser General Public License, Version 2.1, a copy of
+// which can be found in the XORP LICENSE.lgpl file.
+// 
+// XORP, Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/libxorp/test_task.cc,v 1.7 2008/07/23 05:10:56 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/test_task.cc,v 1.13 2009/01/05 18:30:58 jtc Exp $"
 
 //
 // Demo program to test tasks and event loops.
@@ -52,6 +59,19 @@ public:
 	XLOG_ASSERT(_eventloop.events_pending() == false);
 	debug_msg("counter1 = %d counter2 = %d counter3 = %d\n",
 		  _counter1, _counter2, _counter3);
+
+	// handler1 must be run exactly once
+	if (_counter3 != 1 || _counter1 == 0 || _counter2 == 0)
+	    return false;
+
+	// handler2 must run twice as frequently as handler1.  There could be an
+	// error of at most 2 because handler1 could have run and we could have
+	// stopped before getting a chance of running handler2.
+	int err = _counter1 * 2 - _counter2;
+	if (err > 2 || err < 0)
+	    return false;
+
+	// XXX could return true, but lets hardcode values for safety.
 	return (_counter1 == 10 && _counter2 == 20 && _counter3 == 1);
     }
 
@@ -72,6 +92,16 @@ public:
 	XLOG_ASSERT(_eventloop.events_pending() == false);
 	debug_msg("counter1 = %d counter2 = %d counter3 = %d\n",
 		  _counter1, _counter2, _counter3);
+
+	// task 3 and 2 must be starved
+	if (_counter2 > 0 || _counter3 > 0)
+	    return false;
+
+	// task 1 must get all runs
+	if (_counter1 == 0)
+	    return false;
+
+	// XXX could return true.
 	return (_counter1 == 11 && _counter2 == 0 && _counter3 == 0);
     }
 
@@ -92,6 +122,21 @@ public:
 	XLOG_ASSERT(_eventloop.events_pending() == false);
 	debug_msg("counter1 = %d counter2 = %d counter3 = %d\n",
 		  _counter1, _counter2, _counter3);
+
+	// task 1 and 3 must complete
+	if (_counter1 != 10 || _counter3 != 1)
+	    return false;
+
+	// task 2 must get the rest of the cycles
+	int aggressiveness = 0; // eventloop's aggressiveness
+	int cycles = 16;
+	cycles    *= 1 + aggressiveness; // runs per event loop
+	cycles    -= _counter1 + _counter3;
+
+	if (_counter2 != cycles)
+	    return false;
+
+	// XXX could return true
 	return (_counter1 == 10 && _counter2 == 5 && _counter3 == 1);
     }
 
