@@ -1,20 +1,27 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 /* vim:set sts=4 ts=8: */
 
-/* Copyright (c) 2001-2008 XORP, Inc.
+/*
+ * Copyright (c) 2001-2009 XORP, Inc.
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software")
- * to deal in the Software without restriction, subject to the conditions
- * listed in the XORP LICENSE file. These conditions include: you must
- * preserve this copyright notice, and you cannot mention the copyright
- * holders in advertising related to the Software without their permission.
- * The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
- * notice is a summary of the XORP LICENSE file; the license in that file is
- * legally binding.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License, Version
+ * 2.1, June 1999 as published by the Free Software Foundation.
+ * Redistribution and/or modification of this program under the terms of
+ * any other version of the GNU Lesser General Public License is not
+ * permitted.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+ * see the GNU Lesser General Public License, Version 2.1, a copy of
+ * which can be found in the XORP LICENSE.lgpl file.
+ * 
+ * XORP, Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+ * http://xorp.net
  */
 
-#ident "$XORP: xorp/libxorp/xlog.c,v 1.28 2008/07/23 05:10:58 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/xlog.c,v 1.32 2009/01/05 18:30:58 jtc Exp $"
 
 /*
  * Message logging utility.
@@ -77,7 +84,9 @@ static const char	*xlog_level_names[XLOG_LEVEL_MAX] = {
 				"ERROR",
 				"WARNING",
 				"INFO",
-				"TRACE"
+				"TRACE",
+				/* XXX: temp, to be removed; see Bugzilla entry 795 */
+				"RTRMGR_ONLY_NO_PREAMBLE",
 			};
 
 
@@ -162,6 +171,9 @@ xlog_init(const char *argv0, const char *preamble_message)
 	xlog_verbose_level[level] = XLOG_VERBOSE_LOW;		/* Default */
     }
     xlog_verbose_level[XLOG_LEVEL_FATAL] = XLOG_VERBOSE_HIGH;	/* XXX */
+    /* XXX: temp, to be removed; see Bugzilla entry 795 */
+    xlog_verbose_level[XLOG_LEVEL_RTRMGR_ONLY_NO_PREAMBLE] =
+	XLOG_VERBOSE_RTRMGR_ONLY_NO_PREAMBLE;
 
     init_flag = 1;
 
@@ -222,6 +234,9 @@ xlog_exit(void)
 	xlog_verbose_level[level] = XLOG_VERBOSE_LOW;		/* Default */
     }
     xlog_verbose_level[XLOG_LEVEL_FATAL] = XLOG_VERBOSE_HIGH;	/* XXX */
+    /* XXX: temp, to be removed; see Bugzilla entry 795 */
+    xlog_verbose_level[XLOG_LEVEL_RTRMGR_ONLY_NO_PREAMBLE] =
+	XLOG_VERBOSE_RTRMGR_ONLY_NO_PREAMBLE;
 
     /* Release the exit lock */
     xlog_exit_unlock();
@@ -374,6 +389,9 @@ xlog_set_verbose(xlog_verbose_t verbose_level)
     for (i = 0; i < XLOG_LEVEL_MAX; i++) {
 	if (i == XLOG_LEVEL_FATAL)
 	    continue;		/* XXX: XLOG_LEVEL_FATAL cannot be changed */
+	/* XXX: temp, to be removed; see Bugzilla entry 795 */
+	if (i == XLOG_LEVEL_RTRMGR_ONLY_NO_PREAMBLE)
+	    continue;
 	xlog_verbose_level[i] = verbose_level;
     }
 }
@@ -398,6 +416,10 @@ xlog_level_set_verbose(xlog_level_t log_level, xlog_verbose_t verbose_level)
     if (log_level == XLOG_LEVEL_FATAL)
 	return;			/* XXX: XLOG_LEVEL_FATAL cannot be changed */
 
+	/* XXX: temp, to be removed; see Bugzilla entry 795 */
+    if (log_level == XLOG_LEVEL_RTRMGR_ONLY_NO_PREAMBLE)
+	return;
+
     if (XLOG_VERBOSE_HIGH < verbose_level)
 	verbose_level = XLOG_VERBOSE_HIGH;
 
@@ -440,6 +462,8 @@ xlog_fn_abort(fatal, 	XLOG_LEVEL_FATAL)
 xlog_fn(error, 		XLOG_LEVEL_ERROR)
 xlog_fn(warning,	XLOG_LEVEL_WARNING)
 xlog_fn(info, 		XLOG_LEVEL_INFO)
+/* XXX: temp, to be removed; see Bugzilla entry 795 */
+xlog_fn(rtrmgr_only_no_preamble,	XLOG_LEVEL_RTRMGR_ONLY_NO_PREAMBLE)
 
 
 /*
@@ -599,6 +623,11 @@ xlog_record_va(xlog_level_t log_level, const char *module_name,
      * different when we write to more than one outputs.
      */
     switch (xlog_verbose_level[log_level]) {
+    case XLOG_VERBOSE_RTRMGR_ONLY_NO_PREAMBLE:	/* No log information */
+	/* XXX: temp, to be removed; see Bugzilla entry 795 */
+	x_asprintf(&buf_preamble_ptr, "");
+	break;
+	
     case XLOG_VERBOSE_LOW:	/* The minimum log information */
 	x_asprintf(&buf_preamble_ptr, "[ %s %s %s %s ] ",
 		   xlog_localtime2string_short(),
@@ -1294,6 +1323,10 @@ xlog_level_to_syslog_priority(xlog_level_t xloglevel)
     case XLOG_LEVEL_INFO:
 	return (LOG_INFO);
 	break;
+    case XLOG_LEVEL_RTRMGR_ONLY_NO_PREAMBLE:
+	/* XXX: temp, to be removed; see Bugzilla entry 795 */
+	return (LOG_INFO);
+	break;
     default:
 	XLOG_UNREACHABLE();
     }
