@@ -1,18 +1,23 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/mld6igmp/mld6igmp_vif.cc,v 1.89 2008/07/23 05:11:04 pavlin Exp $"
+#ident "$XORP: xorp/mld6igmp/mld6igmp_vif.cc,v 1.92 2009/01/05 18:31:00 jtc Exp $"
 
 
 //
@@ -1192,7 +1197,7 @@ Mld6igmpVif::mld6igmp_process(const IPvX& src,
     //
     // Source address check.
     //
-    if (! src.is_unicast()) {
+    if (! (src.is_unicast() || (allow_src_zero_address && src.is_zero()))) {
 	//
 	// Source address must always be unicast.
 	// The kernel should have checked that, but just in case...
@@ -1216,7 +1221,8 @@ Mld6igmpVif::mld6igmp_process(const IPvX& src,
 		     src.af(), family());
     }
     // Source address must be directly connected
-    if (! mld6igmp_node().is_directly_connected(*this, src)) {
+    if (! (mld6igmp_node().is_directly_connected(*this, src)
+	   || (allow_src_zero_address && src.is_zero()))) {
 	error_msg = c_format("RX %s from %s to %s on vif %s: "
 			     "source must be directly connected",
 			     proto_message_type2ascii(message_type),
