@@ -1,21 +1,26 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
 /*
- * Copyright (c) 2001-2008 XORP, Inc.
+ * Copyright (c) 2001-2009 XORP, Inc.
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software")
- * to deal in the Software without restriction, subject to the conditions
- * listed in the XORP LICENSE file. These conditions include: you must
- * preserve this copyright notice, and you cannot mention the copyright
- * holders in advertising related to the Software without their permission.
- * The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
- * notice is a summary of the XORP LICENSE file; the license in that file is
- * legally binding.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License, Version 2, June
+ * 1991 as published by the Free Software Foundation. Redistribution
+ * and/or modification of this program under the terms of any other
+ * version of the GNU General Public License is not permitted.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+ * see the GNU General Public License, Version 2, a copy of which can be
+ * found in the XORP LICENSE.gpl file.
+ * 
+ * XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+ * http://xorp.net
  */
 
 /*
- * $XORP: xorp/mrt/include/ip_mroute.h,v 1.19 2008/07/23 05:11:07 pavlin Exp $
+ * $XORP: xorp/mrt/include/ip_mroute.h,v 1.22 2009/01/05 18:31:01 jtc Exp $
  */
 
 #ifndef __MRT_INCLUDE_IP_MROUTE_H__
@@ -84,10 +89,20 @@
 #  include <linux/types.h>
 #  ifndef _LINUX_IN_H
 #    define _LINUX_IN_H		/*  XXX: a hack to exclude <linux/in.h> */
+#    define EXCLUDE_LINUX_IN_H
+#  endif
+#  ifndef __LINUX_PIM_H
+#    define __LINUX_PIM_H	/*  XXX: a hack to exclude <linux/pim.h> */
+#    define EXCLUDE_LINUX_PIM_H
+#  endif
 #    include <linux/mroute.h>
+#  ifdef EXCLUDE_LINUX_IN_H
 #    undef _LINUX_IN_H
-#  else
-#    include <linux/mroute.h>
+#    undef EXCLUDE_LINUX_IN_H
+#  endif
+#  ifdef EXCLUDE_LINUX_PIM_H
+#    undef __LINUX_PIM_H
+#    undef EXCLUDE_LINUX_PIM_H
 #  endif
 /*
  * XXX: Conditionally add missing definitions from the <linux/mroute.h>
