@@ -1,19 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/ospf/ospf.cc,v 1.99 2008/07/23 05:11:08 pavlin Exp $"
+#ident "$XORP: xorp/ospf/ospf.cc,v 1.102 2009/01/05 18:31:01 jtc Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -291,12 +296,13 @@ template <typename A>
 bool
 Ospf<A>::transmit(const string& interface, const string& vif,
 		  A dst, A src,
-		  uint8_t* data, uint32_t len)
+		  int ttl, uint8_t* data, uint32_t len)
 {
-    XLOG_TRACE(trace()._packets, "Interface %s Vif %s data %p len %u\n",
-	      interface.c_str(), vif.c_str(), data, len);
-    debug_msg("Interface %s Vif %s data %p len %u\n",
-	      interface.c_str(), vif.c_str(), data, len);
+    XLOG_TRACE(trace()._packets,
+              "Interface %s Vif %s ttl %d data %p len %u\n",
+	      interface.c_str(), vif.c_str(), ttl, data, len);
+    debug_msg("Interface %s Vif %s ttl %d data %p len %u\n",
+	      interface.c_str(), vif.c_str(), ttl, data, len);
 
     // If the transport is IPv6 then the checksum has to include the
     // pseudo header. In the IPv4 case this function is a noop.
@@ -325,7 +331,7 @@ Ospf<A>::transmit(const string& interface, const string& vif,
     }
 #endif
 
-    return _io->send(interface, vif, dst, src, data, len);
+    return _io->send(interface, vif, dst, src, ttl, data, len);
 }
 
 template <typename A>
