@@ -1,19 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/ospf/peer_manager.cc,v 1.158 2008/07/23 05:11:09 pavlin Exp $"
+#ident "$XORP: xorp/ospf/peer_manager.cc,v 1.161 2009/01/05 18:31:01 jtc Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -47,6 +52,9 @@
 #include "peer.hh"
 #include "peer_manager.hh"
 
+#ifndef VLINK_DEF_TTL
+#define VLINK_DEF_TTL 64    // Default TTL for virtual link packets.
+#endif
 
 template <typename A>
 PeerManager<A>::~PeerManager()
@@ -869,11 +877,12 @@ PeerManager<A>::transmit(const string& interface, const string& vif,
 	string interface;
 	string vif;
 	if (_vlink.get_physical_interface_vif(src, dst, interface, vif))
-	    return _ospf.transmit(interface, vif, dst, src, data, len);
+	    return _ospf.transmit(interface, vif, dst, src,
+                                  VLINK_DEF_TTL, data, len);
 	// We didn't find a match fall through.
     }
 
-    return _ospf.transmit(interface, vif, dst, src, data, len);
+    return _ospf.transmit(interface, vif, dst, src, -1, data, len);
 }
 
 template <typename A>
