@@ -1,19 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/ospf/routing_table.cc,v 1.68 2008/07/23 05:11:09 pavlin Exp $"
+#ident "$XORP: xorp/ospf/routing_table.cc,v 1.73 2009/01/05 18:31:01 jtc Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -99,20 +104,22 @@ RoutingTable<A>::add_entry(OspfTypes::AreaID area, IPNet<A> net,
     XLOG_ASSERT(rt.get_directly_connected() || rt.get_nexthop() != A::ZERO());
     bool status = true;
 
+    // This router entry needs to be placed in the table.
+    // In the OSPFv2 IPv4 case the {router ID}/32 is also placed in
+    // the table as it has to be unique and makes for easy lookup.
+    // In the OSPFv3 case lookups of a router will be done by router
+    // ID only, so don't make an entry by net, plus there is no way of
+    // guaranteeing a unique net as the net will be a link-local address.
     if (rt.get_destination_type() == OspfTypes::Router) {
 	status = _adv.add_entry(area, rt.get_router_id(), rt);
-// 	switch(_ospf.get_version()) {
-// 	case OspfTypes::V2:
-// 	    break;
-// 	case OspfTypes::V3:
-// 	    XLOG_ASSERT(!net.is_valid());
-// 	    if (net.is_valid()) {
-// 		XLOG_WARNING("Net should be zero %s", cstring(net));
-// 		status = false;
-// 	    }
-// 	    return status;
-// 	    break;
-// 	}
+	switch(_ospf.get_version()) {
+	case OspfTypes::V2:
+	    break;
+	case OspfTypes::V3:
+//  	    if (!net.is_valid())
+		return true;
+	    break;
+	}
     }
 
     typename Trie<A, InternalRouteEntry<A> >::iterator i;
@@ -140,18 +147,14 @@ RoutingTable<A>::replace_entry(OspfTypes::AreaID area, IPNet<A> net,
 
     if (rt.get_destination_type() == OspfTypes::Router) {
 	status = _adv.replace_entry(area, rt.get_router_id(), rt);
-// 	switch(_ospf.get_version()) {
-// 	case OspfTypes::V2:
-// 	    break;
-// 	case OspfTypes::V3:
-// 	    XLOG_ASSERT(!net.is_valid());
-// 	    if (net.is_valid()) {
-// 		XLOG_WARNING("Net should be zero %s", cstring(net));
-// 		status = false;
-// 	    }
-// 	    return status;
-// 	    break;
-// 	}
+ 	switch(_ospf.get_version()) {
+ 	case OspfTypes::V2:
+ 	    break;
+ 	case OspfTypes::V3:
+//  	    if (!net.is_valid())
+		return true;
+ 	    break;
+ 	}
     }
 
     typename Trie<A, InternalRouteEntry<A> >::iterator i;
@@ -799,6 +802,9 @@ Adv<A>::lookup_entry(OspfTypes::AreaID area, uint32_t adv,
 	return false;
     rt = j->second;
 
+    debug_msg("Found area %s adv %s\n", pr_id(area).c_str(),
+	   pr_id(adv).c_str());
+
     return true;
 }
 
