@@ -1,19 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/ospf/xrl_io.cc,v 1.50 2008/07/23 05:11:10 pavlin Exp $"
+#ident "$XORP: xorp/ospf/xrl_io.cc,v 1.54 2009/01/05 18:31:01 jtc Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -83,7 +88,7 @@ template <>
 bool
 XrlIO<IPv4>::send(const string& interface, const string& vif,
 		  IPv4 dst, IPv4 src,
-		  uint8_t* data, uint32_t len)
+		  int ttl, uint8_t* data, uint32_t len)
 {
     bool success;
 
@@ -105,8 +110,8 @@ XrlIO<IPv4>::send(const string& interface, const string& vif,
 	src,
 	dst,
 	get_ip_protocol_number(),
-	-1,		// XXX: let the FEA set it
-	-1,		// XXX: let the FEA set it
+	ttl,
+	-1,		// XXX: let the FEA set TOS
 	get_ip_router_alert(),
 	true,		// ip_internet_control
 	payload,
@@ -119,7 +124,7 @@ template <>
 bool
 XrlIO<IPv6>::send(const string& interface, const string& vif,
 		  IPv6 dst, IPv6 src,
-		  uint8_t* data, uint32_t len)
+		  int ttl, uint8_t* data, uint32_t len)
 {
     bool success;
 
@@ -142,8 +147,8 @@ XrlIO<IPv6>::send(const string& interface, const string& vif,
 	src,
 	dst,
 	get_ip_protocol_number(),
-	dst.is_multicast() ? 1 : -1,		// XXX: let the FEA set it
-	-1,					// XXX: let the FEA set it
+	dst.is_multicast() ? 1 : ttl,
+	-1,					// XXX: let the FEA set TOS
 	get_ip_router_alert(),
 	true,					// ip_internet_control
 	ext_headers_type,
@@ -1060,7 +1065,7 @@ template<>
 bool
 XrlQueue<IPv6>::sendit_spec(Queued& q, const char *protocol)
 {
-    bool sent;
+    bool sent = false;
     bool unicast = true;
     bool multicast = false;
 
