@@ -1,18 +1,23 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/pim/pim_bsr.cc,v 1.53 2008/07/23 05:11:12 pavlin Exp $"
+#ident "$XORP: xorp/pim/pim_bsr.cc,v 1.58 2009/01/05 18:31:02 jtc Exp $"
 
 
 //
@@ -361,14 +366,30 @@ PimBsr::unicast_pim_bootstrap(PimVif *pim_vif, const IPvX& nbr_addr) const
 BsrZone *
 PimBsr::add_config_bsr_zone(const BsrZone& bsr_zone, string& error_msg)
 {
+    BsrZone* old_bsr_zone = NULL;
+
     if (! can_add_config_bsr_zone(bsr_zone, error_msg))
 	return (NULL);
-    
-    BsrZone *new_bsr_zone = new BsrZone(*this, bsr_zone);
-    new_bsr_zone->set_config_bsr_zone(true);
-    _config_bsr_zone_list.push_back(new_bsr_zone);
-    
-    return (new_bsr_zone);
+
+    //
+    // Add or update the BSR zone
+    //
+    old_bsr_zone = find_config_bsr_zone(bsr_zone.zone_id());
+    if (old_bsr_zone == NULL) {
+	// Add a new BSR zone
+	BsrZone *new_bsr_zone = new BsrZone(*this, bsr_zone);
+	new_bsr_zone->set_config_bsr_zone(true);
+	_config_bsr_zone_list.push_back(new_bsr_zone);
+	return (new_bsr_zone);
+    }
+
+    //
+    // Update the existing BSR zone
+    //
+    if (old_bsr_zone->update_config_bsr_zone(bsr_zone, error_msg) != XORP_OK)
+	return (NULL);
+
+    return (old_bsr_zone);
 }
 
 // Return: a pointer to the BSR zone if it exists and there is no
@@ -1006,12 +1027,6 @@ PimBsr::can_add_config_bsr_zone(const BsrZone& bsr_zone,
 	}
     }
     
-    if (find_config_bsr_zone(bsr_zone.zone_id()) != NULL) {
-	error_msg = c_format("already have scope zone %s",
-			     cstring(bsr_zone.zone_id()));
-	return (false);
-    }
-    
     //
     // Check for consistency
     //
@@ -1019,12 +1034,26 @@ PimBsr::can_add_config_bsr_zone(const BsrZone& bsr_zone,
 	 iter_zone != _config_bsr_zone_list.end();
 	 ++iter_zone) {
 	BsrZone *config_bsr_zone = *iter_zone;
-	
+
+	//
+	// Check whether this is an update for an existing zone.
+	//
+	if (bsr_zone.i_am_candidate_bsr()) {
+	    if (config_bsr_zone->zone_id() == bsr_zone.zone_id()) {
+		// Same zone so it will be updated
+		continue;
+	    }
+	}
+
 	//
 	// Check that zones do not overlap
 	//
 	if (config_bsr_zone->i_am_candidate_bsr()
 	    && bsr_zone.i_am_candidate_bsr()) {
+	    if (config_bsr_zone->zone_id() == bsr_zone.zone_id()) {
+		// Same zone so it will be updated
+		continue;
+	    }
 	    if (config_bsr_zone->zone_id().is_overlap(bsr_zone.zone_id())) {
 		error_msg = c_format("overlapping zones %s and %s",
 				     cstring(config_bsr_zone->zone_id()),
@@ -1454,6 +1483,47 @@ BsrZone::set_test_bsr_zone(bool v)
     }
 }
 
+int
+BsrZone::update_config_bsr_zone(const BsrZone& new_bsr_zone, string& error_msg)
+{
+    bool changed = false;
+
+    UNUSED(error_msg);
+
+    //
+    // Update all relevant fields
+    //
+    if ((i_am_candidate_bsr() != new_bsr_zone.i_am_candidate_bsr())
+	|| (_my_vif_index != new_bsr_zone.my_vif_index())
+	|| (my_bsr_addr() != new_bsr_zone.my_bsr_addr())
+	|| (my_bsr_priority() != new_bsr_zone.my_bsr_priority())) {
+	set_i_am_candidate_bsr(new_bsr_zone.i_am_candidate_bsr(),
+			       new_bsr_zone.my_vif_index(),
+			       new_bsr_zone.my_bsr_addr(),
+			       new_bsr_zone.my_bsr_priority());
+	set_bsr_addr(my_bsr_addr());
+	set_bsr_priority(my_bsr_priority());
+	changed = true;
+    }
+    if (is_my_bsr_addr_explicit() != new_bsr_zone.is_my_bsr_addr_explicit()) {
+	set_is_my_bsr_addr_explicit(new_bsr_zone.is_my_bsr_addr_explicit());
+	changed = true;
+    }
+    if (hash_mask_len() != new_bsr_zone.hash_mask_len()) {
+	_hash_mask_len = new_bsr_zone.hash_mask_len();
+	changed = true;
+    }
+
+    if (changed) {
+	//
+	// TODO: XXX: schedule a timer to send immediately the Bootstrap
+	// message with the updated information.
+	//
+    }
+
+    return (XORP_OK);
+}
+
 bool
 BsrZone::is_consistent(string& error_msg) const
 {
@@ -1856,8 +1926,8 @@ BsrZone::process_candidate_bsr(const BsrZone& cand_bsr_zone)
 	    set_bsr_zone_state(BsrZone::STATE_PENDING_BSR);
 	    set_i_am_candidate_bsr(config_bsr_zone->i_am_candidate_bsr(),
 				   config_bsr_zone->my_vif_index(),
-				   config_bsr_zone->bsr_addr(),
-				   config_bsr_zone->bsr_priority());
+				   config_bsr_zone->my_bsr_addr(),
+				   config_bsr_zone->my_bsr_priority());
 	    // Set BS Timer to BS Timeout
 	    TimeVal tv(PIM_BOOTSTRAP_BOOTSTRAP_TIMEOUT_DEFAULT, 0);
 	    _bsr_timer = pim_bsr().pim_node().eventloop().new_oneoff_after(
