@@ -1,18 +1,23 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/pim/pim_proto_bootstrap.cc,v 1.26 2008/07/23 05:11:15 pavlin Exp $"
+#ident "$XORP: xorp/pim/pim_proto_bootstrap.cc,v 1.29 2009/01/05 18:31:02 jtc Exp $"
 
 
 //
@@ -80,7 +85,13 @@ PimVif::pim_bootstrap_recv(PimNbr *pim_nbr, const IPvX& src,
     string	error_msg;
     string	dummy_error_msg;
     int		ret_value = XORP_ERROR;
-    
+
+    //
+    // XXX: Don't accept Bootstrap-related messages if the BSR is not running
+    //
+    if (! pim_node().pim_bsr().is_up())
+	return (XORP_ERROR);
+
     //
     // Parse the head of the message
     //
@@ -353,6 +364,12 @@ PimVif::pim_bootstrap_send(const IPvX& dst_addr, const BsrZone& bsr_zone,
     size_t avail_buffer_size = 0;
     IPvX src_addr = primary_addr();
 
+    //
+    // XXX: Don't transmit Bootstrap-related messages if the BSR is not running
+    //
+    if (! pim_node().pim_bsr().is_up())
+	return (XORP_ERROR);
+
     if (is_pim_register()) {
 	error_msg = c_format("Cannot send PIM Bootstrap message on "
 			     "PIM Register vif");
