@@ -1,38 +1,41 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/policy/backend/iv_exec.cc,v 1.17 2008/07/23 05:11:23 pavlin Exp $"
+#ident "$XORP: xorp/policy/backend/iv_exec.cc,v 1.29 2009/01/05 18:31:06 jtc Exp $"
 
+#include "policy/policy_module.h"
 #include "libxorp/xorp.h"
-
 #include "policy/common/policy_utils.hh"
 #include "policy/common/elem_null.hh"
 #include "policy/common/element.hh"
-
 #include "iv_exec.hh"
 
-
 IvExec::IvExec() : 
-	       _policies(NULL), _policy_count(0), _sman(NULL), _varrw(NULL),
-	       _finished(false), _fa(DEFAULT), _trash(NULL), _trashc(0),
-	       _trashs(2000), _os(NULL) 
+	       _policies(NULL), _policy_count(0), _stack_bottom(NULL), 
+	       _sman(NULL), _varrw(NULL), _finished(false), _fa(DEFAULT),
+	       _trash(NULL), _trashc(0), _trashs(2000), _profiler(NULL)
 {
     unsigned ss = 128;
     _trash = new Element*[_trashs];
 
-    _stack = new const Element*[ss];
+    _stack_bottom = _stack = new const Element*[ss];
     _stackptr = &_stack[0];
     _stackptr--;
     _stackend = &_stack[ss];
@@ -40,75 +43,102 @@ IvExec::IvExec() :
 
 IvExec::~IvExec()
 {
-    if (_policies)
-	delete [] _policies;
-    
+    delete [] _policies;
+
     clear_trash();
     delete [] _trash;
-    delete [] _stack;
+    delete [] _stack_bottom;
 }
 
 IvExec::FlowAction 
-IvExec::run(VarRW* varrw, ostream* os)
+IvExec::run(VarRW* varrw)
 {
-    _varrw = varrw;
-    _os = os;
+    _varrw     = varrw;
+    _did_trace = false;
+
+    _os.clear();
 
     XLOG_ASSERT(_policies);
     XLOG_ASSERT(_sman);
     XLOG_ASSERT(_varrw);
 
-    if (!_varrw->trace_allowed())
-	_os = NULL;
-
     FlowAction ret = DEFAULT;
 
+    // clear stack
+    _stackptr = _stack = _stack_bottom;
+    _stackptr--;
+
     // execute all policies
-    for(unsigned i = 0; i < _policy_count; ++i) {
+    for (unsigned i = 0; i < _policy_count; ++i) {
 	FlowAction fa = runPolicy(*_policies[i]);
 
 	// if a policy rejected/accepted a route then terminate.
-	if(fa != DEFAULT) {
+	if (fa != DEFAULT) {
 	    ret = fa;
 	    break;
 	}
     }
-    
-    if(_os)
-	*_os << "Outcome of whole filter: " << fa2str(ret) << endl;
+
+    if (_did_trace)
+	_os << "Outcome of whole filter: " << fa2str(ret) << endl;
 
     // important because varrw may hold pointers to trash elements
     _varrw->sync();
+
     clear_trash();
 
-    
     return ret;
 }
 
 IvExec::FlowAction 
 IvExec::runPolicy(PolicyInstr& pi)
 {
-    TermInstr** terms = pi.terms();
-    int termc = pi.termc();
-   
+    TermInstr** terms  = pi.terms();
+    int termc	       = pi.termc();
     FlowAction outcome = DEFAULT;
 
-    if(_os)
-	*_os << "Running policy: " << pi.name() << endl;
+    // create a "stack frame".  We do this just so we can "clear" the stack
+    // frame when running terms and keep the asserts.  In reality if we get rid
+    // of asserts and clearing of stack, we're fine, but we gotta be bug free.
+    //  -sorbo
+    const Element** stack_bottom = _stack;
+    const Element** stack_ptr    = _stackptr;
+    _stack = _stackptr + 1;
+    XLOG_ASSERT(_stack < _stackend && _stack >= _stack_bottom);
+
+    _do_trace = pi.trace();
+
+    _varrw->enable_trace(_do_trace);
+
+    if (_do_trace)
+	_did_trace = true;
+
+    if (_do_trace)
+	_os << "Running policy: " << pi.name() << endl;
+
+    // execute terms sequentially
+    _ctr_flow = Next::TERM;
 
     // run all terms
-    for(int i = 0; i < termc ; ++i) {
+    for (int i = 0; i < termc ; ++i) {
 	FlowAction fa = runTerm(*terms[i]);
 
 	// if term accepted/rejected route, then terminate.
-	if(fa != DEFAULT) {
+	if (fa != DEFAULT) {
 	    outcome = fa;
 	    break;
-	}    
+	}
+
+	if (_ctr_flow == Next::POLICY)
+	    break;
     }
 
-    if(_os)
-	*_os << "Outcome of policy: " << fa2str(outcome) << endl;
+    if (_do_trace)
+	_os << "Outcome of policy: " << fa2str(outcome) << endl;
+
+    // restore stack frame
+    _stack    = stack_bottom;
+    _stackptr = stack_ptr;
 
     return outcome;
 }
@@ -122,26 +152,32 @@ IvExec::runTerm(TermInstr& ti)
     _fa = DEFAULT;
 
     // clear stack
-    _stackptr = &_stack[0];
+    _stackptr = _stack;
     _stackptr--;
 
     int instrc = ti.instrc();
     Instruction** instr = ti.instructions();
 
-    if(_os)
-	*_os << "Running term: " << ti.name() << endl;
+    if (_do_trace)
+	_os << "Running term: " << ti.name() << endl;
 
     // run all instructions
-    for(int i = 0; i < instrc; ++i) {
+    for (int i = 0; i < instrc; ++i) {
+	if (_profiler)
+	    _profiler->start();
+
 	(instr[i])->accept(*this);
 
+	if (_profiler)
+	    _profiler->stop();
+
 	// a flow action occured [accept/reject/default -- exit]
-	if(_finished)
+	if (_finished)
 	    break;
     }
 
-    if(_os)
-	*_os << "Outcome of term: " << fa2str(_fa) << endl;
+    if (_do_trace)
+	_os << "Outcome of term: " << fa2str(_fa) << endl;
 
     return _fa;
 }
@@ -155,8 +191,8 @@ IvExec::visit(Push& p)
     XLOG_ASSERT(_stackptr < _stackend);
     *_stackptr = &e;
     
-    if(_os)
-	*_os << "PUSH " << e.type() << " " << e.str() << endl;
+    if(_do_trace)
+	_os << "PUSH " << e.type() << " " << e.str() << endl;
 }
 
 void 
@@ -170,8 +206,8 @@ IvExec::visit(PushSet& ps)
     XLOG_ASSERT(_stackptr < _stackend);
     *_stackptr = &s;
 
-    if(_os)
-	*_os << "PUSH_SET " << s.type() << " " << name
+    if(_do_trace)
+	_os << "PUSH_SET " << s.type() << " " << name
 	     << ": " << s.str() << endl;
 }
 
@@ -187,8 +223,8 @@ IvExec::visit(OnFalseExit& /* x */)
 	// but maybe it is a ElemNull... in which case its a NOP
 	const Element* e = *_stackptr;
 	if(e->hash() == ElemNull::_hash) {
-	    if(_os)
-		*_os << "GOT NULL ON TOP OF STACK, GOING TO NEXT TERM" << endl;
+	    if(_do_trace)
+		_os << "GOT NULL ON TOP OF STACK, GOING TO NEXT TERM" << endl;
 	    _finished = true;
 	    return;
         }
@@ -211,8 +247,8 @@ IvExec::visit(OnFalseExit& /* x */)
     if(!t->val())
 	_finished = true;
 
-    if(_os)
-	*_os << "ONFALSE_EXIT: " << t->str() << endl;
+    if(_do_trace)
+	_os << "ONFALSE_EXIT: " << t->str() << endl;
 }
 
 void 
@@ -220,8 +256,9 @@ IvExec::visit(Load& l)
 {
     const Element& x = _varrw->read_trace(l.var());
 
-    if(_os)
-	*_os << "LOAD " << l.var() << ": " << x.str() << endl;
+    if (_do_trace)
+	_os << "LOAD " << l.var() << ": " << x.str() << endl;
+
     // varrw owns element [do not trash]
     _stackptr++;
     XLOG_ASSERT(_stackptr < _stackend);
@@ -236,11 +273,12 @@ IvExec::visit(Store& s)
 
     const Element* arg = *_stackptr;
     _stackptr--;
-    XLOG_ASSERT( _stackptr >= (_stack-1));
+    XLOG_ASSERT(_stackptr >= (_stack-1));
+
+    if (arg->hash() == ElemNull::_hash) {
+	if (_do_trace)
+	    _os << "STORE NULL [treated as NOP]" << endl;
 
-    if(arg->hash() == ElemNull::_hash) {
-	if(_os)
-	    *_os << "STORE NULL [treated as NOP]" << endl;
 	return;
     }
 
@@ -248,9 +286,10 @@ IvExec::visit(Store& s)
     // if it had to be trashed, it would have been trashed on creation, so do
     // NOT trash now. And yes, it likely is an element we do not have to
     // trash anyway.
-    _varrw->write_trace(s.var(),*arg);
-    if(_os)
-	*_os << "STORE " << s.var() << ": " << arg->str() << endl;
+    _varrw->write_trace(s.var(), *arg);
+
+    if (_do_trace)
+	_os << "STORE " << s.var() << ": " << arg->str() << endl;
 }
 
 void 
@@ -259,10 +298,31 @@ IvExec::visit(Accept& /* a */)
     // ok we like the route, so exit all execution
     _finished = true;
     _fa = ACCEPT;
-    if(_os)
-	*_os << "ACCEPT" << endl;
+    if(_do_trace)
+	_os << "ACCEPT" << endl;
 }
-    
+
+void
+IvExec::visit(Next& next)
+{
+    _finished = true;
+    _ctr_flow = next.flow();
+
+    if (_do_trace) {
+	_os << "NEXT ";
+
+	switch (_ctr_flow) {
+	case Next::TERM:
+	    _os << "TERM";
+	    break;
+
+	case Next::POLICY:
+	    _os << "POLICY";
+	    break;
+	}
+    }
+}
+
 void 
 IvExec::visit(Reject& /* r */)
 {
@@ -270,8 +330,8 @@ IvExec::visit(Reject& /* r */)
     _finished = true;
     _fa = REJ;
 
-    if(_os)
-	*_os << "REJECT" << endl;
+    if(_do_trace)
+	_os << "REJECT" << endl;
 }
 
 void
@@ -279,28 +339,31 @@ IvExec::visit(NaryInstr& nary)
 {
     unsigned arity = nary.op().arity();
 
-    XLOG_ASSERT(_stackptr-arity+1 >= _stack);
+    XLOG_ASSERT((_stackptr - arity + 1) >= _stack);
 
     // execute the operation
-    Element* r = _disp.run(nary.op(), arity, _stackptr-arity+1);
+    Element* r = _disp.run(nary.op(), arity, _stackptr - arity + 1);
     if (arity)
 	_stackptr -= arity -1;
     else
 	_stackptr++;
 
-    // trash the result for deletion on completion
-    _trash[_trashc] = r;
-    _trashc++;
+    // trash the result.
+    // XXX only if it's a new element.
+    if (r->refcount() == 1) {
+	_trash[_trashc] = r;
+	_trashc++;
 
-    XLOG_ASSERT(_trashc < _trashs);
+	XLOG_ASSERT(_trashc < _trashs);
+    }
 
     // store result on stack
     XLOG_ASSERT(_stackptr < _stackend && _stackptr >= _stack);
     *_stackptr = r;
 
     // output trace
-    if(_os)
-	*_os << nary.op().str() << endl;
+    if (_do_trace)
+	_os << nary.op().str() << endl;
 }
 
 void
@@ -361,3 +424,64 @@ IvExec::set_set_manager(SetManager* sman)
 {
     _sman = sman;
 }
+
+void
+IvExec::set_profiler(PolicyProfiler* pp)
+{
+    _profiler = pp;
+}
+
+string
+IvExec::tracelog()
+{
+    return _os.str();
+}
+
+void
+IvExec::visit(Subr& sub)
+{
+    SUBR::iterator i = _subr->find(sub.target());
+    XLOG_ASSERT(i != _subr->end());
+
+    PolicyInstr* policy = i->second;
+
+    if (_do_trace)
+	_os << "POLICY " << policy->name() << endl;
+
+    FlowAction old_fa = _fa;
+    bool old_finished = _finished;
+
+    FlowAction fa = runPolicy(*policy);
+
+    _fa       = old_fa;
+    _finished = old_finished;
+
+    bool result = true;
+
+    switch (fa) {
+    case DEFAULT:
+    case ACCEPT:
+	result = true;
+	break;
+
+    case REJ:
+	result = false;
+	break;
+    }
+
+    Element* e = new ElemBool(result);
+
+    _stackptr++;
+    XLOG_ASSERT(_stackptr < _stackend);
+    *_stackptr = e;
+
+    _trash[_trashc] = e;
+    _trashc++;
+    XLOG_ASSERT(_trashc < _trashs);
+}
+
+void
+IvExec::set_subr(SUBR* subr)
+{
+    _subr = subr;
+}
