@@ -1,56 +1,74 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/policy/backend/policytags.cc,v 1.14 2008/07/23 05:11:24 pavlin Exp $"
+#ident "$XORP: xorp/policy/backend/policytags.cc,v 1.17 2009/01/05 18:31:06 jtc Exp $"
+
+#include <sstream>
 
 #include "libxorp/xorp.h"
-
 #include "policytags.hh"
 #include "policy/common/elem_set.hh"
 #include "libxipc/xrl_atom.hh"
-#include <sstream>
 
-PolicyTags::PolicyTags()
+PolicyTags::PolicyTags() : _tag(0)
 {
 }
 
-PolicyTags::PolicyTags(const XrlAtomList& alist)
+PolicyTags::PolicyTags(const XrlAtomList& alist) : _tag(0)
 {
+    // first is always tag
+    XLOG_ASSERT(alist.size() > 0);
+
     // go through all the atoms in the list
-    for(unsigned i = 0; i < alist.size(); ++i) {
+    for (unsigned i = 0; i < alist.size(); ++i) {
 	const XrlAtom& atom = alist.get(i);
 
 	// only support u32's
-	if(atom.type() != xrlatom_uint32)
+	if (atom.type() != xrlatom_uint32)
 	    xorp_throw(PolicyTagsError, "XrlAtomList does not contain uint32's");
 
+	uint32_t val = atom.uint32();
+
+	if (i == 0) {
+	    _tag = val;
+	    continue;
+	}
+
 	// it's good, insert it
-	_tags.insert(atom.uint32());    
+	_tags.insert(val);
     }
 }
 
-PolicyTags::PolicyTags(const Element& element)
+void
+PolicyTags::set_ptags(const Element& element)
 {
     // we only support set elements
     const ElemSetU32* es = dynamic_cast<const ElemSetU32*>(&element);
-    if(!es)
+    if (!es)
 	xorp_throw(PolicyTagsError, 
 		   string("Element is not a set: ") + element.type());
 
+    _tags.clear();
+
     // go through all the set elements.
-    for(ElemSetU32::const_iterator i = es->begin(); i != es->end(); ++i) {
+    for (ElemSetU32::const_iterator i = es->begin(); i != es->end(); ++i) {
 	const ElemU32& x = *i;
 
 	// all ElemSet elements are represented as string, so convert and
@@ -81,8 +99,7 @@ PolicyTags::str() const
 bool
 PolicyTags::operator==(const PolicyTags& rhs) const
 {
-    // just check if the set of tags is equal.
-    return _tags == rhs._tags;
+    return (_tags == rhs._tags) && (_tag == rhs._tag);
 }
 
 XrlAtomList
@@ -90,6 +107,8 @@ PolicyTags::xrl_atomlist() const
 {
     XrlAtomList alist;
 
+    alist.append(XrlAtom(_tag));
+
     for(Set::const_iterator i = _tags.begin(); i != _tags.end(); ++i) {
 	uint32_t tag = *i;
 
@@ -110,6 +129,20 @@ PolicyTags::element() const
     return s;
 }
 
+Element*
+PolicyTags::element_tag() const
+{
+    return new ElemU32(_tag);
+}
+
+void
+PolicyTags::set_tag(const Element& e)
+{
+    uint32_t val = dynamic_cast<const ElemU32&>(e).val();
+
+    _tag = val;
+}
+
 void
 PolicyTags::insert(const PolicyTags& ptags)
 {
@@ -133,3 +166,9 @@ PolicyTags::contains_atleast_one(const PolicyTags& tags) const
 	
     return !output.empty();
 }
+
+void
+PolicyTags::insert(uint32_t tag)
+{
+    _tags.insert(tag);
+}
