@@ -1,45 +1,46 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/policy/backend/single_varrw.cc,v 1.18 2008/07/23 05:11:24 pavlin Exp $"
+#ident "$XORP: xorp/policy/backend/single_varrw.cc,v 1.21 2009/01/05 18:31:06 jtc Exp $"
 
 #include "policy/policy_module.h"
-
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
-
 #include "policy/common/elem_null.hh"
-
 #include "single_varrw.hh"
 
-
-SingleVarRW::SingleVarRW() : _trashc(0), _did_first_read(false) 
+SingleVarRW::SingleVarRW() : _trashc(0), _did_first_read(false), _pt(NULL)
 {
     memset(&_elems, 0, sizeof(_elems));
     memset(&_modified, 0, sizeof(_modified));
 }
 
-
-SingleVarRW::~SingleVarRW() {
+SingleVarRW::~SingleVarRW()
+{
     for (unsigned i = 0; i < _trashc; i++)
         delete _trash[i];
 }
 
 const Element&
-SingleVarRW::read(const Id& id) {
-
+SingleVarRW::read(const Id& id)
+{
     // Maybe there was a write before a read for this variable, if so, just
     // return the value... no need to bother the client.
     const Element* e = _elems[id];
@@ -76,7 +77,8 @@ SingleVarRW::read(const Id& id) {
 }
 
 void
-SingleVarRW::write(const Id& id, const Element& e) {
+SingleVarRW::write(const Id& id, const Element& e)
+{
     // XXX no paranoid checks on what we write
 
     _elems[id] = &e;
@@ -84,7 +86,8 @@ SingleVarRW::write(const Id& id, const Element& e) {
 }
 
 void
-SingleVarRW::sync() {
+SingleVarRW::sync()
+{
     bool first = true;
 
     // it's faster doing it this way rather than STL set if VAR_MAX is small...
@@ -92,18 +95,31 @@ SingleVarRW::sync() {
 	if (!_modified[i])
 	    continue;
 
+	const Element* e = _elems[i];
+	XLOG_ASSERT(e);
+	_modified[i] = false;
+
 	if (first) {
 	    // alert derived class we are committing
 	    start_write();
 	    first = false;
-	}    
-	const Element* e = _elems[i];
+	}
 
-	XLOG_ASSERT(e);
-	single_write(i,*e);
-	_modified[i] = false;
+	if (_pt) {
+	    switch (i) {
+	    case VAR_POLICYTAGS:
+		_pt->set_ptags(*e);
+		continue;
+
+	    case VAR_TAG:
+		_pt->set_tag(*e);
+		continue;
+	    }
+	}
+
+	single_write(i, *e);
     }
-    
+
     // done commiting [so the derived class may sync]
     end_write();
 
@@ -117,8 +133,8 @@ SingleVarRW::sync() {
 }
 
 void
-SingleVarRW::initialize(const Id& id, Element* e) {
-
+SingleVarRW::initialize(const Id& id, Element* e)
+{
     // check if we already have a value for a variable.
     // if so, do nothing.
     //
@@ -144,3 +160,12 @@ SingleVarRW::initialize(const Id& id, Element* e) {
     _trash[_trashc] = e;
     _trashc++;
 }
+
+void
+SingleVarRW::initialize(PolicyTags& pt)
+{
+    _pt = &pt;
+
+    initialize(VAR_POLICYTAGS, _pt->element());
+    initialize(VAR_TAG, _pt->element_tag());
+}
