@@ -1,34 +1,40 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/policy/code_generator.cc,v 1.15 2008/07/23 05:11:18 pavlin Exp $"
+#ident "$XORP: xorp/policy/code_generator.cc,v 1.21 2009/01/05 18:31:03 jtc Exp $"
 
 #include "policy_module.h"
-
 #include "libxorp/xorp.h"
+#include "code_generator.hh"
+#include "policy_map.hh"
 
-#include "code_generator.hh"    
-
-
-CodeGenerator::CodeGenerator(const VarMap& varmap) : _varmap(varmap)
+CodeGenerator::CodeGenerator(const VarMap& varmap, PolicyMap& pmap) 
+	: _varmap(varmap), _pmap(pmap), _subr(false)
 {
 }
 
-CodeGenerator::CodeGenerator(const string& proto, 
+CodeGenerator::CodeGenerator(const string& proto,
 			     const filter::Filter& filter,
-			     const VarMap& varmap) : _varmap(varmap)
+			     const VarMap& varmap,
+			     PolicyMap& pmap)
+	: _varmap(varmap), _pmap(pmap), _subr(false)
 {
     _protocol = proto;
     _code.set_target_protocol(proto);
@@ -36,8 +42,9 @@ CodeGenerator::CodeGenerator(const string& proto,
 }
 
 // constructor for import policies
-CodeGenerator::CodeGenerator(const string& proto, const VarMap& varmap) : 
-				_varmap(varmap)
+CodeGenerator::CodeGenerator(const string& proto, const VarMap& varmap,
+			     PolicyMap& pmap) : _varmap(varmap), _pmap(pmap),
+			     _subr(false)
 {
     _protocol = proto;
     _code.set_target_protocol(proto);
@@ -51,20 +58,23 @@ CodeGenerator::~CodeGenerator()
 const Element* 
 CodeGenerator::visit_policy(PolicyStatement& policy)
 {
-    _os << "POLICY_START " << policy.name() << endl;
-
     PolicyStatement::TermContainer& terms = policy.terms();
 
     // go through all the terms
-    for(PolicyStatement::TermContainer::iterator i = terms.begin(); 
-	i != terms.end(); ++i) {
+    for (PolicyStatement::TermContainer::iterator i = terms.begin(); 
+	 i != terms.end(); ++i) {
 	
 	(i->second)->accept(*this);
     }	    
 
-    _os << "POLICY_END\n";
+    ostringstream oss;
+
+    oss << "POLICY_START " << policy.name() << endl;
+    oss << _os.str();
+    oss << "POLICY_END" << endl;
+
+    _code.set_code(oss.str());
 
-    _code.set_code(_os.str());
     return NULL;
 }
 
@@ -136,6 +146,15 @@ CodeGenerator::visit(NodeAssign& node)
 
     VarRW::Id id = _varmap.var2id(protocol(), node.varid());
 
+    // XXX backend should have specialized operators for performance reasons.
+    // For now we just expand expressions such as "a += b" into "a = a + b" in
+    // the frontend. 
+    //  -sorbo
+    if (node.mod()) {
+	_os << "LOAD " << id << endl;
+	_os << node.mod()->str() << endl;
+    }
+
     _os << "STORE " << id << endl;
     return NULL;
 }
@@ -218,3 +237,50 @@ CodeGenerator::protocol()
 {
     return _protocol;
 }
+
+const Element*
+CodeGenerator::visit(NodeNext& next)
+{
+    _os << "NEXT ";
+
+    switch (next.flow()) {
+    case NodeNext::POLICY:
+	_os << "POLICY";
+	break;
+
+    case NodeNext::TERM:
+	_os << "TERM";
+	break;
+    }
+
+    _os << endl;
+
+    return NULL;
+}
+
+const Element*
+CodeGenerator::visit(NodeSubr& node)
+{
+    string policy       = node.policy();
+    PolicyStatement& ps = _pmap.find(policy);
+
+    string tmp = _os.str();
+    _os.clear();
+    _os.str("");
+
+    bool subr = _subr;
+    _subr = true;
+    visit(ps);
+    _subr = subr;
+
+    string code = _code.code();
+    _code.add_subr(policy, code);
+
+    _os.clear();
+    _os.str("");
+
+    _os << tmp;
+    _os << "POLICY " << policy << endl;
+
+    return NULL;
+}
