@@ -1,19 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/policy/common/dispatcher.cc,v 1.14 2008/07/23 05:11:25 pavlin Exp $"
+#ident "$XORP: xorp/policy/common/dispatcher.cc,v 1.19 2009/01/05 18:31:06 jtc Exp $"
 
 #include "libxorp/xorp.h"
 
@@ -26,12 +31,8 @@
 #include "element.hh"
 #include "register_operations.hh"
 
-
 // init static members
 Dispatcher::Value Dispatcher::_map[32768];
-RegisterOperations Dispatcher::_regops;
-unsigned Dispatcher::_ophash = 1;
-unsigned Dispatcher::_elemhash = 1;
 
 Dispatcher::Dispatcher()
 {
@@ -58,9 +59,8 @@ Dispatcher::run(const Oper& op, unsigned argc, const Element** argv) const
 	    return new ElemNull();
 
 	key |= h << (5*(argc-i));
-	    
     }
-    
+
     // check for constructor
     if (argc == 2 && typeid(op) == typeid(OpCtr)) {
 	string arg1type = argv[1]->type();
@@ -69,12 +69,12 @@ Dispatcher::run(const Oper& op, unsigned argc, const Element** argv) const
 	    xorp_throw(OpNotFound,
 		       "First argument of ctr must be txt type, but is: " 
 		       + arg1type);
-	
+
 	const ElemStr& es = dynamic_cast<const ElemStr&>(*argv[1]);
 
 	return operations::ctr(es, *(argv[0]));
     }
-    
+
     // find function
     Value funct = _map[key];
 
@@ -121,38 +121,3 @@ Dispatcher::run(const BinOper& op,
 
     return run(op, 2, argv);
 }
-
-void
-Dispatcher::assign_op_hash(const Oper& op)
-{
-    if (op.hash())
-	return;
-
-    // XXX
-    if (_ophash > 31) {
-	xorp_throw(PolicyException,
-		   "Too many operations for dispatcher---find a better hashing mechanism\n");
-    }
-
-    op.set_hash(_ophash);
-    _ophash++;
-}
-
-void
-Dispatcher::assign_elem_hash(Element& e)
-{
-    if (e.hash())
-	return;
-
-    if (_elemhash == ElemNull::_hash)
-	_elemhash++;
-
-    // XXX
-    if (_elemhash > 31) {
-	xorp_throw(PolicyException, 
-		   "Too many elems for dispatcher---find a better hashing mechanism\n");
-    }
-
-    e.set_hash(_elemhash);
-    _elemhash++;
-}
