@@ -1,19 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/policy/common/elem_set.cc,v 1.18 2008/07/23 05:11:25 pavlin Exp $"
+#ident "$XORP: xorp/policy/common/elem_set.cc,v 1.22 2009/01/05 18:31:06 jtc Exp $"
 
 #include "libxorp/xorp.h"
 
@@ -24,12 +29,12 @@
 
 
 template <class T>
-ElemSetAny<T>::ElemSetAny(const Set& val) : _val(val) 
+ElemSetAny<T>::ElemSetAny(const Set& val) : ElemSet(_hash), _val(val) 
 {
 }
 
 template <class T>
-ElemSetAny<T>::ElemSetAny(const char* c_str)
+ElemSetAny<T>::ElemSetAny(const char* c_str) : ElemSet(_hash)
 {
     if (!c_str)
 	return;
@@ -45,7 +50,7 @@ ElemSetAny<T>::ElemSetAny(const char* c_str)
 }
 
 template <class T>
-ElemSetAny<T>::ElemSetAny()
+ElemSetAny<T>::ElemSetAny() : ElemSet(_hash)
 {
 }
 
@@ -226,6 +231,13 @@ ElemSetAny<T>::erase(const ElemSetAny<T>& rhs)
     }
 }
 
+template <class T>
+void
+ElemSetAny<T>::erase(const ElemSet& rhs)
+{
+    erase(dynamic_cast<const ElemSetAny<T>&>(rhs));
+}
+
 template <class T>
 typename ElemSetAny<T>::iterator
 ElemSetAny<T>::begin()
@@ -254,20 +266,6 @@ ElemSetAny<T>::end() const
     return _val.end();
 }
 
-template <class T>
-void
-ElemSetAny<T>::set_hash(const Hash& x)
-{
-    _hash = x;
-}
-
-template <class T>
-Element::Hash
-ElemSetAny<T>::hash() const
-{
-    return _hash;
-}
-
 template <class T>
 const char*
 ElemSetAny<T>::type() const
@@ -277,21 +275,21 @@ ElemSetAny<T>::type() const
 
 // define the various sets
 template <> const char* ElemSetU32::id = "set_u32";
-template <> Element::Hash ElemSetU32::_hash = 0;
+template <> Element::Hash ElemSetU32::_hash = HASH_ELEM_SET_U32;
 template class ElemSetAny<ElemU32>;
 
 template <> const char* ElemSetCom32::id = "set_com32";
-template <> Element::Hash ElemSetCom32::_hash = 0;
+template <> Element::Hash ElemSetCom32::_hash = HASH_ELEM_SET_COM32;
 template class ElemSetAny<ElemCom32>;
 
 template <> const char* ElemSetIPv4Net::id = "set_ipv4net";
-template <> Element::Hash ElemSetIPv4Net::_hash = 0;
+template <> Element::Hash ElemSetIPv4Net::_hash = HASH_ELEM_SET_IPV4NET;
 template class ElemSetAny<ElemIPv4Net>;
 
 template <> const char* ElemSetIPv6Net::id = "set_ipv6net";
-template <> Element::Hash ElemSetIPv6Net::_hash = 0;
+template <> Element::Hash ElemSetIPv6Net::_hash = HASH_ELEM_SET_IPV6NET;
 template class ElemSetAny<ElemIPv6Net>;
 
 template <> const char* ElemSetStr::id = "set_str";
-template <> Element::Hash ElemSetStr::_hash = 0;
+template <> Element::Hash ElemSetStr::_hash = HASH_ELEM_SET_STR;
 template class ElemSetAny<ElemStr>;
