@@ -1,19 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/policy/common/register_operations.cc,v 1.23 2008/07/23 05:11:27 pavlin Exp $"
+#ident "$XORP: xorp/policy/common/register_operations.cc,v 1.31 2009/01/05 18:31:06 jtc Exp $"
 
 #include "libxorp/xorp.h"
 
@@ -63,17 +68,43 @@ Element* name(const Left& x, const Right& y) \
     return new Result(x.val() op y.val()); \
 }
 
-DEFINE_BINOP(op_and,&&)
-DEFINE_BINOP(op_or,||)
-DEFINE_BINOP(op_xor,^)
+// We'd like partial template specialization for functions, but it's not
+// standard.  We special case when a bool is returned and optimize returning
+// true / false.  We return one of these global objects rather than creating a
+// new one each time.
+ElemBool _true(true);
+ElemBool _false(false);
 
-DEFINE_BINOP(op_eq,==)
-DEFINE_BINOP(op_ne,!=)
+Element*
+return_bool(bool x)
+{
+    Element* r = x ? &_true : &_false;
 
-DEFINE_BINOP(op_lt,<)
-DEFINE_BINOP(op_gt,>)
-DEFINE_BINOP(op_le,<=)
-DEFINE_BINOP(op_ge,>=)
+    XLOG_ASSERT(r->refcount() > 1);
+
+    return r;
+}
+
+#define DEFINE_BINOP_BOOL(name, op) \
+template<class Unused, class Left, class Right> \
+Element* name(const Left& x, const Right& y) \
+{ \
+    bool val = x.val() op y.val(); \
+    \
+    return return_bool(val); \
+}
+
+DEFINE_BINOP_BOOL(op_and, &&)
+DEFINE_BINOP_BOOL(op_or, ||)
+DEFINE_BINOP_BOOL(op_xor, ^)
+
+DEFINE_BINOP_BOOL(op_eq, ==)
+DEFINE_BINOP_BOOL(op_ne, !=)
+
+DEFINE_BINOP_BOOL(op_lt, <)
+DEFINE_BINOP_BOOL(op_gt, >)
+DEFINE_BINOP_BOOL(op_le, <=)
+DEFINE_BINOP_BOOL(op_ge, >=)
 
 DEFINE_BINOP(op_add,+)
 DEFINE_BINOP(op_sub,-)
@@ -129,7 +160,8 @@ op_lt_net(const Left& x, const Right& y)
     bool result;
 
     result = (y.val().contains(x.val()) && (y.val() != x.val()));
-    return new Result(result);
+
+    return return_bool(result);
 }
 
 template<class Result, class Left, class Right>
@@ -139,7 +171,8 @@ op_gt_net(const Left& x, const Right& y)
     bool result;
 
     result = (x.val().contains(y.val()) && (x.val() != y.val()));
-    return new Result(result);
+
+    return return_bool(result);
 }
 
 template<class Result, class Left, class Right>
@@ -149,7 +182,8 @@ op_le_net(const Left& x, const Right& y)
     bool result;
 
     result = y.val().contains(x.val());
-    return new Result(result);
+
+    return return_bool(result);
 }
 
 template<class Result, class Left, class Right>
@@ -159,7 +193,8 @@ op_ge_net(const Left& x, const Right& y)
     bool result;
 
     result = x.val().contains(y.val());
-    return new Result(result);
+
+    return return_bool(result);
 }
 
 // 2 template parameters because U can be T or ElemSetAny<T>
@@ -312,6 +347,46 @@ aspath_regex(const ElemASPath& left, const ElemSetStr& right)
     return new ElemBool(false);
 }
 
+template<class A>
+bool
+net_match(const ElemNet<A>& left, const ElemNet<A>& right)
+{
+    static Dispatcher d;
+    Element* r;
+
+    r = d.run(right.op(), left, right);
+
+    if (r == &_true)
+	return true;
+    else if (r == &_false)
+	return false;
+    else
+	abort();
+}
+
+template<class A>
+Element*
+net_set_match(const ElemNet<A>& left, const ElemSetAny<ElemNet<A> >& right)
+{
+    bool ret = false;
+
+    for (typename ElemSetAny<ElemNet<A> >::const_iterator i = right.begin();
+         i != right.end(); ++i) {
+
+	const ElemNet<A>& r = *i;
+
+	if (net_match(left, r)) {
+	    ret = true;
+	    break;
+	}
+    }
+
+    return return_bool(ret);
+}
+
+// register callbacks
+RegisterOperations _regops;
+
 } // namespace
 
 // XXX: hack to compile on 2.95.x [may not use &operation::op... with templates]
@@ -323,6 +398,10 @@ RegisterOperations::RegisterOperations()
 {
     Dispatcher disp;
 
+    // prevent these from being deleted
+    _true.ref();
+    _false.ref();
+
 #define ADD_BINOP(result,left,right,funct,oper)				\
 do {									\
 	disp.add<left,right,&funct<result,left,right> >(Op##oper());	\
@@ -366,6 +445,12 @@ do {									\
     ADD_BINOP(ElemBool,argl,argr,op_ge,Ge);				\
 } while (0)
 
+#define ADD_ALL_RELOP2(argl, argr)					\
+do {									\
+    ADD_RELOP2(argl, argr);						\
+    ADD_EQOP2(argl, argr);						\
+} while (0)
+
 // MATH OPERATORS
 #define ADD_MATHOP(arg)							\
 do {									\
@@ -478,18 +563,21 @@ do {                                                                    \
 
     // IPV4
     ADD_EQOP(ElemIPv4);
-    ADD_EQOP2(ElemIPv4,ElemIPv4Range);
+    ADD_EQOP2(ElemIPv4, ElemIPv4Range);
     ADD_RELOP(ElemIPv4);
-    ADD_RELOP2(ElemIPv4,ElemIPv4Range);
-//    ADD_SETBINOP(ElemIPv4);
-   
+    ADD_RELOP2(ElemIPv4, ElemIPv4Range);
+//  ADD_SETBINOP(ElemIPv4);
+    ADD_ALL_RELOP2(ElemIPv4NextHop, ElemIPv4Range);
+
     // IPV4NET
     ADD_EQOP(ElemIPv4Net);
-    ADD_EQOP2(ElemIPv4Net,ElemU32Range);
+    ADD_EQOP2(ElemIPv4Net, ElemU32Range);
     ADD_RELOP_SPECIALIZED(ElemIPv4Net, net);
-    ADD_RELOP2(ElemIPv4Net,ElemU32Range);
+    ADD_RELOP2(ElemIPv4Net, ElemU32Range);
     ADD_SETBINOP(ElemSetIPv4Net, ElemIPv4Net);
-   
+    // Special case set operation to make use of modifiers
+    disp.add<ElemIPv4Net, ElemSetIPv4Net, operations::net_set_match>(OpLe());
+
     // IPV6
     ADD_EQOP(ElemIPv6);
     ADD_EQOP2(ElemIPv6,ElemIPv6Range);
@@ -503,4 +591,6 @@ do {                                                                    \
     ADD_RELOP_SPECIALIZED(ElemIPv6Net, net);
     ADD_RELOP2(ElemIPv6Net,ElemU32Range);
     ADD_SETBINOP(ElemSetIPv6Net, ElemIPv6Net);
+    // Special case set operation to make use of modifiers
+    disp.add<ElemIPv6Net, ElemSetIPv6Net, operations::net_set_match>(OpLe());
 }
