@@ -1,19 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/policy/filter_manager.cc,v 1.18 2008/07/23 05:11:18 pavlin Exp $"
+#ident "$XORP: xorp/policy/filter_manager.cc,v 1.24 2009/01/05 18:31:03 jtc Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -21,7 +26,7 @@
 #include "policy_module.h"
 #include "libxorp/xorp.h"
 #include "libxorp/debug.h"
-
+#include "backend/policytags.hh"
 #include "filter_manager.hh"
 
 FilterManager::FilterManager(const CodeMap& imp, 
@@ -92,14 +97,13 @@ FilterManager::update_tagmap(const string& protocol)
 
     const TagSet* ts = (*i).second;
 
-    // convert tags to xrlatomlist
-    XrlAtomList al;
+    // convert tags to PolicyTags
+    PolicyTags pt;
 
-    for(TagSet::const_iterator iter = ts->begin();
-	iter != ts->end(); ++iter) {
-   
-        al.append(XrlAtom(*iter));	    
-    }
+    for (TagSet::const_iterator iter = ts->begin(); iter != ts->end(); ++iter)
+	pt.insert(*iter);	
+
+    XrlAtomList al = pt.xrl_atomlist();
 
     debug_msg("[POLICY] Updating tagmap proto: %s, tags: %s\n",
 	      protocol.c_str(),al.str().c_str());
@@ -119,7 +123,7 @@ FilterManager::policy_backend_cb(const XrlError& e)
 	debug_msg("[POLICY] XRL exception: %s\n", e.str().c_str());
 	error_msg = c_format("XRL policy_backend_cb() error: %s",
 			     e.str().c_str());
-	XLOG_ERROR(error_msg.c_str());
+	XLOG_ERROR("%s", error_msg.c_str());
 //	xorp_throw(FMException, error_msg); // XXX: what else can we do ?
     }	
 }
@@ -286,9 +290,13 @@ FilterManager::birth(const string& protocol)
     // it might be better...
     flush_updates_now();
 
-    // XXX: protocol was just born, so no need to push routes for itself... we
-    // may remove the pushes from the queue since they are delayed.
-    _push_queue.erase(protocol);
+    // We still push routes even though this protocol was just brought up.
+    // The reason being that some routes could have already been processed by
+    // the time the protocol was born and the filters configured.  Some
+    // protocols (like BGP) avoid unnecessary flapping by waiting until this
+    // push is received.  Perhaps there should be an explicit mechanism for the
+    // policy manager to signal protocols that it's done with configuration
+    // rather than overloading the meaning of route push.
 }
 
 void 
@@ -326,13 +334,13 @@ FilterManager::update_queue(const string& protocol,
     // if a process is dead, erase it from the queue if it is there, and then do
     // nothing.
     bool alive = _process_watch.alive(protocol);
-    if(!alive) {
+    if (!alive) {
 	debug_msg("[POLICY] clearing update queue for dead protocol: %s\n",
 		  protocol.c_str());
 
         ConfQueue::iterator i = queue.find(protocol);
 
-        if(i != queue.end())
+        if (i != queue.end())
 	   queue.erase(i);
 
         return;
@@ -341,9 +349,10 @@ FilterManager::update_queue(const string& protocol,
     // check if there is any code present for this protocol, if not reset
     // filter.
     CodeMap::const_iterator i = cm.find(protocol);
-    if(i == cm.end()) {
+    if (i == cm.end()) {
         // reset filter
         queue[protocol] = "";
+
         return;
     }
 
@@ -355,17 +364,29 @@ FilterManager::update_queue(const string& protocol,
     const set<string>& set_names = code->referenced_set_names();
 
     // expand the set names
-    for(set<string>::const_iterator iter = set_names.begin();
+    for (set<string>::const_iterator iter = set_names.begin();
         iter != set_names.end(); ++iter) {
 
 	const string& name = *iter;
         const Element& e = _sets.getSet(*iter);
-	
+
 	conf += string("SET ") + e.type() + " " + name + " \"";
         conf += e.str();
         conf += "\"\n";
     }
 
+    // add subroutines
+    const SUBR& subr = code->subr();
+
+    if (!subr.empty())
+	conf += "SUBR_START\n";
+
+    for (SUBR::const_iterator i = subr.begin(); i != subr.end(); ++i)
+	conf += i->second;
+
+    if (!subr.empty())
+	conf += "SUBR_END\n";
+
     // send it the complete configuration [code + sets]
     queue[protocol] = conf;
 }
