@@ -1,53 +1,162 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/policy/policy_list.cc,v 1.13 2008/07/23 05:11:19 pavlin Exp $"
+#ident "$XORP: xorp/policy/policy_list.cc,v 1.18 2009/01/05 18:31:03 jtc Exp $"
+
+#include <sstream>
 
 #include "policy_module.h"
 #include "libxorp/xorp.h"
-
 #include "policy_list.hh"
 #include "visitor_semantic.hh"
 #include "export_code_generator.hh"
 #include "source_match_code_generator.hh"
+#include "visitor_dep.hh"
+
+uint32_t PolicyList::_pe = 0;
 
 PolicyList::PolicyList(const string& p, PolicyType pt, 
-		       PolicyMap& pmap, SetMap& smap, VarMap& vmap) :
-	
-	_protocol(p), _type(pt), _pmap(pmap), _smap(smap), _varmap(vmap) 
+		       PolicyMap& pmap, SetMap& smap, VarMap& vmap,
+		       string mod)
+		       : _protocol(p), _type(pt), _pmap(pmap),
+		         _smap(smap), _varmap(vmap), _mod(mod),
+			 _mod_term(NULL), _mod_term_import(NULL),
+			 _mod_term_export(NULL)
 {
+    if (!_mod.empty()) {
+	    _mod_term_import = create_mod(Term::SOURCE);
+	    _mod_term_export = create_mod(Term::DEST);
+    }
 }
-    
+
+Term*
+PolicyList::create_mod(Term::BLOCKS block)
+{
+    // We add the modifier term at the beginning of each policy.  If it matches,
+    // we continue executing the policy, else we go to the next one.
+    Term* t  = new Term("__mod"); // XXX leak if exception is thrown below
+
+    ConfigNodeId nid(0, 0);
+
+    string statement = "not " + _mod;
+    t->set_block(block, nid, statement);
+    t->set_block_end(block);
+
+    statement = "next policy;";
+    t->set_block(Term::ACTION, nid, statement);
+    t->set_block_end(Term::ACTION);
+
+    return t;
+}
+
 PolicyList::~PolicyList()
 {
-    for(PolicyCodeList::iterator i = _policies.begin(); 
-	i != _policies.end(); ++i) {
+    for (PolicyCodeList::iterator i = _policies.begin(); 
+	 i != _policies.end(); ++i) {
 
 	PolicyCode& pc = *i;
 
-	_pmap.del_dependancy(pc.first,_protocol);
+	_pmap.del_dependency(pc.first,_protocol);
 
 	delete (*i).second;
     }
+
+    for (POLICIES::iterator i = _pe_policies.begin();
+         i != _pe_policies.end(); ++i)
+	_pmap.delete_policy(*i);
+
+    delete _mod_term_import;
+    delete _mod_term_export;
 }
 
 void 
 PolicyList::push_back(const string& policyname)
 {
-    _policies.push_back(PolicyCode(policyname,NULL));
-    _pmap.add_dependancy(policyname,_protocol);
+    if (!policyname.empty() && policyname.at(0) == '(') {
+	add_policy_expression(policyname);
+
+	return;
+    }
+
+    _policies.push_back(PolicyCode(policyname, NULL));
+    _pmap.add_dependency(policyname, _protocol);
+}
+
+void
+PolicyList::add_policy_expression(const string& exp)
+{
+    // We create an internal policy based on the expression, and execute that
+    // policy.
+    ostringstream oss;
+
+    oss << "PE_" << _pe++;
+
+    string name = oss.str();
+    _pmap.create(name, _smap);
+    _pe_policies.insert(name);
+
+    PolicyStatement& ps = _pmap.find(name);
+
+    // replace "string" into "policy string".  That is, execute policies as
+    // subroutines.
+    oss.str("");
+    int state = 0;
+
+    for (string::const_iterator i = exp.begin(); i != exp.end(); ++i) {
+	char x = *i;
+
+	if (isalnum(x)) {
+	    if (state == 0) {
+		oss << "policy ";
+		state = 1;
+	    }
+	} else
+	    state = 0;
+
+	oss << x;
+    }
+
+    string conf = oss.str();
+    ConfigNodeId order(1, 0);
+
+    // XXX how should this function with export policies?
+    Term* t = new Term("match");
+    t->set_block(_type == IMPORT ? Term::SOURCE : Term::DEST, order, conf);
+    t->set_block(Term::ACTION, order, "accept;");
+    ps.add_term(order, t);
+
+    // XXX handle next-policy too - how should it work?
+    t = new Term("nomatch");
+    t->set_block(Term::ACTION, order, "reject;");
+    ps.add_term(ConfigNodeId(2, 1), t);
+
+    ps.set_policy_end();
+
+    // update dependencies.
+    // XXX we shouldn't be doing this here.  We should have an encapsulated
+    // mechanism for adding "internal" policies and dealing with them correctly.
+    // It seems like a useful feature.
+    VisitorDep dep(_smap, _pmap);
+    ps.accept(dep);
+
+    push_back(name);
 }
 
 void 
@@ -76,27 +185,27 @@ void
 PolicyList::compile(Code::TargetSet& mod, uint32_t& tagstart)
 {
     // go throw all policies in the list
-    for(PolicyCodeList::iterator i = _policies.begin();
-	i != _policies.end(); ++i) {
+    for (PolicyCodeList::iterator i = _policies.begin();
+	 i != _policies.end(); ++i) {
 
 	PolicyCode& pc = *i;
 
 	// deal only with non compiled policies [i.e. policies without
 	// associated code].
-	if(pc.second) 
+	if (pc.second) 
 	    continue;
 
 	// find the policy statement and compile it.
 	PolicyStatement& ps = _pmap.find(pc.first);
     
 	switch(_type) {
-	    case IMPORT:
-		compile_import(i,ps,mod);
-		break;
+	case IMPORT:
+	    compile_import(i, ps, mod);
+	    break;
 	
-	    case EXPORT:
-		compile_export(i,ps,mod,tagstart);
-		break;
+	case EXPORT:
+	    compile_export(i, ps, mod, tagstart);
+	    break;
 	}
     }    
 }
@@ -141,15 +250,15 @@ void
 PolicyList::link_code(Code& ret)
 {
     // go through all the policies, and link the code
-    for(PolicyCodeList::iterator i = _policies.begin();
-	i != _policies.end(); ++i) {
+    for (PolicyCodeList::iterator i = _policies.begin();
+	 i != _policies.end(); ++i) {
 
 	CodeList* cl = (*i).second;
 
 	// because of target set in ret, only relevant code will be linked.
 	cl->link_code(ret);
 
-    }    
+    }
 }
 
 void 
@@ -190,9 +299,14 @@ PolicyList::semantic_check(PolicyStatement& ps,
     // [i.e. protocol and import/export pair].
     SemanticVarRW varrw(_varmap);
 
-    VisitorSemantic sem_check(varrw, _varmap, _smap,_protocol,type);
+    VisitorSemantic sem_check(varrw, _varmap, _smap, _pmap, _protocol, type);
 
     // exception will be thrown if all goes wrong.
+
+    // check modifier [a bit of a hack]
+    if (_mod_term)
+	_mod_term->accept(sem_check);
+
     ps.accept(sem_check);
 }
 
@@ -201,11 +315,18 @@ PolicyList::compile_import(PolicyCodeList::iterator& iter,
 			   PolicyStatement& ps,
 			   Code::TargetSet& modified_targets)
 {
+    _mod_term = _mod_term_import;
+
     // check the policy
-    semantic_check(ps,VisitorSemantic::IMPORT);
+    semantic_check(ps, VisitorSemantic::IMPORT);
 
     // generate the code
-    CodeGenerator cg(_protocol, _varmap);
+    CodeGenerator cg(_protocol, _varmap, _pmap);
+
+    // check modifier [a bit of a hack]
+    if (_mod_term)
+	_mod_term->accept(cg);
+
     ps.accept(cg);
 
     // make a copy of the code
@@ -233,15 +354,27 @@ PolicyList::compile_export(PolicyCodeList::iterator& iter, PolicyStatement& ps,
 			   Code::TargetSet& modified_targets, 
 			   uint32_t& tagstart)
 {
+    _mod_term = _mod_term_export;
+
     // make sure policy makes sense
     semantic_check(ps, VisitorSemantic::EXPORT);
 
     // generate source match code
-    SourceMatchCodeGenerator smcg(tagstart, _varmap);
+    SourceMatchCodeGenerator smcg(tagstart, _varmap, _pmap);
+    
+    // check modifier [a bit of a hack]
+    if (_mod_term)
+	_mod_term->accept(smcg);
+
     ps.accept(smcg);
 
     // generate Export code
-    ExportCodeGenerator ecg(_protocol, smcg.tags(), _varmap);
+    ExportCodeGenerator ecg(_protocol, smcg.tags(), _varmap, _pmap);
+
+    // check modifier [a bit of a hack]
+    if (_mod_term)
+	_mod_term->accept(ecg);
+
     ps.accept(ecg);
 
     // update the global tag start
