@@ -1,38 +1,42 @@
+// -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/policy/policy_statement.cc,v 1.15 2008/07/23 05:11:19 pavlin Exp $"
+#ident "$XORP: xorp/policy/policy_statement.cc,v 1.19 2009/01/05 18:31:03 jtc Exp $"
 
 #include "policy_module.h"
-
 #include "libxorp/xorp.h"
-
 #include "policy/common/policy_utils.hh"
-
 #include "policy_statement.hh"   
-
+#include "policy_map.hh"
 
 using namespace policy_utils;
 
-PolicyStatement::PolicyStatement(const string& name, SetMap& smap) : 
-    _name(name), _smap(smap)
+PolicyStatement::PolicyStatement(const string& name, SetMap& smap,
+				 PolicyMap& pmap) : 
+    _name(name), _smap(smap), _pmap(pmap)
 {
 }
 
 PolicyStatement::~PolicyStatement()
 {
-    del_dependancies();
+    del_dependencies();
     policy_utils::clear_map_container(_terms);
 
     list<pair<ConfigNodeId, Term*> >::iterator iter;
@@ -46,8 +50,8 @@ PolicyStatement::~PolicyStatement()
 void 
 PolicyStatement::add_term(const ConfigNodeId& order, Term* term)
 {
-    if((_terms.find(order) != _terms.end())
-       || (find_out_of_order_term(order) != _out_of_order_terms.end())) {
+    if ((_terms.find(order) != _terms.end())
+        || (find_out_of_order_term(order) != _out_of_order_terms.end())) {
 	xorp_throw(PolicyException,
 		   "Term already present in position: " + order.str());
     }
@@ -168,6 +172,37 @@ PolicyStatement::delete_term(const string& name)
 void
 PolicyStatement::set_policy_end()
 {
+    // The final action lives in an internally created term named __final.
+    // This is a single node (i.e., unlike terms, there can be only one
+    // instance) so the ordering is messed up and we need to fix it.  This call
+    // though should be received after all terms have been added so it is safe
+    // to assume that we simply add the term at the end of the list.
+    for (OOL::iterator i = _out_of_order_terms.begin();
+	 i != _out_of_order_terms.end(); ++i) {
+	 Term* t = i->second;
+
+	 if (t->name().compare("__final") != 0)
+	    continue;
+
+	// find last position and compute next one
+	XLOG_ASSERT(!_terms.empty());
+	TermContainer::iterator j = _terms.end();
+	j--;
+
+	ConfigNodeId order = j->first;
+	ConfigNodeId::UniqueNodeId nid = order.unique_node_id();
+	
+	// XXX we really need operator++ in ConfigNodeId
+	order = ConfigNodeId(nid+1, nid);
+
+	// insert
+	bool res = _terms.insert(order, t).second;
+	XLOG_ASSERT(res);
+
+	_out_of_order_terms.erase(i);
+	break;
+    }
+
     TermContainer::iterator i;
 
     for (i = _terms.begin(); i != _terms.end(); ++i) {
@@ -216,26 +251,31 @@ PolicyStatement::terms()
 }
 
 void 
-PolicyStatement::set_dependancy(const set<string>& sets)
+PolicyStatement::set_dependency(const DEPS& sets, const DEPS& policies)
 {
-    // replace all dependancies
+    // delete dependencies
+    del_dependencies();
 
-    // delete them all
-    del_dependancies();
+    // replace them
+    _sets     = sets;
+    _policies = policies;
 
-    // replace
-    _sets = sets;
+    // re-insert dependencies
+    for (DEPS::iterator i = _sets.begin(); i != _sets.end(); ++i)
+	_smap.add_dependency(*i, _name);
 
-    // re-insert dependancies
-    for(set<string>::iterator i = _sets.begin(); i != _sets.end(); ++i)
-	_smap.add_dependancy(*i,_name);
+    for (DEPS::iterator i = _policies.begin(); i != _policies.end(); ++i)
+	_pmap.add_dependency(*i, _name);
 }
 
 void 
-PolicyStatement::del_dependancies() {
-    // remove all dependancies
-    for(set<string>::iterator i = _sets.begin(); i != _sets.end(); ++i)
-	_smap.del_dependancy(*i,_name);
+PolicyStatement::del_dependencies() {
+    // remove all dependencies
+    for (DEPS::iterator i = _sets.begin(); i != _sets.end(); ++i)
+	_smap.del_dependency(*i, _name);
+
+    for (DEPS::iterator i = _policies.begin(); i != _policies.end(); ++i)
+	_pmap.del_dependency(*i, _name);
 
     _sets.clear();    
 }
