@@ -1,27 +1,29 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/policy/set_map.cc,v 1.12 2008/07/23 05:11:21 pavlin Exp $"
+#ident "$XORP: xorp/policy/set_map.cc,v 1.17 2009/01/05 18:31:03 jtc Exp $"
 
 #include "policy_module.h"
-
 #include "libxorp/xorp.h"
-
 #include "set_map.hh"
 
-
 const Element&
 SetMap::getSet(const string& name) const
 {
@@ -112,34 +114,25 @@ SetMap::delete_from_set(const string& type, const string& name,
 	xorp_throw(SetMapError, error_msg);
     }
 
-    // Get a string with the existing elements and delete the element
-    string elements = e->str();
-    set<string> s;
-    policy_utils::str_to_set(elements.c_str(), s);
-    s.erase(element);
+    // Delete element
+    ElemSet* del = dynamic_cast<ElemSet*>(_ef.create(type, element.c_str()));
+    dynamic_cast<ElemSet*>(e)->erase(*del);
+    delete del;
 
-    // Recreate a string with the remaining elements
-    elements = "";
-    set<string>::iterator i;
-    for (i = s.begin(); i != s.end(); ++i) {
-	if (! elements.empty())
-	    elements += ",";
-	elements += *i;
-    }
-
-    update_set(type, name, elements, modified);
+    // sort out dependencies
+    _deps.get_deps(name, modified);
 }
 
 void 
-SetMap::add_dependancy(const string& setname, const string& policyname)
+SetMap::add_dependency(const string& setname, const string& policyname)
 {
-    _deps.add_dependancy(setname,policyname);
+    _deps.add_dependency(setname,policyname);
 }
 
 void 
-SetMap::del_dependancy(const string& setname, const string& policyname)
+SetMap::del_dependency(const string& setname, const string& policyname)
 {
-    _deps.del_dependancy(setname,policyname);
+    _deps.del_dependency(setname,policyname);
 }
 
 string
@@ -149,8 +142,7 @@ SetMap::str() const
 
     string ret;
 
-
-    while(_deps.has_next(i)) {
+    while (_deps.has_next(i)) {
 	Dep::ObjPair op(_deps.next(i));
 
 	ret += op.name + ": ";
@@ -160,3 +152,18 @@ SetMap::str() const
 
     return ret;
 }
+
+void
+SetMap::sets_by_type(SETS& s, const string& type) const
+{
+    Dep::Map::const_iterator i = _deps.get_iterator();
+
+    while (_deps.has_next(i)) {
+	Dep::ObjPair op(_deps.next(i));
+
+	const Element* e = &op.object;
+
+	if (type.compare(e->type()) == 0)
+	    s.push_back(op.name);
+    }
+}
