@@ -1,19 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/policy/source_match_code_generator.cc,v 1.17 2008/07/23 05:11:21 pavlin Exp $"
+#ident "$XORP: xorp/policy/source_match_code_generator.cc,v 1.21 2009/01/05 18:31:03 jtc Exp $"
 
 #include "policy_module.h"
 #include "libxorp/xorp.h"
@@ -21,9 +26,9 @@
 #include "source_match_code_generator.hh"
 
 SourceMatchCodeGenerator::SourceMatchCodeGenerator(uint32_t tagstart,
-				const VarMap& varmap) : 
-				 CodeGenerator(varmap),
-				 _currtag(tagstart) 
+						   const VarMap& varmap,
+						   PolicyMap& pmap) 
+	   : CodeGenerator(varmap, pmap), _currtag(tagstart)
 {
 }
 
@@ -32,34 +37,39 @@ SourceMatchCodeGenerator::visit_policy(PolicyStatement& policy)
 {
     PolicyStatement::TermContainer& terms = policy.terms();
 
-    PolicyStatement::TermContainer::iterator i;
+    _policy = policy.name();
 
     // go through all the terms
-    for (i = terms.begin(); i != terms.end(); ++i) {
-	// reset code and sets
-	_os.str("");
-	_code.clear_referenced_set_names();
-
+    for (PolicyStatement::TermContainer::iterator i = terms.begin();
+         i != terms.end(); ++i) {
 	Term* term = i->second;
-	// make sure the source of the term has something [non empty source]
-	if (term->source_nodes().size()) {
-	    term->accept(*this);
-
-	    // term may be for a new target, so deal with that.
-	    addTerm(policy.name());
-	}
-	else {
-	    _tags.push_back(Taginfo(false, _currtag));
-	}
+	term->accept(*this);
     }
 
-    // mark the end for all policies
-    for (CodeMap::iterator i = _codes.begin();
- 	 i != _codes.end(); ++i) {
+    // Maybe we got called from subr.  If so, we're not a protocol statement
+    // (maybe one of our internal statements was though).  Reset it.
+    _protocol_statement = false;
 
+    if (_subr)
+	return NULL;
+
+    // mark the end for all policies
+    for (CodeMap::iterator i = _codes.begin(); i != _codes.end(); ++i) {
         Code* c = (*i).second;
         c->add_code("POLICY_END\n");
 
+	// for all subroutines too
+	for (SUBR::const_iterator j = c->subr().begin();
+	     j != c->subr().end();) {
+	    string x = j->second;
+
+	    x += "POLICY_END\n";
+
+	    string p = j->first;
+	    j++;
+	    c->add_subr(p, x);
+	}
+
         _codes_vect.push_back(c);
     }
 
@@ -67,7 +77,7 @@ SourceMatchCodeGenerator::visit_policy(PolicyStatement& policy)
 }
 
 void 
-SourceMatchCodeGenerator::addTerm(const string& pname)
+SourceMatchCodeGenerator::addTerm()
 {
     // copy the code for the term
     Code* term = new Code();
@@ -84,20 +94,56 @@ SourceMatchCodeGenerator::addTerm(const string& pname)
 	Code* existing = (*i).second;
 
 	// link "raw" code
-	term->set_code(_os.str());
+	string s = _os.str();
+
+	if (_subr) {
+	    SUBR::const_iterator j = existing->subr().find(_policy);
+	    XLOG_ASSERT(j != existing->subr().end());
+
+	    term->add_subr(_policy, (j->second) + s);
+	} else
+	    term->set_code(s);
+
 	*existing += *term;
 
 	delete term;
         return;
     }
 
+    XLOG_ASSERT(!_policy.empty());
+
     // code for a new target, need to create policy start header.
-    term->set_code("POLICY_START " + pname + "\n" + _os.str());
+    string s = "POLICY_START " + _policy + "\n" + _os.str();
+
+    if (_subr)
+	term->add_subr(_policy, s);
+    else
+	term->set_code(s);
+
     _codes[_protocol] = term;
 }
 
-const Element* 
+const Element*
 SourceMatchCodeGenerator::visit_term(Term& term)
+{
+    // reset code and sets
+    _os.str("");
+    _code.clear_referenced_set_names();
+
+    // make sure the source of the term has something [non empty source]
+    if (term.source_nodes().size()) {
+	do_term(term);
+
+	// term may be for a new target, so deal with that.
+	addTerm();
+    } else
+	_tags.push_back(Taginfo(false, _currtag));
+
+    return NULL;
+}
+
+void
+SourceMatchCodeGenerator::do_term(Term& term)
 {
     Term::Nodes& source = term.source_nodes();
 
@@ -146,9 +192,18 @@ SourceMatchCodeGenerator::visit_term(Term& term)
     // ignore any destination block [that is dealt with in the export code
     // generator]
 
+    // If subroutine, do actions.  Tags are set by caller.
+    if (_subr) {
+	for (Term::Nodes::iterator i = term.action_nodes().begin();
+	     i != term.action_nodes().end(); ++i) {
 
-    // ignore actions too...
+	    Node* n = i->second;
 
+	    n->accept(*this);
+	}
+
+	return;
+    }
 
     //
     // As an action, store policy tags...
@@ -185,8 +240,6 @@ SourceMatchCodeGenerator::visit_term(Term& term)
 
     // FIXME: integer overflow
     _currtag++;
-
-    return NULL;
 }
 
 const Element* 
