@@ -1,7 +1,7 @@
 #!/bin/sh
 
 #
-# $XORP: xorp/policy/test/test_policy.sh,v 1.2 2004/10/05 22:42:27 pavlin Exp $
+# $XORP: xorp/policy/test/test_policy.sh,v 1.3 2006/04/26 03:26:29 pavlin Exp $
 #
 
 TMPFILE=/tmp/xorp_policy_test.txt
@@ -19,8 +19,8 @@ fi
 
 echo Will run policy $1 with policy_var_map_file $2 with variables $3 checking for exit code $4
 
-if ! ./compilepolicy -s $1 -m $2 -o ${TMPFILE}
-then
+./compilepolicy -s $1 -m $2 -o ${TMPFILE}
+if [ $? -ne 0 ] ; then 
 exit 1
 fi
 
