@@ -1,8 +1,7 @@
-#ifndef lint
-static char const 
-yyrcsid[] = "$FreeBSD: src/usr.bin/yacc/skeleton.c,v 1.28.2.1 2001/07/19 05:46:39 peter Exp $";
-#endif
 #include <stdlib.h>
+#ifndef lint
+#ident "$FreeBSD: src/usr.bin/yacc/skeleton.c,v 1.37 2003/02/12 18:03:55 davidc Exp $"
+#endif
 #define YYBYACC 1
 #define YYMAJOR 1
 #define YYMINOR 9
@@ -112,7 +111,7 @@ typedef union {
 	char *c_str;
 	yy_statements* statements;
 } YYSTYPE;
-#line 116 "yacc.yy_compile_policy.cc"
+#line 119 "yacc.yy_compile_policy.cc"
 #define YYERRCODE 256
 #define YY_INT 257
 #define YY_STR 258
@@ -311,9 +310,9 @@ int
 yyparse (YYPARSE_PARAM_ARG)
     YYPARSE_PARAM_DECL
 {
-    register int yym, yyn, yystate;
+    int yym, yyn, yystate;
 #if YYDEBUG
-    register const char *yys;
+    const char *yys;
 
     if ((yys = getenv("YYDEBUG")))
     {
@@ -445,39 +444,39 @@ yyreduce:
     switch (yyn)
     {
 case 3:
-#line 89 "compilepolicy.y"
+#line 90 "compilepolicy.y"
 {
-				list<string> tmp;
+		list<string> tmp;
 
-				string proto = yyvsp[-2].c_str;
-				string pols = yyvsp[-1].c_str;
+		string proto = yyvsp[-2].c_str;
+		string pols = yyvsp[-1].c_str;
 
-				free(yyvsp[-2].c_str);
-				free(yyvsp[-1].c_str);
-			
-				policy_utils::str_to_list(pols,tmp);
+		free(yyvsp[-2].c_str);
+		free(yyvsp[-1].c_str);
+	
+		policy_utils::str_to_list(pols,tmp);
 
-				_yy_configuration.update_exports(proto,tmp);
-				}
+		_yy_configuration.update_exports(proto, tmp, "");
+	}
 break;
 case 4:
-#line 103 "compilepolicy.y"
+#line 105 "compilepolicy.y"
 {
-				list<string> tmp;
+		list<string> tmp;
 
-				string proto = yyvsp[-2].c_str;
-				string pols = yyvsp[-1].c_str;
+		string proto = yyvsp[-2].c_str;
+		string pols = yyvsp[-1].c_str;
 
-				free(yyvsp[-2].c_str);
-				free(yyvsp[-1].c_str);
-			
-				policy_utils::str_to_list(pols,tmp);
+		free(yyvsp[-2].c_str);
+		free(yyvsp[-1].c_str);
+	
+		policy_utils::str_to_list(pols,tmp);
 
-				_yy_configuration.update_imports(proto,tmp);
-				}
+		_yy_configuration.update_imports(proto, tmp, "");
+	}
 break;
 case 6:
-#line 120 "compilepolicy.y"
+#line 122 "compilepolicy.y"
 {
 	  	string type = yyvsp[-3].c_str;
 	  	string id = yyvsp[-2].c_str;
@@ -490,7 +489,7 @@ case 6:
 	  }
 break;
 case 7:
-#line 134 "compilepolicy.y"
+#line 136 "compilepolicy.y"
 {
 		string pname = yyvsp[-3].c_str;
 		free(yyvsp[-3].c_str);
@@ -520,7 +519,7 @@ case 7:
 	  }
 break;
 case 8:
-#line 166 "compilepolicy.y"
+#line 168 "compilepolicy.y"
 {
 	  	yy_tb* tb = new yy_tb;
 
@@ -534,7 +533,7 @@ case 8:
 	  }
 break;
 case 10:
-#line 182 "compilepolicy.y"
+#line 184 "compilepolicy.y"
 {
 		yy_statements* tmp = _yy_statements;
 		_yy_statements = NULL;
@@ -542,7 +541,7 @@ case 10:
 	}
 break;
 case 11:
-#line 191 "compilepolicy.y"
+#line 193 "compilepolicy.y"
 {
 		yy_statements* tmp = _yy_statements;
 		_yy_statements = NULL;
@@ -550,7 +549,7 @@ case 11:
 	}
 break;
 case 12:
-#line 200 "compilepolicy.y"
+#line 202 "compilepolicy.y"
 {
 		yy_statements* tmp = _yy_statements;
 		_yy_statements = NULL;
@@ -558,7 +557,7 @@ case 12:
 	}
 break;
 case 13:
-#line 208 "compilepolicy.y"
+#line 210 "compilepolicy.y"
 {
 	       
 	       	if (_yy_statements == NULL) {
@@ -573,7 +572,7 @@ case 13:
 		_yy_statements->push_back(statement);
 	       }
 break;
-#line 577 "yacc.yy_compile_policy.cc"
+#line 580 "yacc.yy_compile_policy.cc"
     }
     yyssp -= yym;
     yystate = *yyssp;
