@@ -1,41 +1,44 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/policy/visitor_semantic.cc,v 1.16 2008/07/23 05:11:21 pavlin Exp $"
+#ident "$XORP: xorp/policy/visitor_semantic.cc,v 1.24 2009/01/05 18:31:03 jtc Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#include "policy_module.h"
-
-#include "libxorp/xorp.h"
-#include "libxorp/debug.h"
-
 #include <sstream>
 
+#include "policy_module.h"
+#include "libxorp/xorp.h"
+#include "libxorp/debug.h"
 #include "policy/common/elem_null.hh"
-
 #include "visitor_semantic.hh"
-
+#include "policy_map.hh"
 
 VisitorSemantic::VisitorSemantic(SemanticVarRW& varrw, 
 				 VarMap& varmap,
 				 SetMap& setmap,
+				 PolicyMap& pmap,
 				 const string& protocol, 
 				 PolicyType ptype) : 
-		_varrw(varrw), _varmap(varmap), _setmap(setmap), 
+		_varrw(varrw), _varmap(varmap), _setmap(setmap), _pmap(pmap),
 		_protocol(protocol), _ptype(ptype) 
 {
 }
@@ -43,20 +46,27 @@ VisitorSemantic::VisitorSemantic(SemanticVarRW& varrw,
 const Element* 
 VisitorSemantic::visit(PolicyStatement& policy)
 {
-    PolicyStatement::TermContainer& terms = policy.terms();
-
-    PolicyStatement::TermContainer::iterator i;
-
-    // go through all terms
-    for(i = terms.begin(); i != terms.end(); ++i) {
-	(i->second)->accept(*this);
-    }
+    do_policy_statement(policy);
 
     // helps for garbage gollection in varrw
     _varrw.sync();
+
     return NULL;
 }
 
+void
+VisitorSemantic::do_policy_statement(PolicyStatement& policy)
+{
+    PolicyStatement::TermContainer& terms = policy.terms();
+    PolicyStatement::TermContainer::iterator i;
+
+    _reject = false;
+
+    // go through all terms
+    for (i = terms.begin(); i != terms.end(); ++i)
+	(i->second)->accept(*this);
+}
+
 const Element* 
 VisitorSemantic::visit(Term& term)
 {
@@ -127,11 +137,14 @@ VisitorSemantic::visit(NodeUn& node)
     // see if we may execute unary operation
     try {
 	res = _disp.run(node.op(),*arg);
-	_trash.insert(res);
+
+	if (res->refcount() == 1)
+	    _trash.insert(res);
+
 	return res;
     } 
     // we can't
-    catch(const PolicyException& e) {
+    catch (const PolicyException& e) {
 	ostringstream error;
 
 	error << "Invalid unop " << e.str() << " at line " << node.line();
@@ -146,15 +159,24 @@ VisitorSemantic::visit(NodeBin& node)
     const Element* left = node.left().accept(*this);
     const Element* right = node.right().accept(*this);
 
-    Element* res;
+    return do_bin(*left, *right, node.op(), node);
+}
+
+const Element*
+VisitorSemantic::do_bin(const Element& left, const Element& right,
+			const BinOper& op, const Node& node)
+{
     // see if we may execute bin operation.
     try {
-	res = _disp.run(node.op(),*left,*right);
-	_trash.insert(res);
+	Element* res = _disp.run(op, left, right);
+
+	if (res->refcount() == 1)
+	    _trash.insert(res);
+
 	return res;
-    } 
+    }
     // nope
-    catch(const PolicyException& e) {
+    catch (const PolicyException& e) {
         ostringstream error;
 
         error << "Invalid binop " << e.str() << " at line " << node.line();
@@ -172,8 +194,16 @@ VisitorSemantic::visit(NodeAssign& node)
     // try assignment
     try {
 	VarRW::Id id = _varmap.var2id(semantic_protocol(), node.varid());
+
+	// see if there's a modifier to the assignment
+	if (node.mod()) {
+	    const Element* left = &_varrw.read(id);
+
+	    rvalue = do_bin(*left, *rvalue, *node.mod(), node);
+	}
+
 	_varrw.write(id, *rvalue);
-    } catch(SemanticVarRW::var_error e) {
+    } catch (SemanticVarRW::var_error e) {
         ostringstream error;
 
         error << e.str() << " at line " << node.line();
@@ -230,6 +260,8 @@ VisitorSemantic::visit(NodeAccept& /* node */)
 const Element* 
 VisitorSemantic::visit(NodeReject& /* node */)
 {
+    _reject = true;
+
     return NULL;
 }
 
@@ -276,3 +308,29 @@ VisitorSemantic::semantic_protocol()
 {
     return _semantic_protocol;
 }
+
+const Element*
+VisitorSemantic::visit(NodeNext& /* node */)
+{
+    return NULL;
+}
+
+const Element*
+VisitorSemantic::visit(NodeSubr& node)
+{
+    // XXX check for recursion.
+    PolicyStatement& policy = _pmap.find(node.policy());
+
+    string proto = _protocol;
+    bool reject  = _reject;
+
+    do_policy_statement(policy);
+
+    Element* e = new ElemBool(!_reject);
+    _trash.insert(e);
+
+    change_protocol(proto);
+    _reject = reject;
+
+    return e;
+}
