@@ -1,19 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/policy/xrl_target.cc,v 1.16 2008/07/23 05:11:22 pavlin Exp $"
+#ident "$XORP: xorp/policy/xrl_target.cc,v 1.21 2009/01/05 18:31:03 jtc Exp $"
 
 #include "policy_module.h"
 
@@ -260,11 +265,12 @@ XrlPolicyTarget::policy_0_1_done_global_policy_conf()
 
 XrlCmdError 
 XrlPolicyTarget::policy_0_1_import(const string&   protocol,
-				   const string&   policies) 
+				   const string&   policies,
+				   const string&   modifier) 
 {
     try {
-	_policy_target.update_import(protocol,policies);
-    } catch(const PolicyException& e) {
+	_policy_target.update_import(protocol, policies, modifier);
+    } catch (const PolicyException& e) {
         return XrlCmdError::COMMAND_FAILED("Import of " + protocol + 
 					   " failed: " + e.str());
     }
@@ -274,11 +280,12 @@ XrlPolicyTarget::policy_0_1_import(const string&   protocol,
  
 XrlCmdError 
 XrlPolicyTarget::policy_0_1_export(const string&   protocol,
-				   const string&   policies)
+				   const string&   policies,
+				   const string&   modifier)
 {
     try {
-	_policy_target.update_export(protocol,policies);
-    } catch(const PolicyException& e) {
+	_policy_target.update_export(protocol, policies, modifier);
+    } catch (const PolicyException& e) {
         return XrlCmdError::COMMAND_FAILED("Export of " + protocol + 
 					   " failed: " + e.str());
     }
@@ -357,3 +364,31 @@ XrlPolicyTarget::finder_event_observer_0_1_xrl_target_death(
 
     return XrlCmdError::OKAY();
 }	
+
+XrlCmdError
+XrlPolicyTarget::cli_processor_0_1_process_command(
+				// Input values,
+				const string&   processor_name,
+				const string&   cli_term_name,
+				const uint32_t& cli_session_id,
+				const string&   command_name,
+				const string&   /* command_args */,
+				// Output values,
+				string&         ret_processor_name,
+				string&         ret_cli_term_name,
+				uint32_t&       ret_cli_session_id,
+				string&		ret_command_output)
+{
+    try {
+	ret_processor_name = processor_name;
+	ret_cli_term_name  = cli_term_name;
+	ret_cli_session_id = cli_session_id;
+
+	ret_command_output = _policy_target.cli_command(command_name);
+
+    } catch (const PolicyException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
