@@ -1,19 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/rib/rib_varrw.cc,v 1.13 2008/07/23 05:11:30 pavlin Exp $"
+#ident "$XORP: xorp/rib/rib_varrw.cc,v 1.17 2009/01/05 18:31:07 jtc Exp $"
 
 #include "rib_module.h"
 #include "libxorp/xorp.h"
@@ -31,7 +36,7 @@ template <class A>
 void
 RIBVarRW<A>::start_read()
 {
-    initialize(VAR_POLICYTAGS, _route.policytags().element());
+    initialize(_route.policytags());
 
     read_route_nexthop(_route);
 
@@ -48,8 +53,8 @@ RIBVarRW<IPv4>::read_route_nexthop(IPRouteEntry<IPv4>& route)
 {
     initialize(VAR_NETWORK4,
 	       _ef.create(ElemIPv4Net::id, route.net().str().c_str()));
-    initialize(VAR_NEXTHOP4,
-	       _ef.create(ElemIPv4::id, route.nexthop_addr().str().c_str()));
+    initialize(VAR_NEXTHOP4, _ef.create(ElemIPv4NextHop::id,
+               route.nexthop_addr().str().c_str()));
     initialize(VAR_NETWORK6, NULL);
     initialize(VAR_NEXTHOP6, NULL);
 }
@@ -60,8 +65,8 @@ RIBVarRW<IPv6>::read_route_nexthop(IPRouteEntry<IPv6>& route)
 {
     initialize(VAR_NETWORK6,
 	       _ef.create(ElemIPv6Net::id, route.net().str().c_str()));
-    initialize(VAR_NEXTHOP6,
-	       _ef.create(ElemIPv6::id, route.nexthop_addr().str().c_str()));
+    initialize(VAR_NEXTHOP6, _ef.create(ElemIPv6NextHop::id,
+               route.nexthop_addr().str().c_str()));
 
     initialize(VAR_NETWORK4, NULL);
     initialize(VAR_NEXTHOP4, NULL);
@@ -69,12 +74,8 @@ RIBVarRW<IPv6>::read_route_nexthop(IPRouteEntry<IPv6>& route)
 
 template <class A>
 void
-RIBVarRW<A>::single_write(const Id& id, const Element& e)
+RIBVarRW<A>::single_write(const Id& /* id */, const Element& /* e */)
 {
-    if (id == VAR_POLICYTAGS) {
-	debug_msg("RIBVarRW writing policytags %s\n", e.str().c_str());
-	_route.set_policytags(e);
-    }
 }
 
 template <class A>
