@@ -1,19 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/rib/rt_tab_pol_conn.cc,v 1.13 2008/07/23 05:11:32 pavlin Exp $"
+#ident "$XORP: xorp/rib/rt_tab_pol_conn.cc,v 1.16 2009/01/05 18:31:07 jtc Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -195,15 +200,15 @@ PolicyConnectedTable<A>::push_routes()
     // go through original routes and refilter them
     for (typename RouteContainer::iterator i = _route_table.begin();
 	i != _route_table.end(); ++i) {
-   
+
 	const IPRouteEntry<A>* prev = i.payload();
 
 	// make a copy so filter may [possibly] modify it
-	IPRouteEntry<A>* copy = new IPRouteEntry<A>(*prev);
+	const IPRouteEntry<A>* orig = _parent->lookup_route(prev->net());
+	IPRouteEntry<A>* copy = new IPRouteEntry<A>(*orig);
 
 	do_filtering(*copy);
-	
-	
+
 	// only policytags may change
 	next->replace_policytags(*copy, prev->policytags(), this);
 
@@ -234,12 +239,6 @@ PolicyConnectedTable<A>::do_filtering(IPRouteEntry<A>& route)
 	debug_msg("[RIB] PolicyConnectedTable Filtering: %s\n",
 		  route.str().c_str());
 
-	//
-	// XXX: a hack to reset the policy tags in case the policies
-	// are removed.
-	//
-	route.set_policytags(PolicyTags());
-
 	RIBVarRW<A> varrw(route);
 
 	// only source match filtering!
