@@ -1,19 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/rip/rip_varrw.cc,v 1.12 2008/07/23 05:11:36 pavlin Exp $"
+#ident "$XORP: xorp/rip/rip_varrw.cc,v 1.16 2009/01/05 18:31:08 jtc Exp $"
 
 #include "rip_module.h"
 #include "libxorp/xorp.h"
@@ -36,6 +41,14 @@ RIPVarRW<A>::start_read()
     read_route_nexthop(_route);
 
     initialize(VAR_METRIC, new ElemU32(_route.cost()));
+
+    // XXX which tag wins?
+    ElemU32* e = dynamic_cast<ElemU32*>(_route.policytags().element_tag());
+    if (e->val())
+	_route.set_tag(e->val());
+
+    delete e;
+
     initialize(VAR_TAG, new ElemU32(_route.tag()));
 }
 
@@ -53,7 +66,7 @@ void
 RIPVarRW<A>::single_write(const Id& id, const Element& e)
 {
     if (id == VAR_POLICYTAGS) {
-	_route.set_policytags(e);
+	_route.policytags().set_ptags(e);
 	return;
     }
 
@@ -76,6 +89,7 @@ RIPVarRW<A>::single_write(const Id& id, const Element& e)
 	XLOG_ASSERT(u32 != NULL);
 
 	_route.set_tag(u32->val());
+	_route.policytags().set_tag(e);
 	return;
     }
 }
@@ -86,8 +100,8 @@ template <>
 bool
 RIPVarRW<IPv4>::write_nexthop(const Id& id, const Element& e)
 {
-    if (id == VAR_NEXTHOP4 && e.type() == ElemIPv4::id) {
-	const ElemIPv4* v4 = dynamic_cast<const ElemIPv4*>(&e);
+    if (id == VAR_NEXTHOP4 && e.type() == ElemIPv4NextHop::id) {
+	const ElemIPv4NextHop* v4 = dynamic_cast<const ElemIPv4NextHop*>(&e);
 
 	XLOG_ASSERT(v4 != NULL);
 
@@ -104,7 +118,7 @@ void
 RIPVarRW<IPv4>::read_route_nexthop(RouteEntry<IPv4>& route)
 {
     initialize(VAR_NETWORK4, new ElemIPv4Net(route.net()));
-    initialize(VAR_NEXTHOP4, new ElemIPv4(route.nexthop()));
+    initialize(VAR_NEXTHOP4, new ElemIPv4NextHop(route.nexthop()));
     
     initialize(VAR_NETWORK6, NULL);
     initialize(VAR_NEXTHOP6, NULL);
@@ -118,8 +132,8 @@ template <>
 bool
 RIPVarRW<IPv6>::write_nexthop(const Id& id, const Element& e)
 {
-    if (id == VAR_NEXTHOP6 && e.type() == ElemIPv6::id) {
-	const ElemIPv6* v6 = dynamic_cast<const ElemIPv6*>(&e);
+    if (id == VAR_NEXTHOP6 && e.type() == ElemIPv6NextHop::id) {
+	const ElemIPv6NextHop* v6 = dynamic_cast<const ElemIPv6NextHop*>(&e);
 
 	XLOG_ASSERT(v6 != NULL);
 
@@ -136,7 +150,7 @@ void
 RIPVarRW<IPv6>::read_route_nexthop(RouteEntry<IPv6>& route)
 {
     initialize(VAR_NETWORK6, new ElemIPv6Net(route.net()));
-    initialize(VAR_NEXTHOP6, new ElemIPv6(route.nexthop()));
+    initialize(VAR_NEXTHOP6, new ElemIPv6NextHop(route.nexthop()));
     
     initialize(VAR_NETWORK4, NULL);
     initialize(VAR_NEXTHOP4, NULL);
