@@ -1,20 +1,26 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/rip/xrl_port_manager.cc,v 1.31 2008/07/23 05:11:37 pavlin Exp $"
+#ident "$XORP: xorp/rip/xrl_port_manager.cc,v 1.34 2009/01/05 18:31:08 jtc Exp $"
 
 // #define DEBUG_LOGGING
+#include "rip_module.h"
 
 #include "libxorp/eventloop.hh"
 #include "libxorp/ipv4.hh"
@@ -395,6 +401,10 @@ XrlPortManager<A>::deliver_packet(const string& 		sockid,
     typename PortManagerBase<A>::PortList::iterator i;
     Port<A>* p = NULL;
 
+    XLOG_TRACE(packet_trace()._packets,
+		  "Packet on %s from interface %s vif %s %s/%u %u bytes\n",
+	      sockid.c_str(), ifname.c_str(), vifname.c_str(),
+	      src_addr.str().c_str(), src_port, XORP_UINT_CAST(pdata.size()));
     debug_msg("Packet on %s from interface %s vif %s %s/%u %u bytes\n",
 	      sockid.c_str(), ifname.c_str(), vifname.c_str(),
 	      src_addr.str().c_str(), src_port, XORP_UINT_CAST(pdata.size()));
@@ -402,6 +412,11 @@ XrlPortManager<A>::deliver_packet(const string& 		sockid,
     i = find_if(pl.begin(), pl.end(),
 		is_port_for<A>(&sockid, &ifname, &vifname, &src_addr, &_ifm));
     if (i == this->ports().end()) {
+
+	XLOG_TRACE(packet_trace()._packets,
+		  "Discarding packet %s/%u %u bytes\n",
+		  src_addr.str().c_str(), src_port,
+		  XORP_UINT_CAST(pdata.size()));
 	debug_msg("Discarding packet %s/%u %u bytes\n",
 		  src_addr.str().c_str(), src_port,
 		  XORP_UINT_CAST(pdata.size()));
