@@ -1,17 +1,22 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
 // #define DEBUG_LOGGING
 
@@ -51,7 +56,7 @@ decode_time_string(EventLoop& eventloop, const string& time_string,
     //
     TimeVal now;
     eventloop.current_time(now);
-    time_t local_time = now.secs();
+    time_t local_time = now.sec();
     const struct tm* local_tm = localtime(&local_time);
     memcpy(&tm, local_tm, sizeof(tm));
 
@@ -713,6 +718,22 @@ XrlRipTarget::rip_0_1_get_peer_counters(const string&	ifn,
 					   descs, vals, peer_last_active);
 }
 
+XrlCmdError
+XrlRipTarget::rip_0_1_trace(const string& tvar, const bool& enable)
+{
+    debug_msg("trace variable %s enable %s\n", tvar.c_str(),
+	      bool_c_str(enable));
+
+    if (tvar == "all") {
+	_ct->trace(enable);
+    } else {
+	return XrlCmdError::
+	    COMMAND_FAILED(c_format("Unknown variable %s", tvar.c_str()));
+    }
+
+    return XrlCmdError::OKAY();
+}
+
 
 XrlCmdError
 XrlRipTarget::socket4_user_0_1_recv_event(
