@@ -1,19 +1,24 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/rtrmgr/main_rtrmgr.cc,v 1.77 2008/07/23 05:11:41 pavlin Exp $"
+#ident "$XORP: xorp/rtrmgr/main_rtrmgr.cc,v 1.81 2009/01/05 18:31:10 jtc Exp $"
 
 #include "rtrmgr_module.h"
 
@@ -103,7 +108,7 @@ usage(const char* argv0)
     fprintf(stderr, "Usage: %s [options]\n", xorp_basename(argv0));
     fprintf(stderr, "Options:\n");
     fprintf(stderr, "  -a <allowed host> Host allowed by the finder\n");
-    fprintf(stderr, "  -d        Run as a UNIX daemon (detach from tty)\n");
+    fprintf(stderr, "  -d        Run in daemon mode in background\n");
     fprintf(stderr, "  -l <file> Log to file <file>\n");
     fprintf(stderr, "  -L <facility.priority> Log to syslog facility\n");
     fprintf(stderr, "  -n <allowed net>  Subnet allowed by the finder\n");
@@ -118,7 +123,6 @@ usage(const char* argv0)
     fprintf(stderr, "  -q <secs> Set forced quit period\n");
     fprintf(stderr, "  -t <dir>  Specify templates directory\n");
     fprintf(stderr, "  -x <dir>  Specify Xrl targets directory\n");
-    fprintf(stderr, "  -d        Daemon mode, run in the background\n");
 }
 
 static void
@@ -178,10 +182,24 @@ Rtrmgr::Rtrmgr(const string& template_dir,
       _quit_time(quit_time),
       _daemon_mode(daemon_mode),
       _ready(false),
-      _mct(NULL)
+      _mct(NULL),
+      _xrt(NULL)
 {
 }
 
+Rtrmgr::~Rtrmgr()
+{
+    // Delete allocated state
+    if (_mct != NULL) {
+	delete _mct;
+	_mct = NULL;
+    }
+    if (_xrt != NULL) {
+	delete _xrt;
+	_xrt = NULL;
+    }
+}
+
 int
 Rtrmgr::run()
 {
@@ -345,6 +363,7 @@ Rtrmgr::run()
 	    eventloop.run();
 	}
 	delete _mct;
+	_mct = NULL;
     } catch (const InitError& e) {
 	XLOG_ERROR("rtrmgr shutting down due to an init error: %s",
 		   e.why().c_str());
@@ -360,6 +379,12 @@ Rtrmgr::run()
 	eventloop.run();
     }
 
+    // Delete the XRL rtrmgr interface
+    if (_xrt != NULL) {
+	delete _xrt;
+	_xrt = NULL;
+    }
+
     // Delete the template tree
     delete tt;
 
