@@ -1,18 +1,23 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/rtrmgr/template_tree_node.cc,v 1.81 2008/07/23 05:11:44 pavlin Exp $"
+#ident "$XORP: xorp/rtrmgr/template_tree_node.cc,v 1.84 2009/01/05 18:31:10 jtc Exp $"
 
 #include "rtrmgr_module.h"
 
@@ -2063,7 +2068,7 @@ MacaddrTemplate::MacaddrTemplate(TemplateTree& template_tree,
 	return;
 
     try {
-	_default = new EtherMac(initializer.c_str());
+	_default = new Mac(initializer.c_str());
     } catch (InvalidString) {
 	error_msg = c_format("Bad MacAddr type value \"%s\".",
 			     initializer.c_str());
@@ -2099,7 +2104,7 @@ MacaddrTemplate::type_match(const string& s, string& error_msg) const
     }
 
     try {
-	EtherMac* mac = new EtherMac(tmp.c_str());
+	Mac* mac = new Mac(tmp.c_str());
 	delete mac;
     } catch (InvalidString) {
 	error_msg = "value must be an MAC address (six hex digits separated "
