@@ -1,18 +1,23 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2008 XORP, Inc.
+// Copyright (c) 2001-2009 XORP, Inc.
 //
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the "Software")
-// to deal in the Software without restriction, subject to the conditions
-// listed in the XORP LICENSE file. These conditions include: you must
-// preserve this copyright notice, and you cannot mention the copyright
-// holders in advertising related to the Software without their permission.
-// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
-// notice is a summary of the XORP LICENSE file; the license in that file is
-// legally binding.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, Version 2, June
+// 1991 as published by the Free Software Foundation. Redistribution
+// and/or modification of this program under the terms of any other
+// version of the GNU General Public License is not permitted.
+// 
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. For more details,
+// see the GNU General Public License, Version 2, a copy of which can be
+// found in the XORP LICENSE.gpl file.
+// 
+// XORP Inc, 2953 Bunker Hill Lane, Suite 204, Santa Clara, CA 95054, USA;
+// http://xorp.net
 
-#ident "$XORP: xorp/rtrmgr/xorpsh_main.cc,v 1.71 2008/07/23 05:11:46 pavlin Exp $"
+#ident "$XORP: xorp/rtrmgr/xorpsh_main.cc,v 1.76 2009/01/05 18:31:10 jtc Exp $"
 
 #include "rtrmgr_module.h"
 
@@ -92,8 +97,10 @@ wait_for_xrl_router_ready(EventLoop& eventloop, XrlRouter& xrl_router,
 	    return false;
 	    break;
 	}
-	if (xrl_router.connect_failed() && exit_on_error)
-	    return false;
+	if (exit_on_error) {
+	    if (!xrl_router.connected() || xrl_router.connect_failed())
+		return false;
+	}
     }
     return true;
 }
@@ -230,13 +237,6 @@ XorpShell::run(const string& commands, bool exit_on_error)
     }
 #endif // ! HOST_OS_WINDOWS
 
-    // Signal handlers so we can clean up when we're killed
-    signal(SIGTERM, signal_handler);
-    signal(SIGINT, signal_handler);
-#ifdef SIGPIPE
-    signal(SIGPIPE, signal_handler);
-#endif
-
     // Set the callback when the CLI exits (e.g., after Ctrl-D)
     _cli_node.set_cli_client_delete_callback(callback(exit_handler));
 
@@ -249,6 +249,10 @@ XorpShell::run(const string& commands, bool exit_on_error)
     }
     _is_connected_to_finder = true;
 
+    // Signal handlers so we can clean up when we're killed.
+    signal(SIGTERM, signal_handler);
+    signal(SIGINT, signal_handler);
+
 #ifdef HOST_OS_WINDOWS
     const uint32_t uid = 0;
 #else
@@ -776,12 +780,6 @@ signal_handler(int signal_value)
     case SIGINT:
 	// Ignore Ctrl-C: it is used by the CLI to interrupt a command.
 	break;
-#ifdef SIGPIPE
-    case SIGPIPE:
-	// Ignore SIGPIPE: it may be generated when executing commands
-	// specified on the command line.
-	break;
-#endif
     default:
 	// XXX: anything else we have intercepted will terminate us.
 	is_interrupted = true;
@@ -877,7 +875,7 @@ main(int argc, char *argv[])
 	case 'h':
 	    usage(argv[0]);
 	    display_defaults();
-	    errcode = 1;
+	    errcode = 0;
 	    goto cleanup;
 	}
     }
